@@ -86,76 +86,41 @@ char *cu_strdup(std::string_view sv)
 	return out;
 }
 
-ssize_t cu_utf8_to_mb(cpid_t cpid, const char *src,
-    char *dst, size_t len)
+static const char *cu_resolve_cpid(cpid_t cpid)
 {
-	size_t in_len;
-	size_t out_len;
-	iconv_t conv_id;
-	char temp_charset[256];
-	
-	auto charset = cpid_to_cset(cpid);
-	if (charset == nullptr)
-		return -1;
-	sprintf(temp_charset, "%s//IGNORE",
-		replace_iconv_charset(charset));
-	conv_id = iconv_open(temp_charset, "UTF-8");
-	if (conv_id == (iconv_t)-1)
-		return -1;
-	auto pin = deconst(src);
-	auto pout = dst;
-	in_len = strlen(src) + 1;
-	memset(dst, 0, len);
-	out_len = len;
-	iconv(conv_id, &pin, &in_len, &pout, &len);
-	iconv_close(conv_id);
-	return out_len - len;
+	if (cpid == CP_OEMCP) {
+		/* only valid when invoked under rop environment */
+		auto e = emsmdb_interface_get_emsmdb_info();
+		if (e == nullptr)
+			return nullptr;
+		cpid = e->cpid;
+	}
+	return cpid_to_cset(cpid);
 }
 
-ssize_t cu_mb_to_utf8(cpid_t cpid, const char *src,
-    char *dst, size_t len)
+std::string cu_cvt_str(std::string_view sv, cpid_t cpid, bool to_utf8) try
 {
-	size_t in_len;
-	size_t out_len;
-	iconv_t conv_id;
-
-	cpid_cstr_compatible(cpid);
-	auto charset = cpid_to_cset(cpid);
-	if (charset == nullptr)
-		return -1;
-	conv_id = iconv_open("UTF-8//IGNORE",
-		replace_iconv_charset(charset));
-	if (conv_id == (iconv_t)-1)
-		return -1;
-	auto pin = deconst(src);
-	auto pout = dst;
-	in_len = strlen(src) + 1;
-	memset(dst, 0, len);
-	out_len = len;
-	iconv(conv_id, &pin, &in_len, &pout, &len);	
-	iconv_close(conv_id);
-	return out_len - len;
+	auto cset = cu_resolve_cpid(cpid);
+	if (cset == nullptr) {
+		errno = EINVAL;
+		return {};
+	}
+	return iconvtext(sv, to_utf8 ? cset : "UTF-8", to_utf8 ? "UTF-8" : cset);
+} catch (const std::bad_alloc &) {
+	errno = ENOMEM;
+	return {};
 }
 
-static char *common_util_dup_mb_to_utf8(cpid_t cpid, const char *src)
+char *cu_mb_to_utf8_dup(cpid_t cpid, std::string_view sv)
 {
-	cpid_cstr_compatible(cpid);
-	auto len = mb_to_utf8_len(src);
-	auto pdst = cu_alloc<char>(len);
-	if (pdst == nullptr)
-		return NULL;
-	return cu_mb_to_utf8(cpid, src, pdst, len) >= 0 ? pdst : nullptr;
+	auto cvt = cu_cvt_str(std::move(sv), cpid, true);
+	return errno == 0 ? cu_strdup(cvt) : nullptr;
 }
 
-/* only for being invoked under rop environment */
-ssize_t common_util_convert_string(bool to_utf8, const char *src,
-    char *dst, size_t len)
+char *cu_utf8_to_mb_dup(cpid_t cpid, std::string_view sv)
 {
-	auto pinfo = emsmdb_interface_get_emsmdb_info();
-	if (pinfo == nullptr)
-		return -1;
-	return to_utf8 ? cu_mb_to_utf8(pinfo->cpid, src, dst, len) :
-	       cu_utf8_to_mb(pinfo->cpid, src, dst, len);
+	auto cvt = cu_cvt_str(std::move(sv), cpid, false);
+	return errno == 0 ? cu_strdup(cvt) : nullptr;
 }
 
 void common_util_obfuscate_data(uint8_t *data, uint32_t size)
@@ -223,7 +188,7 @@ std::string cu_username_to_oneoff_s(const char *username, const char *dispname)
 	out.resize(ep.m_offset);
 	return out;
 } catch (const std::bad_alloc &) {
-	mlog(LV_ERR, "E-2254: ENOMEM");
+	mlog(LV_ERR, "%s: ENOMEM", __func__);
 	return {};
 }
 
@@ -293,7 +258,7 @@ std::string cu_fid_to_entryid_s(const logon_object &logon, uint64_t folder_id) t
 		return {};
 	return out;
 } catch (const std::bad_alloc &) {
-	mlog(LV_ERR, "E-2246: ENOMEM");
+	mlog(LV_ERR, "%s: ENOMEM", __func__);
 	return {};
 }
 
@@ -341,7 +306,7 @@ std::string cu_fid_to_sk_s(const logon_object &logon, uint64_t folder_id) try
 		return {};
 	return out;
 } catch (const std::bad_alloc &) {
-	mlog(LV_ERR, "E-2248: ENOMEM");
+	mlog(LV_ERR, "%s: ENOMEM", __func__);
 	return {};
 }
 
@@ -404,7 +369,7 @@ std::string cu_mid_to_entryid_s(const logon_object &logon, uint64_t folder_id,
 	out.resize(ep.m_offset);
 	return out;
 } catch (const std::bad_alloc &) {
-	mlog(LV_ERR, "E-2252: ENOMEM");
+	mlog(LV_ERR, "%s: ENOMEM", __func__);
 	return {};
 }
 
@@ -448,7 +413,7 @@ std::string cu_mid_to_sk_s(const logon_object &logon, uint64_t message_id) try
 		return {};
 	return out;
 } catch (const std::bad_alloc &) {
-	mlog(LV_ERR, "E-2253: ENOMEM");
+	mlog(LV_ERR, "%s: ENOMEM", __func__);
 	return {};
 }
 
@@ -529,7 +494,7 @@ std::string cu_xid_to_bin_s(const XID &xid) try
 	out.resize(ep.m_offset);
 	return out;
 } catch (const std::bad_alloc &) {
-	mlog(LV_ERR, "E-2255: ENOMEM");
+	mlog(LV_ERR, "%s: ENOMEM", __func__);
 	return {};
 }
 
@@ -752,10 +717,9 @@ BOOL common_util_retag_propvals(TPROPVAL_ARRAY *parray,
 	return FALSE;
 }
 
-void common_util_reduce_proptags(PROPTAG_ARRAY *pproptags_minuend,
-	const PROPTAG_ARRAY *pproptags_subtractor)
+void cu_reduce_proptags(PROPTAG_ARRAY *pproptags_minuend, proptag_cspan pproptags_subtractor)
 {
-	for (const auto subtag : *pproptags_subtractor) {
+	for (const auto subtag : pproptags_subtractor) {
 		for (unsigned int i = 0; i < pproptags_minuend->count; ++i) {
 			if (subtag != pproptags_minuend->pproptag[i])
 				continue;
@@ -764,42 +728,41 @@ void common_util_reduce_proptags(PROPTAG_ARRAY *pproptags_minuend,
 				memmove(pproptags_minuend->pproptag + i,
 					pproptags_minuend->pproptag + i + 1,
 					(pproptags_minuend->count - i) *
-					sizeof(uint32_t));
+					sizeof(proptag_t));
 			break;
 		}
 	}
 }
 
-PROPTAG_ARRAY* common_util_trim_proptags(const PROPTAG_ARRAY *pproptags)
+PROPTAG_ARRAY *cu_trim_proptags(proptag_cspan pproptags)
 {
 	auto ptmp_proptags = cu_alloc<PROPTAG_ARRAY>();
 	if (ptmp_proptags == nullptr)
 		return NULL;
-	ptmp_proptags->pproptag = cu_alloc<uint32_t>(pproptags->count);
+	ptmp_proptags->pproptag = cu_alloc<proptag_t>(pproptags.size());
 	if (ptmp_proptags->pproptag == nullptr)
 		return NULL;
 	ptmp_proptags->count = 0;
-	for (unsigned int i = 0; i < pproptags->count; ++i) {
-		const auto tag = pproptags->pproptag[i];
+	for (unsigned int i = 0; i < pproptags.size(); ++i) {
+		const auto tag = pproptags[i];
 		if (PROP_TYPE(tag) == PT_OBJECT)
 			continue;
-		ptmp_proptags->pproptag[ptmp_proptags->count++] = tag;
+		ptmp_proptags->emplace_back(tag);
 	}
 	return ptmp_proptags;
 }
 
-BOOL common_util_propvals_to_row(
-	const TPROPVAL_ARRAY *ppropvals,
-	const PROPTAG_ARRAY *pcolumns, PROPERTY_ROW *prow)
+bool cu_propvals_to_row(const TPROPVAL_ARRAY *ppropvals, proptag_cspan pcolumns,
+    PROPERTY_ROW *prow)
 {
 	unsigned int i;
 	static constexpr uint32_t errcode = ecNotFound;
 	
-	for (i = 0; i < pcolumns->count; ++i)
-		if (!ppropvals->has(pcolumns->pproptag[i]))
+	for (i = 0; i < pcolumns.size(); ++i)
+		if (!ppropvals->has(pcolumns[i]))
 			break;	
-	prow->flag = i < pcolumns->count ? PROPERTY_ROW_FLAG_FLAGGED : PROPERTY_ROW_FLAG_NONE;
-	prow->pppropval = cu_alloc<void *>(pcolumns->count);
+	prow->flag = i < pcolumns.size() ? PROPERTY_ROW_FLAG_FLAGGED : PROPERTY_ROW_FLAG_NONE;
+	prow->pppropval = cu_alloc<void *>(pcolumns.size());
 	if (prow->pppropval == nullptr)
 		return FALSE;
 	/*
@@ -825,8 +788,8 @@ BOOL common_util_propvals_to_row(
 	 *   QueryRows throws MAPI_E_NO_SUPPORT
 	 *   [Gromox: this is where W-2270 would be logged]
 	 */
-	for (i=0; i<pcolumns->count; i++) {
-		const auto tag = pcolumns->pproptag[i];
+	for (i = 0; i < pcolumns.size(); i++) {
+		const auto tag = pcolumns[i];
 		auto val = ppropvals->getval(tag);
 		prow->pppropval[i] = deconst(val);
 		if (prow->flag != PROPERTY_ROW_FLAG_FLAGGED)
@@ -871,47 +834,38 @@ BOOL common_util_convert_unspecified(cpid_t cpid,
 	if (b_unicode) {
 		if (ptyped->type != PT_STRING8)
 			return TRUE;
-		auto tmp_len = mb_to_utf8_len(static_cast<char *>(ptyped->pvalue));
-		auto pvalue = common_util_alloc(tmp_len);
+		auto pvalue = cu_mb_to_utf8_dup(cpid, static_cast<char *>(ptyped->pvalue));
 		if (pvalue == nullptr)
 			return FALSE;
-		if (cu_mb_to_utf8(cpid, static_cast<char *>(ptyped->pvalue),
-		    static_cast<char *>(pvalue), tmp_len) < 0)
-			return FALSE;	
-		ptyped->pvalue = pvalue;
+		ptyped->pvalue = std::move(pvalue);
 		return TRUE;
 	}
 	if (ptyped->type != PT_UNICODE)
 		return TRUE;
-	auto tmp_len = utf8_to_mb_len(static_cast<char *>(ptyped->pvalue));
-	auto pvalue = common_util_alloc(tmp_len);
+	auto pvalue = cu_utf8_to_mb_dup(cpid, static_cast<char *>(ptyped->pvalue));
 	if (pvalue == nullptr)
 		return FALSE;
-	if (cu_utf8_to_mb(cpid, static_cast<char *>(ptyped->pvalue),
-	    static_cast<char *>(pvalue), tmp_len) < 0)
-		return FALSE;
-	ptyped->pvalue = pvalue;
+	ptyped->pvalue = std::move(pvalue);
 	return TRUE;
 }
 
-BOOL common_util_propvals_to_row_ex(cpid_t cpid,
-	BOOL b_unicode, const TPROPVAL_ARRAY *ppropvals,
-	const PROPTAG_ARRAY *pcolumns, PROPERTY_ROW *prow)
+bool cu_propvals_to_row_ex(cpid_t cpid, bool b_unicode,
+    const TPROPVAL_ARRAY *ppropvals, proptag_cspan pcolumns, PROPERTY_ROW *prow)
 {
 	static const uint32_t errcode = ecNotFound;
 	unsigned int i;
 	
-	for (i = 0; i < pcolumns->count; ++i)
-		if (!ppropvals->has(pcolumns->pproptag[i]))
+	for (i = 0; i < pcolumns.size(); ++i)
+		if (!ppropvals->has(pcolumns[i]))
 			break;	
-	prow->flag = i < pcolumns->count ? PROPERTY_ROW_FLAG_FLAGGED : PROPERTY_ROW_FLAG_NONE;
-	prow->pppropval = cu_alloc<void *>(pcolumns->count);
+	prow->flag = i < pcolumns.size() ? PROPERTY_ROW_FLAG_FLAGGED : PROPERTY_ROW_FLAG_NONE;
+	prow->pppropval = cu_alloc<void *>(pcolumns.size());
 	if (prow->pppropval == nullptr)
 		return FALSE;
-	for (i=0; i<pcolumns->count; i++) {
-		prow->pppropval[i] = deconst(ppropvals->getval(pcolumns->pproptag[i]));
+	for (i = 0; i < pcolumns.size(); ++i) {
+		prow->pppropval[i] = deconst(ppropvals->getval(pcolumns[i]));
 		if (prow->pppropval[i] != nullptr &&
-		    PROP_TYPE(pcolumns->pproptag[i]) == PT_UNSPECIFIED &&
+		    PROP_TYPE(pcolumns[i]) == PT_UNSPECIFIED &&
 		    !common_util_convert_unspecified(cpid, b_unicode,
 		    static_cast<TYPED_PROPVAL *>(prow->pppropval[i])))
 			return FALSE;
@@ -922,7 +876,7 @@ BOOL common_util_propvals_to_row_ex(cpid_t cpid,
 			return FALSE;
 		if (NULL == prow->pppropval[i]) {
 			pflagged_val->flag = FLAGGED_PROPVAL_FLAG_ERROR;
-			pflagged_val->pvalue = deconst(ppropvals->getval(CHANGE_PROP_TYPE(pcolumns->pproptag[i], PT_ERROR)));
+			pflagged_val->pvalue = deconst(ppropvals->getval(CHANGE_PROP_TYPE(pcolumns[i], PT_ERROR)));
 			if (pflagged_val->pvalue == nullptr)
 				pflagged_val->pvalue = deconst(&errcode);
 		} else {
@@ -934,11 +888,10 @@ BOOL common_util_propvals_to_row_ex(cpid_t cpid,
 	return TRUE;
 }
 
-BOOL common_util_row_to_propvals(
-	const PROPERTY_ROW *prow, const PROPTAG_ARRAY *pcolumns,
+static bool cu_row_to_propvals(const PROPERTY_ROW *prow, proptag_cspan pcolumns,
 	TPROPVAL_ARRAY *ppropvals)
 {
-	for (unsigned int i = 0; i < pcolumns->count; ++i) {
+	for (unsigned int i = 0; i < pcolumns.size(); ++i) {
 		void *pvalue;
 		if (PROPERTY_ROW_FLAG_NONE == prow->flag) {
 			pvalue = prow->pppropval[i];
@@ -948,15 +901,14 @@ BOOL common_util_row_to_propvals(
 				continue;	
 			pvalue = p->pvalue;
 		}
-		if (cu_set_propval(ppropvals, pcolumns->pproptag[i], pvalue) != ecSuccess)
+		if (cu_set_propval(ppropvals, pcolumns[i], pvalue) != ecSuccess)
 			return false;
 	}
 	return TRUE;
 }
 
-static BOOL common_util_propvals_to_recipient(cpid_t cpid,
-	TPROPVAL_ARRAY *ppropvals, const PROPTAG_ARRAY *pcolumns,
-	RECIPIENT_ROW *prow)
+static bool cu_propvals_to_recipient(cpid_t cpid, TPROPVAL_ARRAY *ppropvals,
+    proptag_cspan pcolumns, RECIPIENT_ROW *prow)
 {
 	memset(prow, 0, sizeof(RECIPIENT_ROW));
 	prow->flags |= RECIPIENT_ROW_FLAG_UNICODE;
@@ -970,13 +922,13 @@ static BOOL common_util_propvals_to_recipient(cpid_t cpid,
 	if (NULL == prow->ptransmittable_name) {
 		auto name = ppropvals->get<const char>(PR_TRANSMITABLE_DISPLAY_NAME_A);
 		if (name != nullptr)
-			prow->ptransmittable_name = common_util_dup_mb_to_utf8(cpid, name);
+			prow->ptransmittable_name = cu_mb_to_utf8_dup(cpid, name);
 	}
 	prow->pdisplay_name = ppropvals->get<char>(PR_DISPLAY_NAME);
 	if (NULL == prow->pdisplay_name) {
 		auto name = ppropvals->get<const char>(PR_DISPLAY_NAME_A);
 		if (name != nullptr)
-			prow->pdisplay_name = common_util_dup_mb_to_utf8(cpid, name);
+			prow->pdisplay_name = cu_mb_to_utf8_dup(cpid, name);
 	}
 	if (NULL != prow->ptransmittable_name && NULL != prow->pdisplay_name &&
 		0 == strcasecmp(prow->pdisplay_name, prow->ptransmittable_name)) {
@@ -991,7 +943,7 @@ static BOOL common_util_propvals_to_recipient(cpid_t cpid,
 	if (NULL == prow->psimple_name) {
 		auto name = ppropvals->get<const char>(PR_EMS_AB_DISPLAY_NAME_PRINTABLE_A);
 		if (name != nullptr)
-			prow->psimple_name = common_util_dup_mb_to_utf8(cpid, name);
+			prow->psimple_name = cu_mb_to_utf8_dup(cpid, name);
 	}
 	if (prow->psimple_name != nullptr)
 		prow->flags |= RECIPIENT_ROW_FLAG_SIMPLE;
@@ -1031,13 +983,12 @@ static BOOL common_util_propvals_to_recipient(cpid_t cpid,
 				return FALSE;
 		}
 	}
-	prow->count = pcolumns->count;
-	return common_util_propvals_to_row(ppropvals, pcolumns, &prow->properties);
+	prow->count = pcolumns.size();
+	return cu_propvals_to_row(ppropvals, pcolumns, &prow->properties);
 }
 
-static BOOL common_util_recipient_to_propvals(cpid_t cpid,
-	RECIPIENT_ROW *prow, const PROPTAG_ARRAY *pcolumns,
-	TPROPVAL_ARRAY *ppropvals)
+static bool cu_recipient_to_propvals(cpid_t cpid, RECIPIENT_ROW *prow,
+    proptag_cspan pcolumns, TPROPVAL_ARRAY *ppropvals)
 {
 	static constexpr uint8_t persist_true = true, persist_false = false;
 	BOOL b_unicode = (prow->flags & RECIPIENT_ROW_FLAG_UNICODE) ? TRUE : false;
@@ -1054,7 +1005,7 @@ static BOOL common_util_recipient_to_propvals(cpid_t cpid,
 		if (b_unicode) {
 			pvalue = prow->ptransmittable_name;
 		} else {
-			pvalue = common_util_dup_mb_to_utf8(cpid, prow->ptransmittable_name);
+			pvalue = cu_mb_to_utf8_dup(cpid, prow->ptransmittable_name);
 			if (pvalue == nullptr)
 				return FALSE;
 		}
@@ -1064,17 +1015,16 @@ static BOOL common_util_recipient_to_propvals(cpid_t cpid,
 	}
 	if (NULL != prow->pdisplay_name) {
 		auto pvalue = b_unicode ? prow->pdisplay_name :
-		              common_util_dup_mb_to_utf8(cpid, prow->pdisplay_name);
-		if (pvalue != nullptr && cu_set_propval(ppropvals,
-		    PR_DISPLAY_NAME, pvalue) != ecSuccess)
-			return false;
+		              cu_mb_to_utf8_dup(cpid, prow->pdisplay_name);
+		if (pvalue != nullptr)
+			cu_set_propval(ppropvals, PR_DISPLAY_NAME, pvalue);
 	}
 	if (NULL != prow->pmail_address) {
 		void *pvalue;
 		if (b_unicode) {
 			pvalue = prow->pmail_address;
 		} else {
-			pvalue = common_util_dup_mb_to_utf8(cpid, prow->pmail_address);
+			pvalue = cu_mb_to_utf8_dup(cpid, prow->pmail_address);
 			if (pvalue == nullptr)
 				return FALSE;
 		}
@@ -1103,8 +1053,8 @@ static BOOL common_util_recipient_to_propvals(cpid_t cpid,
 		return FALSE;
 	}
 
-	const PROPTAG_ARRAY tmp_columns = {prow->count, pcolumns->pproptag};
-	if (!common_util_row_to_propvals(&prow->properties, &tmp_columns, ppropvals))
+	const PROPTAG_ARRAY tmp_columns = {prow->count, deconst(pcolumns.data())};
+	if (!cu_row_to_propvals(&prow->properties, tmp_columns, ppropvals))
 		return FALSE;	
 	auto str = ppropvals->get<const char>(PR_DISPLAY_NAME);
 	if (str != nullptr && *str != '\0' && strcmp(str, "''") != 0 &&
@@ -1120,21 +1070,18 @@ static BOOL common_util_recipient_to_propvals(cpid_t cpid,
 	return TRUE;
 }
 
-BOOL common_util_propvals_to_openrecipient(cpid_t cpid,
-	TPROPVAL_ARRAY *ppropvals, const PROPTAG_ARRAY *pcolumns,
-	OPENRECIPIENT_ROW *prow)
+bool cu_propvals_to_openrecipient(cpid_t cpid, TPROPVAL_ARRAY *ppropvals,
+    proptag_cspan pcolumns, OPENRECIPIENT_ROW *prow)
 {
 	auto pvalue = ppropvals->get<uint32_t>(PR_RECIPIENT_TYPE);
 	prow->recipient_type = pvalue == nullptr ? MAPI_ORIG : *pvalue;
 	prow->reserved = 0;
 	prow->cpid = cpid;
-	return common_util_propvals_to_recipient(cpid,
-		ppropvals, pcolumns, &prow->recipient_row);
+	return cu_propvals_to_recipient(cpid, ppropvals, pcolumns, &prow->recipient_row);
 }
 
-BOOL common_util_propvals_to_readrecipient(cpid_t cpid,
-	TPROPVAL_ARRAY *ppropvals, const PROPTAG_ARRAY *pcolumns,
-	READRECIPIENT_ROW *prow)
+bool cu_propvals_to_readrecipient(cpid_t cpid, TPROPVAL_ARRAY *ppropvals,
+    proptag_cspan cols, READRECIPIENT_ROW *prow)
 {
 	auto pvalue = ppropvals->get<uint32_t>(PR_ROWID);
 	if (pvalue == nullptr)
@@ -1144,16 +1091,14 @@ BOOL common_util_propvals_to_readrecipient(cpid_t cpid,
 	prow->recipient_type = pvalue == nullptr ? MAPI_ORIG : *pvalue;
 	prow->reserved = 0;
 	prow->cpid = cpid;
-	return common_util_propvals_to_recipient(cpid,
-		ppropvals, pcolumns, &prow->recipient_row);
+	return cu_propvals_to_recipient(cpid, ppropvals, cols, &prow->recipient_row);
 }
 
-BOOL common_util_modifyrecipient_to_propvals(cpid_t cpid,
-    const MODIFYRECIPIENT_ROW *prow, const PROPTAG_ARRAY *pcolumns,
-    TPROPVAL_ARRAY *ppropvals)
+bool cu_modifyrecipient_to_propvals(cpid_t cpid, const MODIFYRECIPIENT_ROW *prow,
+    proptag_cspan pcolumns, TPROPVAL_ARRAY *ppropvals)
 {
 	ppropvals->count = 0;
-	ppropvals->ppropval = cu_alloc<TAGGED_PROPVAL>(16 + pcolumns->count);
+	ppropvals->ppropval = cu_alloc<TAGGED_PROPVAL>(16 + pcolumns.size());
 	if (ppropvals->ppropval == nullptr)
 		return FALSE;
 	if (cu_set_propval(ppropvals, PR_ROWID, deconst(&prow->row_id)) != ecSuccess)
@@ -1166,11 +1111,10 @@ BOOL common_util_modifyrecipient_to_propvals(cpid_t cpid,
 		return false;
 	if (prow->precipient_row == nullptr)
 		return TRUE;
-	return common_util_recipient_to_propvals(cpid,
-			prow->precipient_row, pcolumns, ppropvals);
+	return cu_recipient_to_propvals(cpid, prow->precipient_row, pcolumns, ppropvals);
 }
 
-static void common_util_convert_proptag(BOOL to_unicode, uint32_t *pproptag)
+static void common_util_convert_proptag(BOOL to_unicode, proptag_t *pproptag)
 {
 	if (to_unicode) {
 		if (PROP_TYPE(*pproptag) == PT_STRING8)
@@ -1192,27 +1136,19 @@ BOOL common_util_convert_tagged_propval(
 	if (to_unicode) {
 		switch (PROP_TYPE(ppropval->proptag)) {
 		case PT_STRING8: {
-			auto len = mb_to_utf8_len(static_cast<char *>(ppropval->pvalue));
-			auto pstring = cu_alloc<char>(len);
+			auto pstring = cu_mb_to_utf8_dup(CP_OEMCP, static_cast<char *>(ppropval->pvalue));
 			if (pstring == nullptr)
 				return FALSE;
-			if (common_util_convert_string(true,
-			    static_cast<char *>(ppropval->pvalue), pstring, len) < 0)
-				return FALSE;	
-			ppropval->pvalue = pstring;
+			ppropval->pvalue = std::move(pstring);
 			common_util_convert_proptag(TRUE, &ppropval->proptag);
 			break;
 		}
 		case PT_MV_STRING8: {
 			auto sa = static_cast<STRING_ARRAY *>(ppropval->pvalue);
 			for (auto &entry : *sa) {
-				auto len = mb_to_utf8_len(entry);
-				auto pstring = cu_alloc<char>(len);
-				if (pstring == nullptr)
+				entry = cu_mb_to_utf8_dup(CP_OEMCP, entry);
+				if (entry == nullptr)
 					return FALSE;
-				if (common_util_convert_string(true, entry, pstring, len) < 0)
-					return FALSE;	
-				entry = pstring;
 			}
 			common_util_convert_proptag(TRUE, &ppropval->proptag);
 			break;
@@ -1231,27 +1167,19 @@ BOOL common_util_convert_tagged_propval(
 	} else {
 		switch (PROP_TYPE(ppropval->proptag)) {
 		case PT_UNICODE: {
-			auto len = utf8_to_mb_len(static_cast<char *>(ppropval->pvalue));
-			auto pstring = cu_alloc<char>(len);
+			auto pstring = cu_utf8_to_mb_dup(CP_OEMCP, static_cast<char *>(ppropval->pvalue));
 			if (pstring == nullptr)
 				return FALSE;
-			if (common_util_convert_string(false,
-			    static_cast<char *>(ppropval->pvalue), pstring, len) < 0)
-				return FALSE;	
-			ppropval->pvalue = pstring;
+			ppropval->pvalue = std::move(pstring);
 			common_util_convert_proptag(FALSE, &ppropval->proptag);
 			break;
 		}
 		case PT_MV_UNICODE: {
 			auto sa = static_cast<STRING_ARRAY *>(ppropval->pvalue);
 			for (auto &entry : *sa) {
-				auto len = utf8_to_mb_len(entry);
-				auto pstring = cu_alloc<char>(len);
-				if (pstring == nullptr)
+				entry = cu_utf8_to_mb_dup(CP_OEMCP, entry);
+				if (entry == nullptr)
 					return FALSE;
-				if (common_util_convert_string(false, entry, pstring, len) < 0)
-					return FALSE;	
-				entry = pstring;
 			}
 			common_util_convert_proptag(FALSE, &ppropval->proptag);
 			break;
@@ -1423,7 +1351,7 @@ void common_util_notify_receipt(const char *username, int type,
 	if (ret != ecSuccess)
 		mlog2(LV_ERR, "E-1189: ems_send_mail: %s", mapi_strerror(ret));
 } catch (const std::bad_alloc &) {
-	mlog(LV_ERR, "E-2035: ENOMEM");
+	mlog(LV_ERR, "%s: ENOMEM", __func__);
 }
 
 static void common_util_set_dir(const char *dir)
@@ -1454,7 +1382,7 @@ static BOOL common_util_get_propname(propid_t propid, PROPERTY_NAME **pppropname
 	*pppropname = propnames.ppropname;
 	return TRUE;
 } catch (const std::bad_alloc &) {
-	mlog(LV_ERR, "E-2234: ENOMEM");
+	mlog(LV_ERR, "%s: ENOMEM", __func__);
 	return false;
 }
 
@@ -1464,7 +1392,7 @@ static BOOL common_util_get_propname(propid_t propid, PROPERTY_NAME **pppropname
 ec_error_t cu_send_message(logon_object *plogon, message_object *msg,
     const char *ev_from) try
 {
-	uint64_t message_id = msg->get_id();
+	auto message_id = msg->get_id();
 	MAIL imail;
 	void *pvalue;
 	BOOL b_result;
@@ -1515,22 +1443,25 @@ ec_error_t cu_send_message(logon_object *plogon, message_object *msg,
 	}
 	if (rcpt_list.size() == 0) {
 		/* What would EXC do? */
-		mlog2(LV_ERR, "E-1282: Empty converted recipients list attempting to send %s", log_id.c_str());
+		mlog2(LV_ERR, "E-1282: Empty converted recipients list attempting to send %s",
+			log_id.c_str());
 		return MAPI_E_NO_RECIPIENTS;
 	}
-	auto body_type = get_override_format(*pmsgctnt);
 	common_util_set_dir(dir);
-	/* try to avoid TNEF message */
-	if (!oxcmail_export(pmsgctnt, log_id.c_str(), false, body_type, &imail,
-	    common_util_alloc, common_util_get_propids, common_util_get_propname)) {
+	oxcmail_converter cvt;
+	cvt.log_id = log_id.c_str();
+	cvt.alloc = common_util_alloc;
+	cvt.get_propids = common_util_get_propids;
+	cvt.get_propname = common_util_get_propname;
+	cvt.use_format_override(*pmsgctnt);
+	if (!cvt.mapi_to_inet(*pmsgctnt, imail)) {
 		mlog2(LV_ERR, "E-1281: oxcmail_export %s failed", log_id.c_str());
 		return ecError;	
 	}
 
 	imail.set_header("X-Mailer", EMSMDB_UA);
 	if (emsmdb_backfill_transporthdr) {
-		std::unique_ptr<MESSAGE_CONTENT, mc_delete> rmsg(oxcmail_import("utf-8",
-			"UTC", &imail, common_util_alloc, common_util_get_propids));
+		auto rmsg = cvt.inet_to_mapi(imail);
 		if (rmsg != nullptr) {
 			for (auto tag : {PR_TRANSPORT_MESSAGE_HEADERS, PR_TRANSPORT_MESSAGE_HEADERS_A}) {
 				auto th = rmsg->proplist.get<const char>(tag);
@@ -1567,16 +1498,19 @@ ec_error_t cu_send_message(logon_object *plogon, message_object *msg,
 	auto ptarget = pmsgctnt->proplist.get<BINARY>(PR_TARGET_ENTRYID);
 	if (NULL != ptarget) {
 		if (!cu_entryid_to_mid(*plogon, ptarget, &folder_id, &new_id)) {
-			mlog2(LV_WARN, "W-1279: PR_TARGET_ENTRYID inconvertible in %s", log_id.c_str());
+			mlog2(LV_WARN, "W-1279: PR_TARGET_ENTRYID inconvertible in %s",
+				log_id.c_str());
 			return ecWarnWithErrors;	
 		}
 		if (!exmdb_client->clear_submit(dir, message_id, false)) {
-			mlog2(LV_WARN, "W-1278: exrpc clear_submit %s failed", log_id.c_str());
+			mlog2(LV_WARN, "W-1278: exrpc clear_submit %s failed",
+				log_id.c_str());
 			return ecWarnWithErrors;
 		}
 		if (!exmdb_client->movecopy_message(dir, cpid, message_id,
 		    folder_id, new_id, TRUE, &b_result)) {
-			mlog2(LV_WARN, "W-1277: exrpc movecopy_message %s failed", log_id.c_str());
+			mlog2(LV_WARN, "W-1277: exrpc movecopy_message %s failed",
+				log_id.c_str());
 			return ecWarnWithErrors;
 		}
 		return ecSuccess;
@@ -1586,7 +1520,8 @@ ec_error_t cu_send_message(logon_object *plogon, message_object *msg,
 		return ecSuccess;
 	}
 	if (!exmdb_client->clear_submit(dir, message_id, false)) {
-		mlog2(LV_WARN, "W-1276: exrpc clear_submit %s failed", log_id.c_str());
+		mlog2(LV_WARN, "W-1276: exrpc clear_submit %s failed",
+			log_id.c_str());
 		return ecWarnWithErrors;
 	}
 
@@ -1599,12 +1534,13 @@ ec_error_t cu_send_message(logon_object *plogon, message_object *msg,
 	if (!exmdb_client->movecopy_messages(dir, cpid,
 	    false, STORE_OWNER_GRANTED, parent_id, folder_id, false,
 	    &ids, &b_partial)) {
-		mlog2(LV_WARN, "W-1275: exrpc movecopy_message %s failed", log_id.c_str());
+		mlog2(LV_WARN, "W-1275: exrpc movecopy_message %s failed",
+			log_id.c_str());
 		return ecWarnWithErrors;
 	}
 	return ecSuccess;
 } catch (const std::bad_alloc &) {
-	mlog(LV_ERR, "E-2553: ENOMEM");
+	mlog(LV_ERR, "%s: ENOMEM", __func__);
 	return ecServerOOM;
 }
 
