@@ -3,6 +3,7 @@
 // This file is part of Gromox.
 #include <cassert>
 #include <algorithm>
+#include <cerrno>
 #include <chrono>
 #include <climits>
 #include <csignal>
@@ -23,6 +24,7 @@
 #include <gromox/defs.h>
 #include <gromox/mapidefs.h>
 #include <gromox/mysql_adaptor.hpp>
+#include <gromox/notify_types.hpp>
 #include <gromox/proc_common.h>
 #include <gromox/process.hpp>
 #include <gromox/rop_util.hpp>
@@ -91,12 +93,11 @@ struct NOTIFY_ITEM {
 }
 
 static constexpr time_duration HANDLE_VALID_INTERVAL = std::chrono::seconds(2000);
-static constexpr size_t TAG_SIZE = 256;
 static time_point g_start_time;
 static pthread_t g_scan_id;
 static std::mutex g_lock; /* protects g_handle_hash & g_user_hash */
 static std::mutex g_notify_lock;
-static gromox::atomic_bool g_notify_stop{true};
+static gromox::atomic_bool g_emsi_stop{true};
 static thread_local HANDLE_DATA *g_handle_key;
 static std::unordered_map<GUID, HANDLE_DATA> g_handle_hash;
 static std::unordered_map<std::string, std::vector<HANDLE_DATA *>> g_user_hash;
@@ -170,8 +171,8 @@ static uint32_t emsmdb_interface_get_timestamp()
 	return std::chrono::duration_cast<std::chrono::seconds>(d).count() + 1230336000;
 }
 
-BOOL emsmdb_interface_check_acxh(ACXH *pacxh,
-	char *username, uint16_t *pcxr, BOOL b_touch)
+bool emsmdb_interface_inspect_acxh(const ACXH *pacxh, std::string &username,
+    uint16_t *pcxr, bool b_touch) try
 {
 	if (pacxh->handle_type != HANDLE_EXCHANGE_ASYNCEMSMDB)
 		return FALSE;
@@ -182,12 +183,15 @@ BOOL emsmdb_interface_check_acxh(ACXH *pacxh,
 	auto phandle = &iter->second;
 	if (b_touch)
 		phandle->last_time = tp_now();
-	strcpy(username, phandle->username);
+	username = phandle->username;
 	*pcxr = phandle->cxr;
 	return TRUE;
+} catch (const std::bad_alloc &) {
+	mlog(LV_ERR, "%s: ENOMEM", __func__);
+	return false;
 }
 
-bool emsmdb_interface_notifications_pending(ACXH &acxh)
+bool emsmdb_interface_notifications_pending(const ACXH &acxh)
 {
 	auto pacxh = &acxh;
 	if (pacxh->handle_type != HANDLE_EXCHANGE_ASYNCEMSMDB)
@@ -311,7 +315,7 @@ static BOOL emsmdb_interface_create_handle(const char *username,
 {
 	HANDLE_DATA temp_handle;
 	
-	if (!verify_cpid(cpid))
+	if (!acceptable_cpid_for_mapi(cpid))
 		return FALSE;
 	temp_handle.info.cpid = cpid;
 	temp_handle.info.lcid_string = lcid_string;
@@ -431,10 +435,10 @@ void emsmdb_interface_init()
 
 int emsmdb_interface_run()
 {
-	g_notify_stop = false;
+	g_emsi_stop = false;
 	auto ret = pthread_create4(&g_scan_id, nullptr, emsi_scanwork, nullptr);
 	if (ret != 0) {
-		g_notify_stop = true;
+		g_emsi_stop = true;
 		mlog(LV_ERR, "E-1447: pthread_create: %s", strerror(ret));
 		return -4;
 	}
@@ -444,8 +448,8 @@ int emsmdb_interface_run()
 
 void emsmdb_interface_stop()
 {
-	if (!g_notify_stop) {
-		g_notify_stop = true;
+	if (!g_emsi_stop) {
+		g_emsi_stop = true;
 		if (!pthread_equal(g_scan_id, {})) {
 			pthread_kill(g_scan_id, SIGALRM);
 			pthread_join(g_scan_id, NULL);
@@ -515,11 +519,11 @@ static void emsmdb_interface_encode_version(BOOL high_bit,
 ec_error_t emsmdb_interface_connect_ex(uint64_t hrpc, CXH *pcxh, const char *puser_dn,
     uint32_t flags, uint32_t con_mode, uint32_t limit, cpid_t cpid,
     uint32_t lcid_string, uint32_t lcid_sort, uint32_t cxr_link, uint16_t cnvt_cps,
-	uint32_t *pmax_polls, uint32_t *pmax_retry, uint32_t *pretry_delay,
-	uint16_t *pcxr, char *pdn_prefix, char *pdisplayname,
-	const uint16_t pclient_vers[3], uint16_t pserver_vers[3],
-	uint16_t pbest_vers[3], uint32_t *ptimestamp, const uint8_t *pauxin,
-	uint32_t cb_auxin, uint8_t *pauxout, uint32_t *pcb_auxout)
+    uint32_t *pmax_polls, uint32_t *pmax_retry, uint32_t *pretry_delay,
+    uint16_t *pcxr, std::string &pdn_prefix, std::string &pdisplayname,
+    const uint16_t pclient_vers[3], uint16_t pserver_vers[3],
+    uint16_t pbest_vers[3], uint32_t *ptimestamp, const uint8_t *pauxin,
+    uint32_t cb_auxin, uint8_t *pauxout, uint32_t *pcb_auxout) try
 {
 	AUX_INFO aux_out;
 	EXT_PUSH ext_push;
@@ -536,7 +540,7 @@ ec_error_t emsmdb_interface_connect_ex(uint64_t hrpc, CXH *pcxh, const char *pus
 		*pmax_retry = 0;
 		*pretry_delay = 0;
 		*pcxr = 0;
-		pdisplayname[0] = '\0';
+		pdisplayname.clear();
 		memset(pserver_vers, 0, 3 * sizeof(*pserver_vers));
 		memset(pbest_vers, 0, 3 * sizeof(*pbest_vers));
 		*ptimestamp = 0;
@@ -570,7 +574,7 @@ ec_error_t emsmdb_interface_connect_ex(uint64_t hrpc, CXH *pcxh, const char *pus
 	              0 : ext_push.m_offset;
 	aux_out.aux_list.clear();
 	
-	pdn_prefix[0] = '\0';
+	pdn_prefix.clear();
 	rpc_info = get_rpc_info();
 	if (flags & FLAG_PRIVILEGE_ADMIN)
 		return ecLoginPerm;
@@ -591,12 +595,20 @@ ec_error_t emsmdb_interface_connect_ex(uint64_t hrpc, CXH *pcxh, const char *pus
 	if (strcasecmp(username.c_str(), rpc_info.username) != 0)
 		return ecAccessDenied;
 	std::string uds;
-	if (!mysql_adaptor_get_user_displayname(username.c_str(), uds) ||
-	    cu_utf8_to_mb(cpid, uds.c_str(), pdisplayname, 1024) < 0)
+	if (!mysql_adaptor_get_user_displayname(username.c_str(), uds))
 		return ecRpcFailed;
-	if (uds.empty())
-		strcpy(pdisplayname, rpc_info.username);
-	
+	pdisplayname.clear();
+	if (!uds.empty()) {
+		auto uds_cvt = cu_utf8_to_mb(cpid, uds);
+		if (errno == ENOMEM)
+			return ecServerOOM;
+		else if (errno != 0)
+			return ecRpcFailed;
+		pdisplayname = std::move(uds_cvt);
+	}
+	if (pdisplayname.empty())
+		pdisplayname = rpc_info.username;
+
 	emsmdb_interface_decode_version(pclient_vers, client_version);
 	emsmdb_interface_encode_version(TRUE, server_normal_version, pserver_vers);
 	pbest_vers[0] = pclient_vers[0];
@@ -619,6 +631,8 @@ ec_error_t emsmdb_interface_connect_ex(uint64_t hrpc, CXH *pcxh, const char *pus
 		return ecLoginFailure;
 	is_success = true;
 	return ecSuccess;
+} catch (const std::bad_alloc &) {
+	return ecServerOOM;
 }
 
 static bool enable_rop_chaining(uint16_t v[4])
@@ -632,11 +646,9 @@ static bool enable_rop_chaining(uint16_t v[4])
 ec_error_t emsmdb_interface_rpc_ext2(CXH &cxh, uint32_t *pflags,
 	const uint8_t *pin, uint32_t cb_in, uint8_t *pout, uint32_t *pcb_out,
 	const uint8_t *pauxin, uint32_t cb_auxin, uint8_t *pauxout,
-	uint32_t *pcb_auxout, uint32_t *ptrans_time)
+	uint32_t *pcb_auxout, uint32_t *ptrans_time) try
 {
 	auto pcxh = &cxh;
-	uint16_t cxr;
-	char username[UADDR_SIZE];
 	HANDLE_DATA *phandle;
 	auto input_flags = *pflags;
 	*pflags = 0;
@@ -690,13 +702,14 @@ ec_error_t emsmdb_interface_rpc_ext2(CXH &cxh, uint32_t *pflags,
 		input_flags &= ~GROMOX_READSTREAM_NOCHAIN;
 	else
 		input_flags |= GROMOX_READSTREAM_NOCHAIN;
+
 	auto result = rop_processor_proc(input_flags, pin, cb_in, pout, pcb_out);
-	gx_strlcpy(username, phandle->username, std::size(username));
-	cxr = phandle->cxr;
+	std::string username = phandle->username;
+	uint16_t cxr = phandle->cxr;
 	BOOL b_wakeup = double_list_get_nodes_num(&phandle->notify_list) == 0 ? false : TRUE;
 	emsmdb_interface_put_handle_data(phandle);
 	if (b_wakeup)
-		asyncemsmdb_interface_wakeup(username, cxr);
+		asyncemsmdb_interface_wakeup(std::move(username), cxr);
 	g_handle_key = nullptr;
 	if (result != ecSuccess) {
 		*pcb_out = 0;
@@ -704,6 +717,9 @@ ec_error_t emsmdb_interface_rpc_ext2(CXH &cxh, uint32_t *pflags,
 	}
 	*ptrans_time = std::chrono::duration_cast<std::chrono::milliseconds>(tp_now() - first_time).count();
 	return ecSuccess;
+} catch (const std::bad_alloc &) {
+	mlog(LV_ERR, "%s: ENOMEM", __func__);
+	return ecServerOOM;
 }
 	
 ec_error_t emsmdb_interface_async_connect_ex(CXH cxh, ACXH *pacxh)
@@ -827,16 +843,29 @@ BOOL emsmdb_interface_set_rop_num(int num)
 	return TRUE;
 }
 
+/**
+ * Generate keys for the notify map. Since the map can have different kinds of
+ * elements, the tag formats need to be unique.
+ */
+static std::string make_table_notify_tag(const char *dir, uint32_t id)
+{
+	return std::to_string(id) + ":" + dir;
+}
+
+static std::string make_sub_notify_tag(const char *dir, uint32_t id)
+{
+	return std::to_string(id) + "|" + dir;
+}
+
 void emsmdb_interface_add_table_notify(const char *dir,
     uint32_t table_id, uint32_t handle, uint8_t logon_id, GUID *pguid) try
 {
-	char tag_buff[TAG_SIZE];
 	NOTIFY_ITEM tmp_notify;
 	
 	tmp_notify.handle = handle;
 	tmp_notify.logon_id = logon_id;
 	tmp_notify.guid = *pguid;
-	snprintf(tag_buff, std::size(tag_buff), "%u:%s", table_id, dir);
+	auto tag = make_table_notify_tag(dir, table_id);
 	std::lock_guard nt_hold(g_notify_lock);
 	if (ems_max_active_notifh > 0 &&
 	    g_notify_hash.size() >= ems_max_active_notifh) {
@@ -844,20 +873,19 @@ void emsmdb_interface_add_table_notify(const char *dir,
 			ems_max_active_notifh);
 		return;
 	}
-	g_notify_hash.emplace(tag_buff, std::move(tmp_notify));
+	g_notify_hash.emplace(std::move(tag), std::move(tmp_notify));
 	ems_high_active_notifh = std::max(ems_high_active_notifh, g_notify_hash.size());
 } catch (const std::bad_alloc &) {
 	mlog(LV_WARN, "W-1541: ENOMEM");
 }
 
-static BOOL emsmdb_interface_get_table_notify(const char *dir,
-	uint32_t table_id, uint32_t *phandle, uint8_t *plogon_id, GUID *pguid)
+static BOOL emsmdb_interface_get_table_notify(const char *dir, uint32_t table_id,
+    uint32_t *phandle, uint8_t *plogon_id, GUID *pguid) try
 {
-	char tag_buff[TAG_SIZE];
-	snprintf(tag_buff, std::size(tag_buff), "%u:%s", table_id, dir);
+	auto tag = make_table_notify_tag(dir, table_id);
 	std::lock_guard nt_hold(g_notify_lock);
 	const auto &nh = g_notify_hash;
-	auto iter = nh.find(tag_buff);
+	auto iter = nh.find(std::move(tag));
 	if (iter == nh.cend())
 		return FALSE;
 	auto pnotify = &iter->second;
@@ -865,30 +893,29 @@ static BOOL emsmdb_interface_get_table_notify(const char *dir,
 	*plogon_id = pnotify->logon_id;
 	*pguid = pnotify->guid;
 	return TRUE;
+} catch (const std::bad_alloc &) {
+	mlog(LV_WARN, "%s: ENOMEM", __func__);
+	return false;
 }
 
-void emsmdb_interface_remove_table_notify(
-	const char *dir, uint32_t table_id)
+void emsmdb_interface_remove_table_notify(const char *dir, uint32_t table_id) try
 {
-	char tag_buff[TAG_SIZE];
-	
-	snprintf(tag_buff, std::size(tag_buff), "%u:%s", table_id, dir);
+	auto tag = make_table_notify_tag(dir, table_id);
 	std::lock_guard nt_hold(g_notify_lock);
-	g_notify_hash.erase(tag_buff);
+	g_notify_hash.erase(std::move(tag));
+} catch (const std::bad_alloc &) {
+	mlog(LV_WARN, "%s: ENOMEM", __func__);
 }
 
 void emsmdb_interface_add_subscription_notify(const char *dir,
     uint32_t sub_id, uint32_t handle, uint8_t logon_id, GUID *pguid) try
 {
-	char tag_buff[TAG_SIZE];
 	NOTIFY_ITEM tmp_notify;
 	
-	
 	tmp_notify.handle = handle;
 	tmp_notify.logon_id = logon_id;
 	tmp_notify.guid = *pguid;
-	
-	snprintf(tag_buff, std::size(tag_buff), "%u|%s", sub_id, dir);
+	auto tag = make_sub_notify_tag(dir, sub_id);
 	std::lock_guard nt_hold(g_notify_lock);
 	if (ems_max_active_notifh > 0 &&
 	    g_notify_hash.size() >= ems_max_active_notifh) {
@@ -896,21 +923,19 @@ void emsmdb_interface_add_subscription_notify(const char *dir,
 			ems_max_active_notifh);
 		return;
 	}
-	g_notify_hash.emplace(tag_buff, std::move(tmp_notify));
+	g_notify_hash.emplace(std::move(tag), std::move(tmp_notify));
 	ems_high_active_notifh = std::max(ems_high_active_notifh, g_notify_hash.size());
 } catch (const std::bad_alloc &) {
 	mlog(LV_WARN, "W-1542: ENOMEM");
 }
 
-static BOOL emsmdb_interface_get_subscription_notify(
-	const char *dir, uint32_t sub_id, uint32_t *phandle,
-	uint8_t *plogon_id, GUID *pguid)
+static BOOL emsmdb_interface_get_subscription_notify(const char *dir,
+    uint32_t sub_id, uint32_t *phandle, uint8_t *plogon_id, GUID *pguid) try
 {
-	char tag_buff[TAG_SIZE];
-	snprintf(tag_buff, std::size(tag_buff), "%u|%s", sub_id, dir);
+	auto tag = make_sub_notify_tag(dir, sub_id);
 	std::lock_guard nt_hold(g_notify_lock);
 	const auto &nh = g_notify_hash;
-	auto iter = nh.find(tag_buff);
+	auto iter = nh.find(std::move(tag));
 	if (iter == nh.cend())
 		return FALSE;
 	auto pnotify = &iter->second;
@@ -918,16 +943,18 @@ static BOOL emsmdb_interface_get_subscription_notify(
 	*plogon_id = pnotify->logon_id;
 	*pguid = pnotify->guid;
 	return TRUE;
+} catch (const std::bad_alloc &) {
+	mlog(LV_WARN, "%s: ENOMEM", __func__);
+	return false;
 }
 
-void emsmdb_interface_remove_subscription_notify(
-	const char *dir, uint32_t sub_id)
+void emsmdb_interface_remove_subscription_notify(const char *dir, uint32_t sub_id) try
 {
-	char tag_buff[TAG_SIZE];
-	
-	snprintf(tag_buff, std::size(tag_buff), "%u|%s", sub_id, dir);
+	auto tag = make_sub_notify_tag(dir, sub_id);
 	std::lock_guard nt_hold(g_notify_lock);
-	g_notify_hash.erase(tag_buff);
+	g_notify_hash.erase(std::move(tag));
+} catch (const std::bad_alloc &) {
+	mlog(LV_WARN, "%s: ENOMEM", __func__);
 }
 
 static BOOL emsmdb_interface_merge_content_row_deleted(
@@ -1030,13 +1057,13 @@ static BOOL emsmdb_interface_merge_folder_modified(
 }
 
 void emsmdb_interface_event_proc(const char *dir, BOOL b_table,
-	uint32_t notify_id, const DB_NOTIFY *pdb_notify)
+    uint32_t notify_id, const DB_NOTIFY *pdb_notify) try
 {
 	CXH cxh;
 	uint16_t cxr;
 	uint8_t logon_id;
 	BOOL b_processing;
-	char username[UADDR_SIZE];
+	std::string username;
 	uint32_t obj_handle;
 	HANDLE_DATA *phandle;
 	DOUBLE_LIST_NODE *pnode;
@@ -1062,28 +1089,28 @@ void emsmdb_interface_event_proc(const char *dir, BOOL b_table,
 		return;
 	case db_notify_type::hiertbl_row_modified:
 		if (!emsmdb_interface_merge_hierarchy_row_modified(
-		    static_cast<const DB_NOTIFY_HIERARCHY_TABLE_ROW_MODIFIED *>(pdb_notify->pdata),
+		    std::any_cast<const DB_NOTIFY_HIERARCHY_TABLE_ROW_MODIFIED>(&pdb_notify->pdata),
 		    obj_handle, logon_id, &phandle->notify_list))
 			break;
 		b_processing = phandle->b_processing;
 		if (!b_processing) {
 			cxr = phandle->cxr;
-			gx_strlcpy(username, phandle->username, std::size(username));
+			username = phandle->username;
 		}
 		emsmdb_interface_put_handle_notify_list(phandle);
 		if (!b_processing)
-			asyncemsmdb_interface_wakeup(username, cxr);
+			asyncemsmdb_interface_wakeup(std::move(username), cxr);
 		return;
 	case db_notify_type::message_modified:
 		if (!emsmdb_interface_merge_message_modified(
-		    static_cast<const DB_NOTIFY_MESSAGE_MODIFIED *>(pdb_notify->pdata),
+		    std::any_cast<const DB_NOTIFY_MESSAGE_MODIFIED>(&pdb_notify->pdata),
 		    obj_handle, logon_id, &phandle->notify_list))
 			break;
 		emsmdb_interface_put_handle_notify_list(phandle);
 		return;
 	case db_notify_type::folder_modified:
 		if (!emsmdb_interface_merge_folder_modified(
-		    static_cast<const DB_NOTIFY_FOLDER_MODIFIED *>(pdb_notify->pdata),
+		    std::any_cast<const DB_NOTIFY_FOLDER_MODIFIED>(&pdb_notify->pdata),
 		    obj_handle, logon_id, &phandle->notify_list))
 			break;
 		emsmdb_interface_put_handle_notify_list(phandle);
@@ -1100,7 +1127,7 @@ void emsmdb_interface_event_proc(const char *dir, BOOL b_table,
 	}
 	ems_high_pending_sesnotif = std::max(ems_high_pending_sesnotif, notifnum);
 	cxr = phandle->cxr;
-	gx_strlcpy(username, phandle->username, std::size(username));
+	username = phandle->username;
 	pnode = me_alloc<DOUBLE_LIST_NODE>();
 	if (NULL == pnode) {
 		emsmdb_interface_put_handle_notify_list(phandle);
@@ -1128,12 +1155,14 @@ void emsmdb_interface_event_proc(const char *dir, BOOL b_table,
 		free(pnode);
 	}
 	if (!b_processing)
-		asyncemsmdb_interface_wakeup(username, cxr);
+		asyncemsmdb_interface_wakeup(std::move(username), cxr);
+} catch (const std::bad_alloc &) {
+	mlog(LV_ERR, "%s: ENOMEM", __func__);
 }
 
 static void *emsi_scanwork(void *pparam)
 {
-	while (!g_notify_stop) {
+	while (!g_emsi_stop) {
 		std::vector<GUID> temp_list;
 		auto cur_time = tp_now();
 		std::unique_lock gl_hold(g_lock);
@@ -1144,7 +1173,7 @@ static void *emsi_scanwork(void *pparam)
 			if (cur_time - phandle->last_time > HANDLE_VALID_INTERVAL) try {
 				temp_list.push_back(guid);
 			} catch (const std::bad_alloc &) {
-				mlog(LV_ERR, "E-1624: ENOMEM");
+				mlog(LV_ERR, "%s: ENOMEM", __func__);
 				continue;
 			}
 		}
