@@ -36,7 +36,7 @@ struct icsdownctx_object final {
 
 	public:
 	~icsdownctx_object();
-	static std::unique_ptr<icsdownctx_object> create(logon_object *, folder_object *, uint8_t sync_type, uint8_t send_options, uint16_t sync_flags, const RESTRICTION *, uint32_t extra_flags, const PROPTAG_ARRAY *);
+	static std::unique_ptr<icsdownctx_object> create(logon_object *, folder_object *, uint8_t sync_type, uint8_t send_options, uint16_t sync_flags, const RESTRICTION *, uint32_t extra_flags, proptag_cspan);
 	bool begin_state_stream(gromox::proptag_t state);
 	BOOL continue_state_stream(const BINARY *stream_data);
 	BOOL end_state_stream();
@@ -61,7 +61,7 @@ struct icsdownctx_object final {
 	uint8_t send_options = 0;
 	uint16_t sync_flags = 0;
 	uint32_t extra_flags = 0, divisor = 1;
-	PROPTAG_ARRAY *pproptags = nullptr;
+	proptag_vector pproptags;
 	RESTRICTION *prestriction = nullptr;
 	uint64_t total_steps = 0, progress_steps = 0, next_progress_steps = 0;
 };
