@@ -335,7 +335,7 @@ static pack_result rop_ext_pull(EXT_PULL &x, SETSEARCHCRITERIA_REQUEST &r)
 			return pack_result::format;
 		x.m_offset = offset;
 	}
-	TRY(x.g_uint64_sa(&r.folder_ids));
+	TRY(x.g_eid_a(&r.folder_ids, 2));
 	return x.g_uint32(&r.search_flags);
 }
 
@@ -361,14 +361,14 @@ static pack_result rop_ext_push(EXT_PUSH &x, const GETSEARCHCRITERIA_RESPONSE &r
 		x.m_offset = offset2;
 	}
 	TRY(x.p_uint8(r.logon_id));
-	TRY(x.p_uint64_sa(r.folder_ids));
+	TRY(x.p_eid_a(r.folder_ids, 2));
 	return x.p_uint32(r.search_status);
 }
 
 static pack_result rop_ext_pull(EXT_PULL &x, MOVECOPYMESSAGES_REQUEST &r)
 {
 	TRY(x.g_uint8(&r.dhindex));
-	TRY(x.g_uint64_sa(&r.message_ids));
+	TRY(x.g_eid_a(&r.message_ids, 2));
 	TRY(x.g_uint8(&r.want_asynchronous));
 	return x.g_uint8(&r.want_copy);
 }
@@ -409,14 +409,14 @@ static pack_result rop_ext_pull(EXT_PULL &x, DELETEMESSAGES_REQUEST &r)
 {
 	TRY(x.g_uint8(&r.want_asynchronous));
 	TRY(x.g_uint8(&r.notify_non_read));
-	return x.g_uint64_sa(&r.message_ids);
+	return x.g_eid_a(&r.message_ids, 2);
 }
 
 static pack_result rop_ext_pull(EXT_PULL &x, HARDDELETEMESSAGES_REQUEST &r)
 {
 	TRY(x.g_uint8(&r.want_asynchronous));
 	TRY(x.g_uint8(&r.notify_non_read));
-	return x.g_uint64_sa(&r.message_ids);
+	return x.g_eid_a(&r.message_ids, 2);
 }
 
 static pack_result rop_ext_pull(EXT_PULL &x, GETHIERARCHYTABLE_REQUEST &r)
@@ -570,7 +570,7 @@ static pack_result rop_ext_push(EXT_PUSH &x, const FINDROW_RESPONSE &r)
 	if (r.prow == nullptr)
 		return x.p_uint8(0);
 	TRY(x.p_uint8(1));
-	TRY(x.p_proprow(*r.pcolumns, *r.prow));
+	TRY(x.p_proprow(r.pcolumns, *r.prow));
 	return pack_result::ok;
 }
 
@@ -787,7 +787,7 @@ static pack_result rop_ext_pull(EXT_PULL &x, SETREADFLAGS_REQUEST &r)
 {
 	TRY(x.g_uint8(&r.want_asynchronous));
 	TRY(x.g_uint8(&r.read_flags));
-	return x.g_uint64_sa(&r.message_ids);
+	return x.g_eid_a(&r.message_ids, 2);
 }
 
 static pack_result rop_ext_pull(EXT_PULL &x, SETMESSAGEREADFLAG_REQUEST &r,
@@ -995,7 +995,7 @@ static pack_result rop_ext_pull(EXT_PULL &x, GETPROPERTIESSPECIFIC_REQUEST &r)
 static pack_result rop_ext_push(EXT_PUSH &x,
     const GETPROPERTIESSPECIFIC_RESPONSE &r)
 {
-	return x.p_proprow(*r.pproptags, r.row);
+	return x.p_proprow(r.pproptags, r.row);
 }
 
 static pack_result rop_ext_pull(EXT_PULL &x, GETPROPERTIESALL_REQUEST &r)
@@ -1320,7 +1320,7 @@ static pack_result rop_ext_pull(EXT_PULL &x,
     FASTTRANSFERSOURCECOPYMESSAGES_REQUEST &r)
 {
 	TRY(x.g_uint8(&r.ohindex));
-	TRY(x.g_uint64_sa(&r.message_ids));
+	TRY(x.g_eid_a(&r.message_ids, 2));
 	TRY(x.g_uint8(&r.flags));
 	return x.g_uint8(&r.send_options);
 }
