@@ -42,7 +42,7 @@ using namespace gromox;
 
 static int g_scan_interval;
 static pthread_t g_scan_id;
-static gromox::atomic_bool g_notify_stop{true};
+static gromox::atomic_bool g_rop_stop{true};
 static std::mutex g_hash_lock;
 static std::unordered_map<std::string, uint32_t> g_logon_hash;
 static unsigned int g_emsmdb_full_parenting;
@@ -147,7 +147,7 @@ int32_t rop_processor_create_logon_item(LOGMAP *plogmap,
 		g_logon_hash.emplace(rlogon->get_dir(), 1);
 	return handle;
 } catch (const std::bad_alloc &) {
-	mlog(LV_ERR, "E-1974: ENOMEM");
+	mlog(LV_ERR, "%s: ENOMEM", __func__);
 	return -ENOMEM;
 }
 
@@ -246,7 +246,7 @@ int32_t rop_processor_add_object_handle(LOGMAP *plogmap, uint8_t logon_id,
 	}
 	return pobjnode->handle;
 } catch (const std::bad_alloc &) {
-	mlog(LV_ERR, "E-1975: ENOMEM");
+	mlog(LV_ERR, "%s: ENOMEM", __func__);
 	return -ENOMEM;
 }
 
@@ -317,7 +317,7 @@ static void *emsrop_scanwork(void *param)
 	int count;
 	
 	count = 0;
-	while (!g_notify_stop) try {
+	while (!g_rop_stop) try {
 		sleep(1);
 		count ++;
 		if (count < g_scan_interval) {
@@ -347,10 +347,10 @@ void rop_processor_init(int scan_interval)
 
 int rop_processor_run()
 {
-	g_notify_stop = false;
+	g_rop_stop = false;
 	auto ret = pthread_create4(&g_scan_id, nullptr, emsrop_scanwork, nullptr);
 	if (ret != 0) {
-		g_notify_stop = true;
+		g_rop_stop = true;
 		mlog(LV_ERR, "emsmdb: failed to create scanning thread "
 		       "for logon hash table: %s", strerror(ret));
 		return -5;
@@ -361,8 +361,8 @@ int rop_processor_run()
 
 void rop_processor_stop()
 {
-	if (!g_notify_stop) {
-		g_notify_stop = true;
+	if (!g_rop_stop) {
+		g_rop_stop = true;
 		if (!pthread_equal(g_scan_id, {})) {
 			pthread_kill(g_scan_id, SIGALRM);
 			pthread_join(g_scan_id, NULL);
@@ -541,7 +541,7 @@ static ec_error_t rop_processor_execute_and_push(uint8_t *pbuff,
 					    0, &propvals) || propvals.count == 0)
 						goto NEXT_NOTIFY;
 				}
-				if (!common_util_propvals_to_row(&propvals, pcolumns, &tmp_row) ||
+				if (!cu_propvals_to_row(&propvals, *pcolumns, &tmp_row) ||
 				    ext_push1.p_proprow(*pcolumns, tmp_row) != pack_result::ok)
 					goto NEXT_NOTIFY;
 				tmp_bin.cb = ext_push1.m_offset;
@@ -569,7 +569,7 @@ static ec_error_t rop_processor_execute_and_push(uint8_t *pbuff,
 		return ecError;
 	return ecSuccess;
 } catch (const std::bad_alloc &) {
-	mlog(LV_ERR, "E-1173: ENOMEM");
+	mlog(LV_ERR, "%s: ENOMEM", __func__);
 	return ecServerOOM;
 }
 
