@@ -116,7 +116,7 @@ BOOL exmdb_client_local::get_store_all_proptags(const char *dir, PROPTAG_ARRAY *
 	return xbresult;
 }
 
-BOOL exmdb_client_local::get_store_properties(const char *dir, cpid_t cpid, const PROPTAG_ARRAY *pproptags, TPROPVAL_ARRAY *propvals)
+BOOL exmdb_client_local::get_store_properties(const char *dir, cpid_t cpid, proptag_cspan pproptags, TPROPVAL_ARRAY *propvals)
 {
 	BOOL xb_private;
 
@@ -144,7 +144,7 @@ BOOL exmdb_client_local::set_store_properties(const char *dir, cpid_t cpid, cons
 	return xbresult;
 }
 
-BOOL exmdb_client_local::remove_store_properties(const char *dir, const PROPTAG_ARRAY *pproptags)
+BOOL exmdb_client_local::remove_store_properties(const char *dir, proptag_cspan pproptags)
 {
 	BOOL xb_private;
 
@@ -312,7 +312,7 @@ BOOL exmdb_client_local::get_folder_all_proptags(const char *dir, uint64_t folde
 	return xbresult;
 }
 
-BOOL exmdb_client_local::get_folder_properties(const char *dir, cpid_t cpid, uint64_t folder_id, const PROPTAG_ARRAY *pproptags, TPROPVAL_ARRAY *propvals)
+BOOL exmdb_client_local::get_folder_properties(const char *dir, cpid_t cpid, uint64_t folder_id, proptag_cspan pproptags, TPROPVAL_ARRAY *propvals)
 {
 	BOOL xb_private;
 
@@ -340,7 +340,7 @@ BOOL exmdb_client_local::set_folder_properties(const char *dir, cpid_t cpid, uin
 	return xbresult;
 }
 
-BOOL exmdb_client_local::remove_folder_properties(const char *dir, uint64_t folder_id, const PROPTAG_ARRAY *pproptags)
+BOOL exmdb_client_local::remove_folder_properties(const char *dir, uint64_t folder_id, proptag_cspan pproptags)
 {
 	BOOL xb_private;
 
@@ -410,7 +410,7 @@ BOOL exmdb_client_local::copy_folder_internal(const char *dir, cpid_t cpid, BOOL
 	return xbresult;
 }
 
-BOOL exmdb_client_local::get_search_criteria(const char *dir, uint64_t folder_id, uint32_t *search_status, RESTRICTION **prestriction, LONGLONG_ARRAY *folder_ids)
+BOOL exmdb_client_local::get_search_criteria(const char *dir, uint64_t folder_id, uint32_t *search_status, RESTRICTION **prestriction, EID_ARRAY *folder_ids)
 {
 	BOOL xb_private;
 
@@ -424,7 +424,7 @@ BOOL exmdb_client_local::get_search_criteria(const char *dir, uint64_t folder_id
 	return xbresult;
 }
 
-BOOL exmdb_client_local::set_search_criteria(const char *dir, cpid_t cpid, uint64_t folder_id, uint32_t search_flags, const RESTRICTION *prestriction, const LONGLONG_ARRAY *pfolder_ids, BOOL *b_result)
+BOOL exmdb_client_local::set_search_criteria(const char *dir, cpid_t cpid, uint64_t folder_id, uint32_t search_flags, const RESTRICTION *prestriction, const EID_ARRAY *pfolder_ids, BOOL *b_result)
 {
 	BOOL xb_private;
 
@@ -634,7 +634,7 @@ BOOL exmdb_client_local::sum_table(const char *dir, uint32_t table_id, uint32_t
 	return xbresult;
 }
 
-BOOL exmdb_client_local::query_table(const char *dir, const char *username, cpid_t cpid, uint32_t table_id, const PROPTAG_ARRAY *pproptags, uint32_t start_pos, int32_t row_needed, TARRAY_SET *set)
+BOOL exmdb_client_local::query_table(const char *dir, const char *username, cpid_t cpid, uint32_t table_id, proptag_cspan pproptags, uint32_t start_pos, int32_t row_needed, TARRAY_SET *set)
 {
 	BOOL xb_private;
 
@@ -648,7 +648,7 @@ BOOL exmdb_client_local::query_table(const char *dir, const char *username, cpid
 	return xbresult;
 }
 
-BOOL exmdb_client_local::match_table(const char *dir, const char *username, cpid_t cpid, uint32_t table_id, BOOL b_forward, uint32_t start_pos, const RESTRICTION *pres, const PROPTAG_ARRAY *pproptags, int32_t *position, TPROPVAL_ARRAY *propvals)
+BOOL exmdb_client_local::match_table(const char *dir, const char *username, cpid_t cpid, uint32_t table_id, BOOL b_forward, uint32_t start_pos, const RESTRICTION *pres, proptag_cspan pproptags, int32_t *position, TPROPVAL_ARRAY *propvals)
 {
 	BOOL xb_private;
 
@@ -676,7 +676,7 @@ BOOL exmdb_client_local::locate_table(const char *dir, uint32_t table_id, uint64
 	return xbresult;
 }
 
-BOOL exmdb_client_local::read_table_row(const char *dir, const char *username, cpid_t cpid, uint32_t table_id, const PROPTAG_ARRAY *pproptags, uint64_t inst_id, uint32_t inst_num, TPROPVAL_ARRAY *propvals)
+BOOL exmdb_client_local::read_table_row(const char *dir, const char *username, cpid_t cpid, uint32_t table_id, proptag_cspan pproptags, uint64_t inst_id, uint32_t inst_num, TPROPVAL_ARRAY *propvals)
 {
 	BOOL xb_private;
 
@@ -1012,7 +1012,7 @@ BOOL exmdb_client_local::get_instance_all_proptags(const char *dir, uint32_t ins
 	return xbresult;
 }
 
-BOOL exmdb_client_local::get_instance_properties(const char *dir, uint32_t size_limit, uint32_t instance_id, const PROPTAG_ARRAY *pproptags, TPROPVAL_ARRAY *propvals)
+BOOL exmdb_client_local::get_instance_properties(const char *dir, uint32_t size_limit, uint32_t instance_id, proptag_cspan pproptags, TPROPVAL_ARRAY *propvals)
 {
 	BOOL xb_private;
 
@@ -1040,7 +1040,7 @@ BOOL exmdb_client_local::set_instance_properties(const char *dir, uint32_t insta
 	return xbresult;
 }
 
-BOOL exmdb_client_local::remove_instance_properties(const char *dir, uint32_t instance_id, const PROPTAG_ARRAY *pproptags, PROBLEM_ARRAY *problems)
+BOOL exmdb_client_local::remove_instance_properties(const char *dir, uint32_t instance_id, proptag_cspan pproptags, PROBLEM_ARRAY *problems)
 {
 	BOOL xb_private;
 
@@ -1194,7 +1194,7 @@ BOOL exmdb_client_local::get_message_instance_attachment_table_all_proptags(cons
 	return xbresult;
 }
 
-BOOL exmdb_client_local::query_message_instance_attachment_table(const char *dir, uint32_t instance_id, const PROPTAG_ARRAY *pproptags, uint32_t start_pos, int32_t row_needed, TARRAY_SET *set)
+BOOL exmdb_client_local::query_message_instance_attachment_table(const char *dir, uint32_t instance_id, proptag_cspan pproptags, uint32_t start_pos, int32_t row_needed, TARRAY_SET *set)
 {
 	BOOL xb_private;
 
@@ -1250,7 +1250,7 @@ BOOL exmdb_client_local::get_message_rcpts(const char *dir, uint64_t message_id,
 	return xbresult;
 }
 
-BOOL exmdb_client_local::get_message_properties(const char *dir, const char *username, cpid_t cpid, uint64_t message_id, const PROPTAG_ARRAY *pproptags, TPROPVAL_ARRAY *propvals)
+BOOL exmdb_client_local::get_message_properties(const char *dir, const char *username, cpid_t cpid, uint64_t message_id, proptag_cspan pproptags, TPROPVAL_ARRAY *propvals)
 {
 	BOOL xb_private;
 
@@ -1292,7 +1292,7 @@ BOOL exmdb_client_local::set_message_read_state(const char *dir, const char *use
 	return xbresult;
 }
 
-BOOL exmdb_client_local::remove_message_properties(const char *dir, cpid_t cpid, uint64_t message_id, const PROPTAG_ARRAY *pproptags)
+BOOL exmdb_client_local::remove_message_properties(const char *dir, cpid_t cpid, uint64_t message_id, proptag_cspan pproptags)
 {
 	BOOL xb_private;
 
@@ -1852,7 +1852,7 @@ BOOL exmdb_client_local::set_maintenance(const char *dir, uint32_t mode)
 	return xbresult;
 }
 
-BOOL exmdb_client_local::autoreply_getprop(const char *dir, cpid_t cpid, const PROPTAG_ARRAY *pproptags, TPROPVAL_ARRAY *propvals)
+BOOL exmdb_client_local::autoreply_getprop(const char *dir, cpid_t cpid, proptag_cspan pproptags, TPROPVAL_ARRAY *propvals)
 {
 	BOOL xb_private;
 
@@ -1880,3 +1880,31 @@ BOOL exmdb_client_local::autoreply_setprop(const char *dir, cpid_t cpid, const T
 	return xbresult;
 }
 
+BOOL exmdb_client_local::read_delegates(const char *dir, uint32_t mode, std::vector<std::string> *userlist)
+{
+	BOOL xb_private;
+
+	if (!exmdb_client_is_local(dir, &xb_private))
+		return exmdb_client_remote::read_delegates(dir, mode, userlist);
+	auto tstart = gromox::tp_now();
+	exmdb_server::build_env(EM_LOCAL | (xb_private ? EM_PRIVATE : 0), dir);
+	auto xbresult = exmdb_server::read_delegates(dir, mode, userlist);
+	lpc_log(xbresult, dir, "read_delegates", tstart, gromox::tp_now());
+	exmdb_server::free_env();
+	return xbresult;
+}
+
+BOOL exmdb_client_local::write_delegates(const char *dir, uint32_t mode, const std::vector<std::string> &userlist)
+{
+	BOOL xb_private;
+
+	if (!exmdb_client_is_local(dir, &xb_private))
+		return exmdb_client_remote::write_delegates(dir, mode, userlist);
+	auto tstart = gromox::tp_now();
+	exmdb_server::build_env(EM_LOCAL | (xb_private ? EM_PRIVATE : 0), dir);
+	auto xbresult = exmdb_server::write_delegates(dir, mode, userlist);
+	lpc_log(xbresult, dir, "write_delegates", tstart, gromox::tp_now());
+	exmdb_server::free_env();
+	return xbresult;
+}
+
