@@ -37,6 +37,7 @@ enum class mlist_priv {
 #define MLIST_RESULT_PRIVIL_SPECIFIED	4
 #define JOIN_WITH_DISPLAYTYPE "LEFT JOIN user_properties AS dt ON u.id=dt.user_id AND dt.proptag=956628995" /* PR_DISPLAY_TYPE_EX */
 #define JOIN_ALTNAMES "LEFT JOIN altnames AS alt ON u.id=alt.user_id AND alt.altname='{0}'"
+#define JOIN_ALIASES "LEFT JOIN aliases AS als ON u.username=als.mainname AND als.aliasname='{0}'"
 
 /*
  * Terminology you might encounter in this file
@@ -339,7 +340,14 @@ bool mysql_plugin::get_user_displayname(const char *username, std::string &out)
 		"LEFT JOIN user_properties AS u2 ON u.id=u2.user_id AND u2.proptag=805371935 " /* PR_DISPLAY_NAME */
 		"LEFT JOIN user_properties AS u3 ON u.id=u3.user_id AND u3.proptag=978255903 " /* PR_NICKNAME */
 		JOIN_ALTNAMES " "
-		"WHERE alt.altname='{0}' LIMIT 2) LIMIT 2",
+		"WHERE alt.altname='{0}' LIMIT 2) UNION"
+		"(SELECT u2.propval_str AS real_name, "
+		"u3.propval_str AS nickname, dt.propval_str AS dtypx FROM users AS u "
+		JOIN_WITH_DISPLAYTYPE " "
+		"LEFT JOIN user_properties AS u2 ON u.id=u2.user_id AND u2.proptag=805371935 " /* PR_DISPLAY_NAME */
+		"LEFT JOIN user_properties AS u3 ON u.id=u3.user_id AND u3.proptag=978255903 " /* PR_NICKNAME */
+		JOIN_ALIASES " "
+		"WHERE als.aliasname='{0}' LIMIT 2) LIMIT 2",
 		q_user);
 	if (!conn->query(qstr))
 		return false;
@@ -516,7 +524,10 @@ bool mysql_plugin::get_user_ids(const char *username, unsigned int *puser_id,
 		" WHERE u.username='{0}' LIMIT 2) UNION"
 		" (SELECT u.id, u.domain_id, dt.propval_str AS dtypx"
 		" FROM users AS u " JOIN_WITH_DISPLAYTYPE " " JOIN_ALTNAMES
-		" WHERE alt.altname='{0}' LIMIT 2) LIMIT 2",
+		" WHERE alt.altname='{0}' LIMIT 2) UNION"
+		" (SELECT u.id, u.domain_id, dt.propval_str AS dtypx "
+		" FROM users AS u " JOIN_WITH_DISPLAYTYPE " " JOIN_ALIASES
+		" WHERE als.aliasname='{0}' LIMIT 2) LIMIT 2",
 		q_user);
 	if (!conn->query(qstr))
 		return false;
