@@ -1,5 +1,5 @@
 // SPDX-License-Identifier: GPL-2.0-only WITH linking exception
-// SPDX-FileCopyrightText: 2020–2025 grommunio GmbH
+// SPDX-FileCopyrightText: 2020–2026 grommunio GmbH
 // This file is part of Gromox.
 #ifdef HAVE_CONFIG_H
 #	include "config.h"
@@ -89,13 +89,10 @@ char g_org_name[256];
 static thread_local const char *g_dir_key;
 static thread_local unsigned int g_env_refcount;
 static thread_local std::unique_ptr<env_context> g_env_key;
-static char g_default_charset[32];
 static char g_submit_command[1024];
 static constexpr char ZCORE_UA[] = PACKAGE_NAME "-zcore " PACKAGE_VERSION;
 
-BOOL common_util_verify_columns_and_sorts(
-	const PROPTAG_ARRAY *pcolumns,
-	const SORTORDER_SET *psort_criteria)
+bool cu_verify_columns_and_sorts(proptag_cspan cols, const SORTORDER_SET *psort_criteria)
 {
 	proptag_t proptag = 0;
 	for (size_t i = 0; i < psort_criteria->count; ++i) {
@@ -106,9 +103,8 @@ BOOL common_util_verify_columns_and_sorts(
 		proptag = PROP_TAG(psort_criteria->psort[i].type, psort_criteria->psort[i].propid);
 		break;
 	}
-	for (size_t i = 0; i < pcolumns->count; ++i)
-		if (pcolumns->pproptag[i] & MV_INSTANCE &&
-		    proptag != pcolumns->pproptag[i])
+	for (const auto t : cols)
+		if (t & MV_INSTANCE && proptag != t)
 			return FALSE;
 	return TRUE;
 }
@@ -120,8 +116,7 @@ bool cu_extract_delegator(message_object *pmessage, std::string &username)
 	static constexpr proptag_t proptag_buff[] =
 		{PR_SENT_REPRESENTING_ADDRTYPE, PR_SENT_REPRESENTING_EMAIL_ADDRESS,
 		PR_SENT_REPRESENTING_SMTP_ADDRESS, PR_SENT_REPRESENTING_ENTRYID};
-	static constexpr PROPTAG_ARRAY tmp_proptags = {std::size(proptag_buff), deconst(proptag_buff)};
-	if (!pmessage->get_properties(&tmp_proptags, &tmp_propvals))
+	if (!pmessage->get_properties(proptag_buff, &tmp_propvals))
 		return FALSE;	
 	if (0 == tmp_propvals.count) {
 		username.clear();
@@ -165,12 +160,8 @@ static int cu_test_delegate_perm_MD(const char *account,
     const char *maildir, bool send_as) try
 {
 	std::vector<std::string> delegate_list;
-	auto path = maildir + std::string(send_as ? "/config/sendas.txt" : "/config/delegates.txt");
-	auto ret = read_file_by_line(path.c_str(), delegate_list);
-	if (ret != 0 && ret != ENOENT) {
-		mlog(LV_WARN, "W-2057: %s: %s", path.c_str(), strerror(ret));
-		return ret;
-	}
+	if (!exmdb_client->read_delegates(maildir, send_as, &delegate_list))
+		return -1;
 	for (const auto &d : delegate_list)
 		if (strcasecmp(d.c_str(), account) == 0)
 			return 1;
@@ -206,6 +197,8 @@ repr_grant cu_get_delegate_perm_AA(const char *account, const char *repr)
 	sql_meta_result mres;
 	if (mysql_adaptor_meta(repr, WANTPRIV_METAONLY, mres) != 0)
 		return repr_grant::error;
+	if (strcasecmp(account, mres.username.c_str()) == 0)
+		return repr_grant::send_as;
 	return cu_get_delegate_perm_MD(account, mres.maildir.c_str());
 }
 
@@ -247,12 +240,12 @@ void common_util_remove_propvals(TPROPVAL_ARRAY *parray, proptag_t proptag)
 	}
 }
 
-void common_util_reduce_proptags(PROPTAG_ARRAY *pproptags_minuend,
-	const PROPTAG_ARRAY *pproptags_subtractor)
+void cu_reduce_proptags(PROPTAG_ARRAY *pproptags_minuend,
+    proptag_cspan pproptags_subtractor)
 {
-	for (unsigned int j = 0; j < pproptags_subtractor->count; ++j) {
+	for (const auto t : pproptags_subtractor) {
 		for (unsigned int i = 0; i < pproptags_minuend->count; ++i) {
-			if (pproptags_subtractor->pproptag[j] != pproptags_minuend->pproptag[i])
+			if (t != pproptags_minuend->pproptag[i])
 				continue;
 			pproptags_minuend->count--;
 			if (i < pproptags_minuend->count)
@@ -268,7 +261,7 @@ void common_util_reduce_proptags(PROPTAG_ARRAY *pproptags_minuend,
 BOOL common_util_essdn_to_uid(const char *pessdn, int *puid)
 {
 	char tmp_essdn[1024];
-	auto tmp_len = snprintf(tmp_essdn, std::size(tmp_essdn),
+	auto tmp_len = gx_snprintf(tmp_essdn, std::size(tmp_essdn),
 	               "/o=%s/" EAG_RCPTS "/cn=", g_org_name);
 	if (strncasecmp(pessdn, tmp_essdn, tmp_len) != 0 ||
 	    pessdn[tmp_len+16] != '-')
@@ -281,7 +274,7 @@ BOOL common_util_essdn_to_ids(const char *pessdn,
 	int *pdomain_id, int *puser_id)
 {
 	char tmp_essdn[1024];
-	auto tmp_len = snprintf(tmp_essdn, std::size(tmp_essdn),
+	auto tmp_len = gx_snprintf(tmp_essdn, std::size(tmp_essdn),
 	               "/o=%s/" EAG_RCPTS "/cn=", g_org_name);
 	if (strncasecmp(pessdn, tmp_essdn, tmp_len) != 0 ||
 	    pessdn[tmp_len+16] != '-')
@@ -291,9 +284,8 @@ BOOL common_util_essdn_to_ids(const char *pessdn,
 	return TRUE;	
 }
 
-BOOL common_util_exmdb_locinfo_from_string(
-	const char *loc_string, uint8_t *ptype,
-	int *pdb_id, uint64_t *peid)
+bool common_util_exmdb_locinfo_from_string(const char *loc_string,
+    uint8_t *ptype, int *pdb_id, eid_t *peid)
 {
 	int tmp_len;
 	uint64_t tmp_val;
@@ -328,12 +320,11 @@ BOOL common_util_exmdb_locinfo_from_string(
 	return TRUE;
 }
 
-void common_util_init(const char *org_name, const char *default_charset,
+void common_util_init(const char *org_name,
     unsigned int max_rcpt, size_t max_mail_len,
     unsigned int max_rule_len, std::string &&smtp_url, const char *submit_command)
 {
 	gx_strlcpy(g_org_name, org_name, std::size(g_org_name));
-	gx_strlcpy(g_default_charset, default_charset, std::size(g_default_charset));
 	g_max_rcpt = max_rcpt;
 	g_max_mail_len = max_mail_len;
 	g_max_rule_len = g_max_extrule_len = max_rule_len;
@@ -537,8 +528,7 @@ uint16_t common_util_get_messaging_entryid_type(BINARY bin)
 	return folder_type;
 }
 
-BOOL cu_entryid_to_fid(BINARY bin,
-	BOOL *pb_private, int *pdb_id, uint64_t *pfolder_id)
+bool cu_entryid_to_fid(BINARY bin, BOOL *pb_private, int *pdb_id, eid_t *pfolder_id)
 {
 	uint16_t replid;
 	EXT_PULL ext_pull;
@@ -577,8 +567,8 @@ BOOL cu_entryid_to_fid(BINARY bin,
 	}
 }
 
-BOOL cu_entryid_to_mid(BINARY bin, BOOL *pb_private,
-	int *pdb_id, uint64_t *pfolder_id, uint64_t *pmessage_id)
+bool cu_entryid_to_mid(BINARY bin, BOOL *pb_private,
+    int *pdb_id, eid_t *pfolder_id, eid_t *pmessage_id)
 {
 	uint16_t replid;
 	EXT_PULL ext_pull;
@@ -1064,12 +1054,10 @@ static BOOL common_util_get_propname(propid_t propid, PROPERTY_NAME **pppropname
 ec_error_t cu_send_message(store_object *pstore, message_object *msg,
     const char *ev_from) try
 {
-	uint64_t message_id = msg->get_id();
+	eid_t message_id = msg->get_id();
 	void *pvalue;
 	BOOL b_private, b_partial = false;
 	int account_id;
-	uint64_t new_id;
-	uint64_t folder_id;
 	TAGGED_PROPVAL *ppropval;
 	MESSAGE_CONTENT *pmsgctnt;
 	
@@ -1112,19 +1100,20 @@ ec_error_t cu_send_message(store_object *pstore, message_object *msg,
 		return MAPI_E_NO_RECIPIENTS;
 	}
 
-	auto body_type = get_override_format(*pmsgctnt);
 	common_util_set_dir(pstore->get_dir());
-	/* try to avoid TNEF message */
 	MAIL imail;
-	if (!oxcmail_export(pmsgctnt, log_id.c_str(), false, body_type,
-	    &imail, common_util_alloc, common_util_get_propids,
-	    common_util_get_propname))
+	oxcmail_converter cvt;
+	cvt.log_id = log_id.c_str();
+	cvt.alloc = common_util_alloc;
+	cvt.get_propids = common_util_get_propids;
+	cvt.get_propname = common_util_get_propname;
+	cvt.use_format_override(*pmsgctnt);
+	if (!cvt.mapi_to_inet(*pmsgctnt, imail))
 		return ecError;
 
 	imail.set_header("X-Mailer", ZCORE_UA);
 	if (zcore_backfill_transporthdr) {
-		std::unique_ptr<MESSAGE_CONTENT, mc_delete> rmsg(oxcmail_import(nullptr,
-			"UTC", &imail, common_util_alloc, common_util_get_propids));
+		auto rmsg = cvt.inet_to_mapi(imail);
 		if (rmsg != nullptr) {
 			for (auto tag : {PR_TRANSPORT_MESSAGE_HEADERS, PR_TRANSPORT_MESSAGE_HEADERS_A}) {
 				auto th = rmsg->proplist.get<const char>(tag);
@@ -1154,6 +1143,7 @@ ec_error_t cu_send_message(store_object *pstore, message_object *msg,
 	common_util_remove_propvals(&pmsgctnt->proplist, PidTagSentMailSvrEID);
 	auto ptarget = pmsgctnt->proplist.get<BINARY>(PR_TARGET_ENTRYID);
 	if (NULL != ptarget) {
+		eid_t folder_id{}, new_id{};
 		if (!cu_entryid_to_mid(*ptarget,
 		    &b_private, &account_id, &folder_id, &new_id))
 			return ecWarnWithErrors;
@@ -1173,6 +1163,7 @@ ec_error_t cu_send_message(store_object *pstore, message_object *msg,
 	if (!exmdb_client->clear_submit(pstore->get_dir(), message_id, false))
 		return ecWarnWithErrors;
 	ptarget = pmsgctnt->proplist.get<BINARY>(PR_SENTMAIL_ENTRYID);
+	eid_t folder_id{};
 	if (ptarget == nullptr || !cu_entryid_to_fid(*ptarget,
 	    &b_private, &account_id, &folder_id))
 		folder_id = rop_util_make_eid_ex(1, PRIVATE_FID_SENT_ITEMS);
@@ -1252,7 +1243,7 @@ static MOVECOPY_ACTION *cu_cvt_from_zmovecopy(const ZMOVECOPY_ACTION &src)
 	if (!cu_entryid_to_fid(src.folder_eid,
 	    &b_private, &db_id, &psvreid->folder_id))
 		return NULL;
-	psvreid->message_id = 0;
+	psvreid->message_id = eid_t(0);
 	psvreid->instance = 0;
 	dst->pfolder_eid = psvreid;
 	return dst;
@@ -1595,20 +1586,19 @@ static EID_ARRAY *common_util_load_folder_messages(store_object *pstore,
 	    nullptr, nullptr, &table_id, &row_count))
 		return NULL;	
 	static constexpr proptag_t tmp_proptag[] = {PidTagMid};
-	static constexpr PROPTAG_ARRAY proptags = {std::size(tmp_proptag), deconst(tmp_proptag)};
 	if (!exmdb_client->query_table(pstore->get_dir(), nullptr, CP_ACP,
-	    table_id, &proptags, 0, row_count, &tmp_set))
+	    table_id, tmp_proptag, 0, row_count, &tmp_set))
 		return NULL;	
 	exmdb_client->unload_table(pstore->get_dir(), table_id);
 	pmessage_ids = cu_alloc<EID_ARRAY>();
 	if (pmessage_ids == nullptr)
 		return NULL;
 	pmessage_ids->count = 0;
-	pmessage_ids->pids = cu_alloc<uint64_t>(tmp_set.count);
+	pmessage_ids->pids = cu_alloc<eid_t>(tmp_set.count);
 	if (pmessage_ids->pids == nullptr)
 		return NULL;
 	for (size_t i = 0; i < tmp_set.count; ++i) {
-		auto pmid = tmp_set.pparray[i]->get<uint64_t>(PidTagMid);
+		auto pmid = tmp_set.pparray[i]->get<eid_t>(PidTagMid);
 		if (pmid == nullptr)
 			return NULL;
 		pmessage_ids->pids[pmessage_ids->count++] = *pmid;
@@ -1632,7 +1622,7 @@ ec_error_t cu_remote_copy_folder(store_object *src_store, uint64_t folder_id,
 	    folder_id, &tmp_proptags))
 		return ecError;
 	if (!exmdb_client->get_folder_properties(src_store->get_dir(), CP_ACP,
-	    folder_id, &tmp_proptags, &tmp_propvals))
+	    folder_id, tmp_proptags, &tmp_propvals))
 		return ecError;
 	if (new_name != nullptr) {
 		auto err = cu_set_propval(&tmp_propvals, PR_DISPLAY_NAME, new_name);
@@ -1668,9 +1658,8 @@ ec_error_t cu_remote_copy_folder(store_object *src_store, uint64_t folder_id,
 		return ecError;
 
 	static constexpr proptag_t xb_proptag[] = {PidTagFolderId};
-	static constexpr PROPTAG_ARRAY xb_proptags = {std::size(xb_proptag), deconst(xb_proptag)};
 	if (!exmdb_client->query_table(src_store->get_dir(), nullptr, CP_ACP,
-	    table_id, &xb_proptags, 0, row_count, &tmp_set))
+	    table_id, xb_proptag, 0, row_count, &tmp_set))
 		return ecError;
 	exmdb_client->unload_table(src_store->get_dir(), table_id);
 	for (size_t i = 0; i < tmp_set.count; ++i) {
@@ -1708,13 +1697,16 @@ BOOL common_util_message_to_rfc822(store_object *pstore, uint64_t inst_id,
 		ppropval[pmsgctnt->proplist.count++].pvalue = &cpid;
 		pmsgctnt->proplist.ppropval = ppropval;
 	}
-	auto body_type = get_override_format(*pmsgctnt);
 	common_util_set_dir(pstore->get_dir());
-	/* try to avoid TNEF message */
 	auto log_id = pstore->get_dir() + ":i"s + std::to_string(inst_id);
 	MAIL imail;
-	if (!oxcmail_export(pmsgctnt, log_id.c_str(), false, body_type, &imail,
-	    common_util_alloc, common_util_get_propids, common_util_get_propname))
+	oxcmail_converter cvt;
+	cvt.log_id = log_id.c_str();
+	cvt.alloc = common_util_alloc;
+	cvt.get_propids = common_util_get_propids;
+	cvt.get_propname = common_util_get_propname;
+	cvt.use_format_override(*pmsgctnt);
+	if (!cvt.mapi_to_inet(*pmsgctnt, imail))
 		return FALSE;	
 	auto mail_len = imail.get_length();
 	if (mail_len < 0)
@@ -1758,31 +1750,21 @@ static void zc_unwrap_clearsigned(MAIL &ma) try
 	mlog(LV_ERR, "%s: ENOMEM", __func__);
 }
 
-MESSAGE_CONTENT *cu_rfc822_to_message(store_object *pstore,
+std::unique_ptr<message_content, mc_delete> cu_rfc822_to_message(store_object *pstore,
     unsigned int mxf_flags, /* effective-moved-from */ BINARY *peml_bin)
 {
-	char charset[32];
-	auto pinfo = zs_get_info();
 	MAIL imail;
 	if (!imail.refonly_parse(peml_bin->pc, peml_bin->cb))
 		return NULL;
 	if (mxf_flags & MXF_UNWRAP_SMIME_CLEARSIGNED)
 		zc_unwrap_clearsigned(imail);
-	auto c = lang_to_charset(pinfo->get_lang());
-	if (c != nullptr && *c != '\0')
-		gx_strlcpy(charset, c, std::size(charset));
-	else
-		strcpy(charset, g_default_charset);
-	sql_meta_result mres;
-	auto tmzone = mysql_adaptor_meta(pinfo->get_username(),
-	              WANTPRIV_METAONLY, mres) == 0 ?
-	              mres.timezone.c_str() : nullptr;
-	if (*znul(tmzone) == '\0')
-		tmzone = common_util_get_default_timezone();
 	common_util_set_dir(pstore->get_dir());
-	auto pmsgctnt = oxcmail_import(charset, tmzone, &imail,
-	                common_util_alloc, common_util_get_propids_create);
-	return pmsgctnt;
+	oxcmail_converter cvt;
+	cvt.alloc = common_util_alloc;
+	cvt.get_propids = common_util_get_propids_create;
+	if (mxf_flags & MXF_ADD_RCVD_TIMESTAMP)
+		cvt.add_rcvd_timestamp = true;
+	return cvt.inet_to_mapi(imail);
 }
 
 BOOL common_util_message_to_ical(store_object *pstore, uint64_t message_id,
@@ -1799,8 +1781,13 @@ BOOL common_util_message_to_ical(store_object *pstore, uint64_t message_id,
 		return FALSE;
 	common_util_set_dir(dir);
 	auto log_id = dir + ":m"s + std::to_string(message_id);
-	if (!oxcical_export(pmsgctnt, log_id.c_str(), ical, g_org_name,
-	    common_util_alloc, common_util_get_propids, mysql_adaptor_userid_to_name)) {
+	oxcical_converter cvt;
+	cvt.log_id = log_id.c_str();
+	cvt.org_name = g_org_name;
+	cvt.alloc = common_util_alloc;
+	cvt.get_propids = common_util_get_propids;
+	cvt.id2user = mysql_adaptor_userid_to_name;
+	if (!cvt.mapi_to_ical(*pmsgctnt, ical)) {
 		mlog(LV_ERR, "E-2202: oxcical_export %s failed", log_id.c_str());
 		return FALSE;
 	}
@@ -1820,13 +1807,6 @@ BOOL common_util_message_to_ical(store_object *pstore, uint64_t message_id,
 message_ptr cu_ical_to_message(store_object *pstore, const BINARY *pical_bin) try
 {
 	ical ical;
-	auto pinfo = zs_get_info();
-	sql_meta_result mres;
-	auto tmzone = mysql_adaptor_meta(pinfo->get_username(),
-	              WANTPRIV_METAONLY, mres) == 0 ?
-	              mres.timezone.c_str() : nullptr;
-	if (*znul(tmzone) == '\0')
-		tmzone = common_util_get_default_timezone();
 	auto pbuff = cu_alloc<char>(pical_bin->cb + 1);
 	if (pbuff == nullptr)
 		return nullptr;
@@ -1835,8 +1815,12 @@ message_ptr cu_ical_to_message(store_object *pstore, const BINARY *pical_bin) tr
 	if (!ical.load_from_str_move(pbuff))
 		return NULL;
 	common_util_set_dir(pstore->get_dir());
-	return oxcical_import_single(tmzone, ical, common_util_alloc,
-	       common_util_get_propids_create, common_util_username_to_entryid);
+
+	oxcical_converter cvt;
+	cvt.alloc = common_util_alloc;
+	cvt.get_propids = common_util_get_propids_create;
+	cvt.username_to_entryid = common_util_username_to_entryid;
+	return cvt.ical_to_mapi_single(ical);
 } catch (const std::bad_alloc &) {
 	mlog(LV_ERR, "%s: ENOMEM", __func__);
 	return nullptr;
@@ -1845,21 +1829,16 @@ message_ptr cu_ical_to_message(store_object *pstore, const BINARY *pical_bin) tr
 ec_error_t cu_ical_to_message2(store_object *store, char *ical_data,
     std::vector<message_ptr> &msgvec) try
 {
-	auto info = zs_get_info();
-	sql_meta_result mres;
-	auto tmzone = mysql_adaptor_meta(info->get_username(),
-	              WANTPRIV_METAONLY, mres) == 0 ?
-	              mres.timezone.c_str() : nullptr;
-	if (*znul(tmzone) == '\0')
-		tmzone = common_util_get_default_timezone();
-
 	ical icobj;
 	if (!icobj.load_from_str_move(ical_data))
 		return ecError;
 	common_util_set_dir(store->get_dir());
-	return oxcical_import_multi(tmzone, icobj, common_util_alloc,
-	       common_util_get_propids_create,
-	       common_util_username_to_entryid, msgvec);
+
+	oxcical_converter cvt;
+	cvt.alloc = common_util_alloc;
+	cvt.get_propids = common_util_get_propids_create;
+	cvt.username_to_entryid = common_util_username_to_entryid;
+	return cvt.ical_to_mapi_multi(icobj, msgvec);
 } catch (const std::bad_alloc &) {
 	mlog(LV_ERR, "%s: ENOMEM", __func__);
 	return ecServerOOM;
@@ -1877,26 +1856,29 @@ BOOL common_util_message_to_vcf(message_object *pmessage, BINARY *pvcf_bin)
 	    message_id, &pmsgctnt) || pmsgctnt == nullptr)
 		return FALSE;
 	common_util_set_dir(pstore->get_dir());
-	auto log_id = pstore->get_dir() + ":m"s + std::to_string(rop_util_get_gc_value(message_id));
+
+	std::string cvt_log_id = pstore->get_dir() + ":m"s + std::to_string(rop_util_get_gc_value(message_id));
+	oxvcard_converter cvt;
+	cvt.log_id = cvt_log_id.c_str();
+	cvt.get_propids = common_util_get_propids;
 	vcard vcard;
-	if (!oxvcard_export(pmsgctnt, log_id.c_str(), vcard, common_util_get_propids))
+	if (!cvt.mapi_to_vcard(*pmsgctnt, vcard))
 		return FALSE;
-	pvcf_bin->pv = common_util_alloc(VCARD_MAX_BUFFER_LEN);
+	std::string vcf_out;
+	if (!vcard.serialize(vcf_out))
+		return FALSE;	
+	pvcf_bin->cb = vcf_out.size();
+	pvcf_bin->pv = common_util_alloc(pvcf_bin->cb);
 	if (pvcf_bin->pv == nullptr)
 		return FALSE;
-	if (!vcard.serialize(pvcf_bin->pc, VCARD_MAX_BUFFER_LEN))
-		return FALSE;	
-	pvcf_bin->cb = strlen(pvcf_bin->pc);
-	if (!pmessage->write_message(pmsgctnt))
+	memcpy(pvcf_bin->pv, vcf_out.c_str(), vcf_out.size());
+	if (!pmessage->write_message(*pmsgctnt))
 		/* ignore */;
 	return TRUE;
 }
 	
-MESSAGE_CONTENT *common_util_vcf_to_message(store_object *pstore,
-    const BINARY *pvcf_bin)
+message_ptr common_util_vcf_to_message(store_object *pstore, const BINARY *pvcf_bin)
 {
-	MESSAGE_CONTENT *pmsgctnt;
-	
 	auto pbuff = cu_alloc<char>(pvcf_bin->cb + 1);
 	if (pbuff == nullptr)
 		return nullptr;
@@ -1907,8 +1889,10 @@ MESSAGE_CONTENT *common_util_vcf_to_message(store_object *pstore,
 	if (ret != ecSuccess)
 		return nullptr;
 	common_util_set_dir(pstore->get_dir());
-	pmsgctnt = oxvcard_import(&vcard, common_util_get_propids_create);
-	return pmsgctnt;
+
+	oxvcard_converter cvt;
+	cvt.get_propids = common_util_get_propids_create;
+	return cvt.vcard_to_mapi(vcard);
 }
 
 ec_error_t cu_vcf_to_message2(store_object *store, char *vcf_data,
@@ -1919,8 +1903,11 @@ ec_error_t cu_vcf_to_message2(store_object *store, char *vcf_data,
 	if (ret != ecSuccess)
 		return ret;
 	common_util_set_dir(store->get_dir());
+
+	oxvcard_converter cvt;
+	cvt.get_propids = common_util_get_propids_create;
 	for (const auto &vcard : cardvec) {
-		message_ptr mc(oxvcard_import(&vcard, common_util_get_propids_create));
+		auto mc = cvt.vcard_to_mapi(vcard);
 		if (mc == nullptr)
 			return ecError;
 		msgvec.push_back(std::move(mc));
@@ -1953,9 +1940,9 @@ void *cu_read_storenamedprop(const char *dir, const GUID &guid,
 	    name_rsp.size() != name_req.size() || name_rsp[0] == 0)
 		return nullptr;
 	auto proptag = PROP_TAG(proptype, name_rsp[0]);
-	const PROPTAG_ARRAY tags = {1, deconst(&proptag)};
 	TPROPVAL_ARRAY values{};
-	if (!exmdb_client->get_store_properties(dir, CP_ACP, &tags, &values))
+	if (!exmdb_client->get_store_properties(dir, CP_ACP,
+	    {&proptag, 1}, &values))
 		return nullptr;
 	return values.getval(proptag);
 }
