@@ -1,5 +1,5 @@
 // SPDX-License-Identifier: GPL-2.0-only WITH linking exception
-// SPDX-FileCopyrightText: 2020–2025 grommunio GmbH
+// SPDX-FileCopyrightText: 2020–2026 grommunio GmbH
 // This file is part of Gromox.
 #include <cerrno>
 #include <csignal>
@@ -92,13 +92,11 @@ static constexpr cfg_directive zcore_cfg_defaults[] = {
 	{"address_table_size", "3000", CFG_SIZE, "1"},
 	{"config_file_path", PKGSYSCONFDIR "/zcore:" PKGSYSCONFDIR},
 	{"data_file_path", PKGDATADIR "/zcore:" PKGDATADIR},
-	{"default_charset", "utf-8"},
 	{"mail_max_length", "64M", CFG_SIZE, "1"},
 	{"mailbox_ping_interval", "5min", CFG_TIME, "1min", "1h"},
 	{"max_ext_rule_length", "510K", CFG_SIZE, "1"},
 	{"max_rcpt_num", "256", CFG_SIZE, "1"},
 	{"notify_stub_threads_num", "10", CFG_SIZE, "1", "100"},
-	{"oxcical_allday_ymd", "1", CFG_BOOL},
 	{"rpc_proxy_connection_num", "10", CFG_SIZE, "1", "100"},
 	{"submit_command", "/usr/bin/php " PKGDATADIR "/submit.php"},
 	{"user_cache_interval", "1h", CFG_TIME, "1min", "1day"},
@@ -146,7 +144,6 @@ static bool zcore_reload_config(std::shared_ptr<CONFIG_FILE> gxcfg = nullptr,
 		pconfig->get_ll("zcore_log_level"),
 		pconfig->get_value("running_identity"));
 	g_zrpc_debug = pconfig->get_ll("zrpc_debug");
-	g_oxcical_allday_ymd = pconfig->get_ll("oxcical_allday_ymd");
 	zcore_max_obh_per_session = pconfig->get_ll("zcore_max_obh_per_session");
 	return true;
 }
@@ -277,6 +274,7 @@ int main(int argc, char **argv)
 		return EXIT_FAILURE;
 	if (!zcore_reload_config(gxconfig, pconfig))
 		return EXIT_FAILURE;
+	setup_utf8_locale();
 
 	unsigned int threads_num = pconfig->get_ll("zcore_threads_num");
 	mlog(LV_INFO, "system: connection threads number is %d", threads_num);
@@ -319,7 +317,6 @@ int main(int argc, char **argv)
 	mlog(LV_NOTICE, "system: SMTP server is %s", smtp_url.c_str());
 	
 	common_util_init(g_config_file->get_value("x500_org_name"),
-		g_config_file->get_value("default_charset"),
 		max_rcpt, max_length, max_rule_len, std::move(smtp_url),
 		g_config_file->get_value("submit_command"));
 	
