@@ -81,10 +81,15 @@ class GX_EXPORT wrapfd {
 	int m_fd = -1;
 };
 
-extern GX_EXPORT std::string iconvtext(const char *, size_t, const char *from, const char *to);
+enum {
+	ICONVTEXT_TRANSLIT = 0x1U,
+};
+
+extern GX_EXPORT std::string iconvtext(std::string_view, const char *from, const char *to, unsigned int flags = 0);
 extern GX_EXPORT pid_t popenfd(const char *const *, int *, int *, int *, const char *const *);
 extern GX_EXPORT int feed_w3m(std::string_view, const char *in_cset, std::string &out);
-extern GX_EXPORT std::vector<std::string> gx_split(const std::string_view &, char sep);
+extern GX_EXPORT std::vector<std::string> gx_split(std::string_view, char sep);
+extern GX_EXPORT std::vector<std::string> gx_split_ws(std::string_view, char sep);
 extern GX_EXPORT DIR_mp opendir_sd(const char *, const char *);
 extern GX_EXPORT std::unique_ptr<FILE, file_deleter> fopen_sd(const char *, const char *);
 extern GX_EXPORT std::string resource_parse_stcode_line(const char *);
