@@ -1,5 +1,5 @@
 #pragma once
-#if defined(COMPILE_DIAG) || defined(DEBUG_UMTA)
+#ifdef COMPILE_DIAG
 #	include <cassert>
 #endif
 #include <cstdint>
@@ -12,7 +12,6 @@
 #include <gromox/common_types.hpp>
 #include <gromox/ext_buffer.hpp>
 #include <gromox/mapidefs.h>
-#include <gromox/range_set.hpp>
 
 /* https://learn.microsoft.com/en-us/office/client-developer/outlook/mapi/wrapstoreentryid */
 struct GX_EXPORT STORE_ENTRYID {
@@ -63,9 +62,8 @@ struct GX_EXPORT EXT_MOVECOPY_ACTION {
 
 /* reply or OOF action */
 struct GX_EXPORT REPLY_ACTION {
-	uint64_t template_folder_id;
-	uint64_t template_message_id;
-	GUID template_guid;
+	eid_t template_folder_id{}, template_message_id{};
+	GUID template_guid{};
 
 	std::string repr() const;
 };
@@ -238,12 +236,12 @@ struct GX_EXPORT PROBLEM_ARRAY {
 	I_BEGIN_END(pproblem, count);
 	bool have_index(unsigned int) const;
 	PROBLEM_ARRAY &operator+=(PROBLEM_ARRAY &&);
-	inline void emplace_back(unsigned int i, uint32_t tag, uint32_t err) {
+	inline void emplace_back(unsigned int i, gromox::proptag_t tag, uint32_t err) {
 		pproblem[count++] = PROPERTY_PROBLEM{static_cast<uint16_t>(i), tag, err};
 	}
 	void transform(const std::vector<uint16_t> &);
-	size_t indexof(uint32_t tag) const;
-	inline bool has(uint32_t tag) const { return indexof(tag) != npos; }
+	size_t indexof(gromox::proptag_t) const;
+	inline bool has(gromox::proptag_t t) const { return indexof(t) != npos; }
 	static constexpr size_t npos = -1;
 };
 
@@ -575,7 +573,10 @@ static constexpr unsigned int
 	wed = 1U << 3, /* rdfWed */
 	thu = 1U << 4, /* rdfThu */
 	fri = 1U << 5, /* rdfFri */
-	sat = 1U << 6; /* rdfSat */
+	sat = 1U << 6, /* rdfSat */
+	weekday = mon | tue | wed | thu | fri,
+	weekend = sat | sun,
+	day = weekday | weekend;
 }
 
 #define RECURRENCENUM_FIRST							0x00000001
@@ -591,14 +592,6 @@ union PATTERNTYPE_SPECIFIC {
 	} monthnth;
 };
 
-#define FIRSTDOW_SUNDAY								0x00000000
-#define FIRSTDOW_MONDAY								0x00000001
-#define FIRSTDOW_TUESDAY							0x00000002
-#define FIRSTDOW_WEDNESDAY							0x00000003
-#define FIRSTDOW_THURSDAY							0x00000004
-#define FIRSTDOW_FRIDAY								0x00000005
-#define FIRSTDOW_SATURDAY							0x00000006
-
 enum {
 	ENDDATE_MISSING        = 0x5ae980df,
 	ENDDATE_MISSING_RDELTA = 0x5ae980e1,
@@ -700,8 +693,9 @@ struct GX_EXPORT GLOBALOBJECTID {
 
 struct GX_EXPORT EID_ARRAY {
 	uint32_t count;
-	uint64_t *pids;
+	eid_t *pids;
 	I_BEGIN_END(pids, count);
+	void emplace_back(eid_t t) { pids[count++] = t; }
 };
 
 using INDEX_ARRAY = PROPTAG_ARRAY;
@@ -722,188 +716,6 @@ struct GX_EXPORT EXTENDED_ERROR {
 	BINARY *paux_bytes;
 };
 
-using REPLICA_MAPPING = BOOL (*)(BOOL, void *, uint16_t *, GUID *);
-using REPLIST_ENUM = void (*)(void *, uint16_t);
-using REPLICA_ENUM = void (*)(void *, uint64_t);
-
-struct GX_EXPORT repl_node {
-	repl_node() = default;
-	repl_node(uint16_t r) : replid(r) {}
-	repl_node(const GUID &g) : replguid(g) {}
-
-	union {
-		uint16_t replid;
-		GUID replguid{};
-	};
-	using range_list_t = gromox::range_set<uint64_t>;
-	range_list_t range_list; /* GLOBSET */
-};
-
-class GX_EXPORT idset {
-	public:
-	enum class type : uint8_t {
-		id_packed   = 0x41, id_loose   = 0x42,
-		guid_packed = 0x81, guid_loose = 0x82,
-	};
-
-	idset(idset::type t) : repl_type(t) {}
-	static std::unique_ptr<idset> create(idset::type);
-	bool packed() const { return static_cast<uint8_t>(repl_type) & 0x1; }
-
-	BOOL register_mapping(void *logon_obj, REPLICA_MAPPING);
-	void clear() { repl_list.clear(); }
-	bool empty() const { return repl_list.empty(); }
-	BOOL append(uint64_t eid);
-	BOOL append_range(uint16_t replid, uint64_t low_value, uint64_t high_value);
-	void remove(uint64_t eid);
-	BOOL concatenate(const idset *set_src);
-	bool contains(uint64_t eid) const;
-	BINARY *serialize();
-	BINARY *serialize_replid() const;
-	BINARY *serialize_replguid();
-	BOOL deserialize(const BINARY &);
-	/* convert from deserialize idset into serialize idset */
-	BOOL convert();
-	/* get maximum of first range in idset for specified replid */
-	BOOL get_repl_first_max(uint16_t replid, uint64_t *eid);
-	BOOL enum_replist(void *param, REPLIST_ENUM);
-	BOOL enum_repl(uint16_t replid, void *param, REPLICA_ENUM);
-	inline const std::vector<repl_node> &get_repl_list() const { return repl_list; }
-	void dump(FILE * = nullptr) const;
-#ifdef COMPILE_DIAG
-	inline size_t nelem() const {
-		size_t x = 0;
-		for (const auto &i : repl_list)
-			x += i.range_list.nelem();
-		return x;
-	}
-#endif
-
-	private:
-	std::pair<bool, repl_node::range_list_t *> get_range_by_id(uint16_t);
-
-	void *pparam = nullptr;
-	REPLICA_MAPPING mapping = nullptr;
-	idset::type repl_type = idset::type::id_packed;
-	/* If @repl_type is guid_packed, repl_nodes are REPLGUID_NODE. */
-	std::vector<repl_node> repl_list;
-};
-
-enum class db_notify_type : uint8_t {
-	new_mail = 1, folder_created, message_created, link_created,
-	folder_deleted, message_deleted, link_deleted, folder_modified,
-	message_modified, folder_moved, message_moved, folder_copied,
-	message_copied, search_completed, hiertbl_changed, cttbl_changed,
-	srchtbl_changed, hiertbl_row_added, cttbl_row_added, srchtbl_row_added,
-	hiertbl_row_deleted, cttbl_row_deleted, srchtbl_row_deleted,
-	hiertbl_row_modified, cttbl_row_modified, srchtbl_row_modified,
-};
-
-struct GX_EXPORT DB_NOTIFY {
-	enum db_notify_type type;
-	void *pdata;
-};
-
-struct GX_EXPORT DB_NOTIFY_NEW_MAIL {
-	uint64_t folder_id;
-	uint64_t message_id;
-	uint32_t message_flags;
-	const char *pmessage_class;
-};
-
-struct GX_EXPORT DB_NOTIFY_FOLDER_CREATED {
-	uint64_t folder_id;
-	uint64_t parent_id;
-	PROPTAG_ARRAY proptags;
-};
-
-struct GX_EXPORT DB_NOTIFY_MESSAGE_CREATED {
-	uint64_t folder_id;
-	uint64_t message_id;
-	PROPTAG_ARRAY proptags;
-};
-
-struct GX_EXPORT DB_NOTIFY_LINK_CREATED {
-	uint64_t folder_id;
-	uint64_t message_id;
-	uint64_t parent_id;
-	PROPTAG_ARRAY proptags;
-};
-
-struct GX_EXPORT DB_NOTIFY_FOLDER_DELETED {
-	uint64_t folder_id;
-	uint64_t parent_id;
-};
-
-struct GX_EXPORT DB_NOTIFY_MESSAGE_DELETED {
-	uint64_t folder_id;
-	uint64_t message_id;
-};
-	
-struct GX_EXPORT DB_NOTIFY_LINK_DELETED {
-	uint64_t folder_id;
-	uint64_t message_id;
-	uint64_t parent_id;
-};
-
-struct GX_EXPORT DB_NOTIFY_FOLDER_MODIFIED {
-	uint64_t folder_id;
-	uint64_t parent_id;
-	uint32_t *ptotal;
-	uint32_t *punread;
-	PROPTAG_ARRAY proptags;
-};
-	
-struct GX_EXPORT DB_NOTIFY_MESSAGE_MODIFIED {
-	uint64_t folder_id;
-	uint64_t message_id;
-	PROPTAG_ARRAY proptags;
-};
-
-struct GX_EXPORT DB_NOTIFY_FOLDER_MVCP {
-	uint64_t folder_id;
-	uint64_t parent_id;
-	uint64_t old_folder_id;
-	uint64_t old_parent_id;
-};
-
-struct GX_EXPORT DB_NOTIFY_MESSAGE_MVCP {
-	uint64_t folder_id;
-	uint64_t message_id;
-	uint64_t old_folder_id;
-	uint64_t old_message_id;
-};
-
-struct GX_EXPORT DB_NOTIFY_SEARCH_COMPLETED {
-	uint64_t folder_id;
-};
-
-struct GX_EXPORT DB_NOTIFY_HIERARCHY_TABLE_ROW_MODIFIED {
-	uint64_t row_folder_id;
-	uint64_t after_folder_id;
-};
-using DB_NOTIFY_HIERARCHY_TABLE_ROW_ADDED = DB_NOTIFY_HIERARCHY_TABLE_ROW_MODIFIED;
-
-struct GX_EXPORT DB_NOTIFY_CONTENT_TABLE_ROW_MODIFIED {
-	uint64_t row_folder_id;
-	uint64_t row_message_id;
-	uint64_t row_instance;
-	uint64_t after_folder_id;
-	uint64_t after_row_id;
-	uint64_t after_instance;
-};
-using DB_NOTIFY_CONTENT_TABLE_ROW_ADDED = DB_NOTIFY_CONTENT_TABLE_ROW_MODIFIED;
-
-struct GX_EXPORT DB_NOTIFY_HIERARCHY_TABLE_ROW_DELETED {
-	uint64_t row_folder_id;
-};
-
-struct GX_EXPORT DB_NOTIFY_CONTENT_TABLE_ROW_DELETED {
-	uint64_t row_folder_id;
-	uint64_t row_message_id;
-	uint64_t row_instance;
-};
-
 #define LOGON_FLAG_PRIVATE							0x1
 #define LOGON_FLAG_UNDER_COVER						0x2
 #define LOGON_FLAG_GHOSTED							0x4
