@@ -1102,13 +1102,13 @@ struct GX_EXPORT PROPNAME_ARRAY {
 struct GX_EXPORT proptag_cspan : public std::span<const gromox::proptag_t> {
 	private:
 	using base_t = std::span<const gromox::proptag_t>;
-	static constexpr size_t npos = -1;
 
 	public:
 	using base_t::base_t;
 	size_t indexof(gromox::proptag_t) const;
 	inline bool has(gromox::proptag_t t) const { return indexof(t) != npos; }
 	std::string repr() const;
+	static constexpr size_t npos = -1;
 };
 
 struct GX_EXPORT proptag_span : public std::span<gromox::proptag_t> {
@@ -1138,6 +1138,7 @@ struct GX_EXPORT PROPTAG_ARRAY {
 	inline bool has(gromox::proptag_t t) const { return proptag_cspan(*this).has(t); }
 	inline std::string repr() const { return proptag_cspan(*this).repr(); }
 	void emplace_back(gromox::proptag_t t) { pproptag[count++] = t; }
+	void emplace_back_nd(gromox::proptag_t t) { if (!has(t)) pproptag[count++] = t; }
 
 	uint16_t count;
 	gromox::proptag_t *pproptag;
@@ -1190,10 +1191,9 @@ struct GX_EXPORT STRING_ARRAY {
 };
 
 struct GX_EXPORT SVREID {
-	BINARY *pbin;
-	uint64_t folder_id;
-	uint64_t message_id;
-	uint32_t instance;
+	BINARY *pbin = nullptr;
+	eid_t folder_id{}, message_id{};
+	uint32_t instance = 0;
 
 	std::strong_ordering operator<=>(const SVREID &) const;
 	inline bool operator==(const SVREID &o) const { return (*this <=> o) == 0; }
@@ -1640,6 +1640,7 @@ extern GX_EXPORT const GUID
 	gx_dbguid_store_private, gx_dbguid_store_public,
 	exc_replid2, exc_replid3, exc_replid4;
 extern GX_EXPORT const uint8_t MACBINARY_ENCODING[9], OLE_TAG[11], ThirdPartyGlobalId[12];
+extern GX_EXPORT const char IPM_Appointment_Exception[];
 
 namespace gromox {
 extern GX_EXPORT std::string guid2name(const FLATUID);
