@@ -121,7 +121,7 @@ enum {
 	// PR_DISC_VAL = PROP_TAG(PT_BOOLEAN, 0x004A),
 	PR_ORIG_MESSAGE_CLASS = PROP_TAG(PT_UNICODE, 0x004B), /* PidTagOriginalMessageClass */
 	PR_ORIG_MESSAGE_CLASS_A = PROP_TAG(PT_STRING8, 0x004B),
-	// PR_ORIGINAL_AUTHOR_ENTRYID = PROP_TAG(PT_BINARY, 0x004C), /* PidTagOriginalAuthorEntryId */
+	PR_ORIGINAL_AUTHOR_ENTRYID = PROP_TAG(PT_BINARY, 0x004C), /* PidTagOriginalAuthorEntryId */
 	// PR_ORIGINAL_AUTHOR_NAME = PROP_TAG(PT_UNICODE, 0x004D), /* PidTagOriginalAuthorName */
 	// PR_ORIGINAL_SUBMIT_TIME = PROP_TAG(PT_SYSTIME, 0x004E), /* PidTagOriginalSubmitTime */
 	PR_REPLY_RECIPIENT_ENTRIES = PROP_TAG(PT_BINARY, 0x004F), /* PidTagReplyRecipientEntries */
@@ -131,16 +131,16 @@ enum {
 	PR_READ_RECEIPT_SEARCH_KEY = PROP_TAG(PT_BINARY, 0x0053), /* PidTagReadReceiptSearchKey */
 	// PR_REPORT_SEARCH_KEY = PROP_TAG(PT_BINARY, 0x0054), /* PidTagReportSearchKey */
 	PR_ORIGINAL_DELIVERY_TIME = PROP_TAG(PT_SYSTIME, 0x0055), /* PidTagOriginalDeliveryTime */
-	// PR_ORIGINAL_AUTHOR_SEARCH_KEY = PROP_TAG(PT_BINARY, 0x0056), /* PidTagOriginalAuthorSearchKey */
+	PR_ORIGINAL_AUTHOR_SEARCH_KEY = PROP_TAG(PT_BINARY, 0x0056), /* PidTagOriginalAuthorSearchKey */
 	PR_MESSAGE_TO_ME = PROP_TAG(PT_BOOLEAN, 0x0057), /* PidTagMessageToMe */
 	PR_MESSAGE_CC_ME = PROP_TAG(PT_BOOLEAN, 0x0058), /* PidTagMessageCcMe */
 	PR_MESSAGE_RECIP_ME = PROP_TAG(PT_BOOLEAN, 0x0059), /* PidTagMessageRecipientMe */
 	// PR_ORIGINAL_SENDER_NAME = PROP_TAG(PT_UNICODE, 0x005A), /* PidTagOriginalSenderName */
-	// PR_ORIGINAL_SENDER_ENTRYID = PROP_TAG(PT_BINARY, 0x005B), /* PidTagOriginalSenderEntryId */
-	// PR_ORIGINAL_SENDER_SEARCH_KEY = PROP_TAG(PT_BINARY, 0x005C), /* PidTagOriginalSenderSearchKey */
+	PR_ORIGINAL_SENDER_ENTRYID = PROP_TAG(PT_BINARY, 0x005B), /* PidTagOriginalSenderEntryId */
+	PR_ORIGINAL_SENDER_SEARCH_KEY = PROP_TAG(PT_BINARY, 0x005C), /* PidTagOriginalSenderSearchKey */
 	// PR_ORIGINAL_SENT_REPRESENTING_NAME = PROP_TAG(PT_UNICODE, 0x005D), /* PidTagOriginalSentRepresentingName */
-	// PR_ORIGINAL_SENT_REPRESENTING_ENTRYID = PROP_TAG(PT_BINARY, 0x005E), /* PidTagOriginalSentRepresentingEntryId */
-	// PR_ORIGINAL_SENT_REPRESENTING_SEARCH_KEY = PROP_TAG(PT_BINARY, 0x005F), /* PidTagOriginalSentRepresentingSearchKey */
+	PR_ORIGINAL_SENT_REPRESENTING_ENTRYID = PROP_TAG(PT_BINARY, 0x005E), /* PidTagOriginalSentRepresentingEntryId */
+	PR_ORIGINAL_SENT_REPRESENTING_SEARCH_KEY = PROP_TAG(PT_BINARY, 0x005F), /* PidTagOriginalSentRepresentingSearchKey */
 	PR_START_DATE = PROP_TAG(PT_SYSTIME, 0x0060), /* PidTagStartDate */
 	PR_END_DATE = PROP_TAG(PT_SYSTIME, 0x0061), /* PidTagEndDate */
 	PR_OWNER_APPT_ID = PROP_TAG(PT_LONG, 0x0062), /* PidTagOwnerAppointmentId */
@@ -312,7 +312,7 @@ enum {
 	PR_RTF_COMPRESSED = PROP_TAG(PT_BINARY, 0x1009), /* PidTagRtfCompressed */
 	// PR_RTF_SYNC_PREFIX_COUNT = PROP_TAG(PT_LONG, 0x1010), /* PidTagRtfSyncPrefixCount */
 	// PR_RTF_SYNC_TRAILING_COUNT = PROP_TAG(PT_LONG, 0x1011), /* PidTagRtfSyncTrailingCount */
-	// PR_ORIGINALLY_INTENDED_RECIP_ENTRYID = PROP_TAG(PT_BINARY, 0x1012), /* PidTagOriginallyIntendedRecipEntryId */
+	PR_ORIGINALLY_INTENDED_RECIP_ENTRYID = PROP_TAG(PT_BINARY, 0x1012), /* PidTagOriginallyIntendedRecipEntryId */
 	PR_BODY_HTML = PROP_TAG(PT_UNICODE, 0x1013), /* PidTagBodyHtml */
 	PR_BODY_HTML_A = PROP_TAG(PT_STRING8, 0x1013),
 	PR_HTML = PROP_TAG(PT_BINARY, 0x1013), /* PidTagHtml */
@@ -1022,8 +1022,8 @@ enum {
 	// PR_OFFLINE_FOLDER = PROP_TAG(PT_BINARY, 0x7C05),
 	// PR_ROAMING_DATATYPES = PROP_TAG(PT_LONG, 0x7C06), /* PidTagRoamingDatatypes */
 	// PR_ROAMING_DICTIONARY = PROP_TAG(PT_BINARY, 0x7C07), /* PidTagRoamingDictionary */
-	// PR_ROAMING_XMLSTREAM  = PROP_TAG(PT_BINARY, 0x7C08), /* PidTagRoamingXmlStream */
-	// PR_ROAMING_BINARYSTREAM = PROP_TAG(PT_BINARY, 0x7C09), /* PidTagRoamingBinary */
+	PR_ROAMING_XMLSTREAM  = PROP_TAG(PT_BINARY, 0x7C08), /* PidTagRoamingXmlStream */
+	PR_ROAMING_BINARYSTREAM = PROP_TAG(PT_BINARY, 0x7C09), /* PidTagRoamingBinary */
 	// PR_STORE_SLOWLINK = PROP_TAG(PT_BOOLEAN, 0x7C0A),
 	// PR_OSC_SYNC_ENABLEDONSERVER = PROP_TAG(PT_BOOLEAN, 0x7C24), /* PidTagOscSyncEnabled */
 	// PR_FORCE_USE_ENTRYID_SERVER = PROP_TAG(PT_BOOLEAN, 0x7CFE),
@@ -1122,6 +1122,7 @@ enum {
 
 	/* PSETID_Address */
 	PidLidFileAs = 0x8005,
+	PidLidHasPicture = 0x8015,
 	PidLidHomeAddress = 0x801a,
 	PidLidBusinessAddress = 0x801b,
 	PidLidOtherAddress = 0x801c,
@@ -1211,6 +1212,7 @@ enum {
 	PidLidMonthOfYearMask = 0x0017,
 	PidLidOwnerCriticalChange = 0x001A,
 	PidLidCleanGlobalObjectId = 0x0023,
+	PidLidAppointmentMessageClass = 0x0024,
 	PidLidMeetingType = 0x0026,
 
 	/* PSETID_Common */
@@ -1236,5 +1238,6 @@ enum {
 	PidLidClassificationDescription = 0x85B7,
 	PidLidClassificationGuid = 0x85B8,
 	PidLidClassificationKeep = 0x85BA,
+	PidLidCommon85D8 = 0x85D8, /* undocumented */
 	PROP_ID_INVALID = 0xFFFF,
 };
