@@ -1,5 +1,5 @@
 // SPDX-License-Identifier: AGPL-3.0-or-later
-// SPDX-FileCopyrightText: 2021–2025 grommunio GmbH
+// SPDX-FileCopyrightText: 2021–2026 grommunio GmbH
 // This file is part of Gromox.
 #ifdef HAVE_CONFIG_H
 #	include "config.h"
@@ -56,6 +56,7 @@
 #include <gromox/atomic.hpp>
 #include <gromox/clock.hpp>
 #include <gromox/config_file.hpp>
+#include <gromox/cookie_parser.hpp>
 #include <gromox/fileio.h>
 #include <gromox/generic_connection.hpp>
 #include <gromox/json.hpp>
@@ -533,7 +534,16 @@ static int utf8_writeout(FILE *fp, const void *vsrc, size_t src_size, const char
 		if (fwrite(buffer, sizeof(buffer) - dst_size, 1, fp) != 1)
 			return -1;
 	}
-	errno = 0;
+
+	/* Flush pending shift and/or state */
+	auto dst = buffer;
+	size_t dst_size = sizeof(buffer);
+	auto ret = iconv(cd, nullptr, 0, &dst, &dst_size);
+	if (ret == static_cast<size_t>(-1))
+		/* ignore */;
+	if (dst_size != sizeof(buffer) &&
+	    fwrite(buffer, sizeof(buffer) - dst_size, 1, fp) != 1)
+		return -1;
 	return 0;
 }
 
@@ -546,9 +556,10 @@ static int utf8_writeout(FILE *fp, const void *vsrc, size_t src_size, const char
  * @outbuf: result variable for caller
  *
  * It is valid for @inbuf to point to the same object as @outbuf.
- * Returns 0 on success, non-zero on error with errno set.
+ * Returns 0 on success, negative non-zero on error with errno set.
+ * @outbuf is only replaced on success.
  */
-int feed_w3m(std::string_view inbuf, const char *cset, std::string &outbuf) try
+int feed_w3m(std::string_view inbuf, const char *cset, std::string &final_buf) try
 {
 	std::string filename;
 	auto tmpdir = getenv("TMPDIR");
@@ -584,17 +595,18 @@ int feed_w3m(std::string_view inbuf, const char *cset, std::string &outbuf) try
 		return -1;
 	int status = 0;
 	auto cl3 = HX::make_scope_exit([&]() { waitpid(pid, &status, 0); });
-	outbuf.clear();
+
+	std::string outbuf;
 	ssize_t ret;
 	char fbuf[4096];
 	while ((ret = read(fout, fbuf, std::size(fbuf))) > 0)
 		outbuf.append(fbuf, ret);
-	if (!WIFEXITED(status))
+	cl3.release();
+	waitpid(pid, &status, 0);
+	if (!WIFEXITED(status) || WEXITSTATUS(status) != 0)
 		return -1;
-	if (outbuf.empty())
-		return 0;
-	/* The caller can just look at outbuf.size() */
-	return 1;
+	final_buf = std::move(outbuf);
+	return 0;
 } catch (...) {
 	return -1;
 }
@@ -948,17 +960,19 @@ bool cset_cstr_compatible(const char *s)
 	return true;
 }
 
-static void init_locale()
+bool setup_utf8_locale()
 {
-	setlocale(LC_ALL, "C.UTF-8");
-	if (iswalnum(0x79C1))
-		return;
-	setlocale(LC_ALL, "en_US.UTF-8");
+	if (setlocale(LC_ALL, "C.UTF-8") != nullptr && iswalnum(0x79C1))
+		return true;
+	if (setlocale(LC_ALL, "en_US.UTF-8") != nullptr)
+		return true;
+	mlog(LV_INFO, "Could not set the program to UTF-8 locale. "
+		"Text operations, e.g. PR_SUBJECT_PREFIX extraction, may fail to produce results.");
+	return false;
 }
 
 int iconv_validate()
 {
-	init_locale();
 	for (const auto s : {"UTF-16LE", "windows-1252",
 	     "iso-8859-1", "iso-2022-jp"}) {
 		auto k = iconv_open("UTF-8", s);
@@ -979,7 +993,7 @@ bool get_digest(const Json::Value &jval, const char *key, std::string &out) try
 	out = jval[key].asString();
 	return TRUE;
 } catch (const std::bad_alloc &) {
-	mlog(LV_ERR, "E-1988: ENOMEM");
+	mlog(LV_ERR, "%s: ENOMEM", __PRETTY_FUNCTION__);
 	return false;
 }
 
@@ -994,7 +1008,7 @@ bool get_digest(const Json::Value &jval, const char *key, char *out, size_t outm
 		gx_strlcpy(out, memb.asString().c_str(), outmax);
 	return TRUE;
 } catch (const std::bad_alloc &) {
-	mlog(LV_ERR, "E-1988: ENOMEM");
+	mlog(LV_ERR, "%s: ENOMEM", __PRETTY_FUNCTION__);
 	return false;
 }
 
@@ -1005,7 +1019,7 @@ bool get_digest(const char *json, const char *key, char *out, size_t outmax) try
 		return false;
 	return get_digest(jval, key, out, outmax);
 } catch (const std::bad_alloc &) {
-	mlog(LV_ERR, "E-1324: ENOMEM");
+	mlog(LV_ERR, "%s: ENOMEM", __PRETTY_FUNCTION__);
 	return false;
 }
 
@@ -1021,7 +1035,7 @@ set_digest2(char *json, size_t iomax, const char *key, T &&val) try
 	gx_strlcpy(json, Json::writeString(swb, std::move(jval)).c_str(), iomax);
 	return true;
 } catch (const std::bad_alloc &) {
-	mlog(LV_ERR, "E-1989: ENOMEM");
+	mlog(LV_ERR, "%s: ENOMEM", __func__);
 	return false;
 }
 
@@ -1086,6 +1100,7 @@ int tmpfile::open_impl(const char *dir, unsigned int flags, unsigned int mode,
 	m_fd = open(m_path.c_str(), O_CREAT | flags, mode);
 	return m_fd >= 0 ? m_fd : -errno;
 } catch (const std::bad_alloc &) {
+	mlog(LV_ERR, "%s: ENOMEM", __PRETTY_FUNCTION__);
 	return -ENOMEM;
 }
 
@@ -1414,6 +1429,7 @@ errno_t gx_decompress_file(const char *infile, BINARY &outbin,
 	outbin.pb[outbin.cb] = '\0';
 	return 0;
 } catch (const std::bad_alloc &) {
+	mlog(LV_ERR, "%s: ENOMEM", __func__);
 	return ENOMEM;
 }
 
@@ -1517,12 +1533,13 @@ errno_t parse_imap_seq(imap_seq_list &r, const char *s) try
 	}
 	return 0;
 } catch (const std::bad_alloc &) {
+	mlog(LV_ERR, "%s: ENOMEM", __func__);
 	return ENOMEM;
 }
 
 /**
- * @h: class on message
- * @n: class to test
+ * @h:  PR_MESSAGE_CLASS/PR_CONTAINER_CLASS value
+ * @n:  prefix to test for
  *
  * On match, 0 is returned; otherwise whatever strcasecmp would yield.
  */
@@ -1538,7 +1555,11 @@ int class_match_prefix(const char *h, const char *n)
 }
 
 /**
+ * @h:  PR_MESSAGE_CLASS/PR_CONTAINER_CLASS value
+ * @n:  suffix to test for
+ *
  * On match, 0 is returned; otherwise anything non-zero.
+ * This function is not meant for testing for root classes like "IPM".
  */
 int class_match_suffix(const char *h, const char *n)
 {
@@ -1573,41 +1594,73 @@ size_t utf8_printable_prefix(const void *vinput, size_t max)
  * escape without conversion to ENOMEM. Callers should ideally check for ENOMEM
  * anyway.
  */
-std::string iconvtext(const char *src, size_t src_size,
-    const char *from, const char *to)
+std::string iconvtext(std::string_view sv,
+    const char *from, const char *to, unsigned int flags) try
 {
 	if (strcasecmp(from, to) == 0) {
 		errno = 0;
-		return {reinterpret_cast<const char *>(src), src_size};
+		return std::string(sv);
 	}
-	auto cs = to + "//IGNORE"s;
-	auto cd = iconv_open(cs.c_str(), from);
+	auto cd = iconv_open(to, from);
 	if (cd == reinterpret_cast<iconv_t>(-1)) {
-		mlog(LV_ERR, "E-2116: iconv_open %s: %s",
-		        cs.c_str(), strerror(errno));
+		mlog(LV_ERR, "E-2116: iconv_open(%s -> %s): %s", from, to, strerror(errno));
 		errno = EINVAL;
 		return {};
 	}
-	auto cleanup = HX::make_scope_exit([&]() { iconv_close(cd); });
-	char buffer[4096];
-	std::string out;
+	try {
+		char buffer[4096];
+		std::string out;
+		bool last_bad = false;
+		auto src = deconst(sv.data());
+		size_t src_size = sv.size();
 
-	while (src_size > 0) {
+		while (src_size > 0) {
+			auto dst = buffer;
+			size_t dst_size = sizeof(buffer);
+			errno = 0;
+			auto ret = iconv(cd, &src, &src_size, &dst, &dst_size);
+			if (dst_size != sizeof(buffer)) {
+				last_bad = false;
+				out.append(buffer, sizeof(buffer) - dst_size);
+			}
+			if (ret != (size_t)-1 || src_size == 0) {
+				last_bad = false;
+				continue;
+			}
+			if (errno == EILSEQ || errno == EINVAL) {
+				--src_size;
+				++src;
+				if (flags & ICONVTEXT_TRANSLIT) {
+					if (!last_bad)
+						out += '?';
+					last_bad = true;
+				}
+			}
+		}
+
+		/* Flush pending shift and/or state */
 		auto dst = buffer;
 		size_t dst_size = sizeof(buffer);
-		auto ret = iconv(cd, (char**)&src, &src_size, (char**)&dst, &dst_size);
-		if (ret != static_cast<size_t>(-1) || dst_size != sizeof(buffer)) {
+		errno = 0;
+		auto ret = iconv(cd, nullptr, 0, &dst, &dst_size);
+		if (dst_size != sizeof(buffer))
 			out.append(buffer, sizeof(buffer) - dst_size);
-			continue;
+		if (ret != (size_t)-1 || src_size == 0) {
+		} else if (errno == EILSEQ || errno == EINVAL) {
+			if (flags & ICONVTEXT_TRANSLIT)
+				out += '?';
 		}
-		if (src_size > 0) {
-			--src_size;
-			++src;
-		}
-		out.append(buffer, sizeof(buffer) - dst_size);
+		errno = 0;
+		iconv_close(cd);
+		return out;
+	} catch (const std::bad_alloc &) {
+		iconv_close(cd);
+		errno = ENOMEM;
+		return {};
 	}
-	errno = 0;
-	return out;
+} catch (const std::bad_alloc &) {
+	errno = ENOMEM;
+	return {};
 }
 
 std::string base64_encode(const std::string_view &x)
@@ -1812,6 +1865,7 @@ errno_t canonical_hostname(std::string &out) try
 	out = aires->ai_canonname;
 	return 0;
 } catch (const std::bad_alloc &) {
+	mlog(LV_ERR, "%s: ENOMEM", __func__);
 	return ENOMEM;
 }
 
@@ -1840,7 +1894,7 @@ int gx_mkbasedir(const char *file, unsigned int mode)
 {
 	std::unique_ptr<char[], stdlib_delete> base(HX_dirname(file));
 	if (base == nullptr)
-		return ENOMEM;
+		return -ENOMEM;
 	if (mode & (S_IRUSR | S_IWUSR))
 		mode |= S_IXUSR;
 	if (mode & (S_IRGRP | S_IWGRP))
@@ -1850,6 +1904,79 @@ int gx_mkbasedir(const char *file, unsigned int mode)
 	return HX_mkdir(base.get(), mode);
 }
 
+static std::string cookie_rmeta(std::string_view sv)
+{
+	std::string s{sv};
+	auto o = s.begin();
+	for (auto i = s.begin(); i < s.end(); ++i) {
+		if (*i != '%') {
+			*o++ = *i;
+			continue;
+		}
+		if (i + 1 == s.end() || i + 2 == s.end())
+			break;
+		char tb[3] = {i[1], i[2], '\0'};
+		char *end = nullptr;
+		uint8_t c = strtoul(tb, &end, 16);
+		if (end == &tb[2])
+			*o++ = c;
+		i += 2;
+	}
+	s.erase(o, s.end());
+	return s;
+}
+
+static inline size_t sv_cspn(std::string_view sv, char c)
+{
+	auto p = sv.find(c);
+	return p != sv.npos ? p : sv.size();
+}
+
+ec_error_t cookie_jar::add(std::string_view sv) try
+{
+	while (sv.size() > 0) {
+		while (sv.size() > 0 && HX_isspace(sv[0]))
+			sv.remove_prefix(1);
+		auto klen = sv.find_first_of(";=");
+		if (klen == sv.npos)
+			klen = sv.size();
+		auto &value = emplace(sv.substr(0, klen), std::string()).first->second;
+		sv.remove_prefix(klen);
+		if (sv.size() == 0)
+			break; /* attribute without cookie-value */
+		if (sv[0] != '=') {
+			sv.remove_prefix(1); /* ; */
+			continue;
+		}
+		sv.remove_prefix(1); /* = */
+		if (sv.size() == 0)
+			break;
+		size_t vlen;
+		if (sv[0] == '"') {
+			sv.remove_prefix(1);
+			vlen  = sv_cspn(sv, '\"');
+			value = cookie_rmeta(sv.substr(0, vlen));
+			sv.remove_prefix(vlen);
+			vlen  = sv_cspn(sv, ';');
+		} else {
+			vlen  = sv_cspn(sv, ';');
+			value = cookie_rmeta(sv.substr(0, vlen));
+		}
+		sv.remove_prefix(vlen);
+		if (sv.size() > 0)
+			sv.remove_prefix(1); /* ; */
+	}
+	return ecSuccess;
+} catch (const std::bad_alloc &) {
+	return ecServerOOM;
+}
+
+const char *cookie_jar::operator[](const char *name) const
+{
+	auto i = map::find(name);
+	return i != cend() ? i->second.c_str() : nullptr;
+}
+
 }
 
 int XARRAY::append(MITEM &&ptr, unsigned int tag) try
@@ -1981,7 +2108,7 @@ void config_file::set_value(const char *sk, const char *sv) try
 		}
 	}
 } catch (const std::bad_alloc &) {
-	mlog(LV_ERR, "E-2367: ENOMEM");
+	mlog(LV_ERR, "%s: ENOMEM", __PRETTY_FUNCTION__);
 }
 
 BOOL config_file::save()
@@ -2064,16 +2191,11 @@ std::shared_ptr<CONFIG_FILE> config_file_initd(const char *fb,
 	}
 	return std::make_shared<CONFIG_FILE>(key_desc);
 } catch (const std::bad_alloc &) {
+	mlog(LV_ERR, "%s: ENOMEM", __func__);
 	errno = ENOMEM;
 	return nullptr;
 }
 
-static const char *default_searchpath()
-{
-	const char *ed = getenv("GROMOX_CONFIG_PATH");
-	return ed != nullptr ? ed : PKGSYSCONFDIR;
-}
-
 /**
  * Routine intended for programs:
  *
@@ -2084,7 +2206,7 @@ std::shared_ptr<CONFIG_FILE> config_file_prg(const char *ov, const char *fb,
     const cfg_directive *key_desc)
 {
 	if (ov == nullptr)
-		return config_file_initd(fb, default_searchpath(), key_desc);
+		return config_file_initd(fb, PKGSYSCONFDIR, key_desc);
 	auto cfg = config_file_init(ov, key_desc);
 	if (cfg == nullptr)
 		mlog(LV_ERR, "config_file_init %s: %s", ov, strerror(errno));
@@ -2166,3 +2288,37 @@ generic_connection generic_connection::accept(int sv_sock,
 	conn.last_timestamp = tp_now();
 	return conn;
 }
+
+/**
+ * Convert Unicode code point @wchar to its UTF-8 representation.
+ */
+std::string wchar_to_utf8(uint32_t w)
+{
+	std::string s;
+	if (w <= 0x7f) {
+		s.resize(1);
+		s[0] = w;
+	} else if (w <= 0x7ff) {
+		s.resize(2);
+		s[0] = 192 + w / 64;
+		s[1] = 128 + w % 64;
+	} else if (w <= 0xffff) {
+		s.resize(3);
+		s[0] = 224 + w / 4096;
+		s[1] = 128 + w / 64 % 64;
+		s[2] = 128 + w % 64;
+	} else if (w <= 0x10ffff) {
+		s.resize(4);
+		s[0] = 240 + w / 262144;
+		s[1] = 128 + w / 4096 % 64;
+		s[2] = 128 + w / 64 % 64;
+		s[3] = 128 + w % 64;
+	} else {
+		s.resize(3);
+		w = 0xfffd;
+		s[0] = 224 + w / 4096;
+		s[1] = 128 + w / 64 % 64;
+		s[2] = 128 + w % 64;
+	}
+	return s;
+}
