@@ -341,10 +341,9 @@ static ec_error_t rx_npid_replace(rxparam &par, MESSAGE_CONTENT &ctnt,
 
 ec_error_t rxparam::is_oof(bool *oof) const
 {
-	static constexpr uint32_t tags[] = {PR_OOF_STATE};
-	static constexpr PROPTAG_ARRAY pt = {std::size(tags), deconst(tags)};
+	static constexpr proptag_t tags[] = {PR_OOF_STATE};
 	TPROPVAL_ARRAY props{};
-	if (!exmdb_client->get_store_properties(cur.dirc(), CP_UTF8, &pt, &props))
+	if (!exmdb_client->get_store_properties(cur.dirc(), CP_UTF8, tags, &props))
 		return ecError;
 	auto flag = props.get<uint8_t>(PR_OOF_STATE);
 	*oof = flag != nullptr ? *flag : 0;
@@ -375,13 +374,12 @@ ec_error_t rxparam::load_std_rules(bool oof,
 	    &table_id, &row_count))
 		return ecError;
 	auto cl_0 = HX::make_scope_exit([&]() { exmdb_client->unload_table(dir, table_id); });
-	static constexpr uint32_t tags[] = {
+	static constexpr proptag_t tags[] = {
 		PR_RULE_STATE, PR_RULE_ID, PR_RULE_SEQUENCE, PR_RULE_NAME,
 		PR_RULE_PROVIDER, PR_RULE_CONDITION, PR_RULE_ACTIONS,
 	};
-	const PROPTAG_ARRAY ptags = {std::size(tags), deconst(tags)};
 	tarray_set output_rows{};
-	if (!exmdb_client->query_table(dir, nullptr, CP_ACP, table_id, &ptags,
+	if (!exmdb_client->query_table(dir, nullptr, CP_ACP, table_id, tags,
 	    0, row_count, &output_rows))
 		return ecError;
 
@@ -436,17 +434,15 @@ ec_error_t rxparam::load_ext_rules(bool oof,
 		return ecError;
 	auto cl_0 = HX::make_scope_exit([&]() { exmdb_client->unload_table(dir, table_id); });
 
-	static constexpr uint32_t tags[] = {
+	static constexpr proptag_t tags[] = {
 		PR_RULE_MSG_STATE, PidTagMid, PR_RULE_MSG_SEQUENCE,
 		PR_RULE_MSG_PROVIDER,
 	};
-	static constexpr uint32_t tags2[] = {
+	static constexpr proptag_t tags2[] = {
 		PR_EXTENDED_RULE_MSG_CONDITION, PR_EXTENDED_RULE_MSG_ACTIONS,
 	};
-	const PROPTAG_ARRAY ptags = {std::size(tags), deconst(tags)};
-	const PROPTAG_ARRAY ptags2 = {std::size(tags2), deconst(tags2)};
 	tarray_set output_rows{};
-	if (!exmdb_client->query_table(dir, nullptr, CP_ACP, table_id, &ptags,
+	if (!exmdb_client->query_table(dir, nullptr, CP_ACP, table_id, tags,
 	    0, row_count, &output_rows))
 		return ecError;
 
@@ -469,7 +465,7 @@ ec_error_t rxparam::load_ext_rules(bool oof,
 		rule.provider = znul(row->get<const char>(PR_RULE_MSG_PROVIDER));
 		TPROPVAL_ARRAY vals2{};
 		if (!exmdb_client->get_message_properties(dir, nullptr, CP_ACP,
-		    *mid, &ptags2, &vals2))
+		    *mid, tags2, &vals2))
 			continue;
 		auto cond = vals2.get<const BINARY>(PR_EXTENDED_RULE_MSG_CONDITION);
 		auto act  = vals2.get<const BINARY>(PR_EXTENDED_RULE_MSG_ACTIONS);
@@ -750,7 +746,7 @@ static ec_error_t op_copy_other(rxparam &par, const rule_node &rule,
 	/* Copy done, delete original message object */
 	EID_ARRAY del_mids{};
 	del_mids.count = 1;
-	del_mids.pids = reinterpret_cast<uint64_t *>(&par.cur.mid);
+	del_mids.pids  = &par.cur.mid;
 	BOOL partial = false;
 	if (!exmdb_client->delete_messages(par.cur.dirc(), CP_UTF8, nullptr,
 	    par.cur.fid, &del_mids, true, &partial))
@@ -930,14 +926,17 @@ static ec_error_t mr_get_policy(const char *ev_to, mr_policy &pol)
 	auto &uprop = *props;
 	if (!mysql_adaptor_get_user_properties(ev_to, uprop))
 		return ecError;
+	auto value = uprop.get<uint32_t>(PR_DISPLAY_TYPE_EX);
+	pol.dtyp = value == nullptr ? 0 : *value & DTE_MASK_LOCAL;
 	auto flag = uprop.get<const uint8_t>(PR_SCHDINFO_DISALLOW_OVERLAPPING_APPTS);
-	pol.decline_overlap = flag != nullptr && *flag != 0;
+	if (flag != nullptr)
+		pol.decline_overlap = *flag != 0;
+	else
+		pol.decline_overlap = pol.dtyp == DT_ROOM || pol.dtyp == DT_EQUIPMENT;
 	flag = uprop.get<uint8_t>(PR_SCHDINFO_DISALLOW_RECURRING_APPTS);
 	pol.decline_recurring = flag != nullptr && *flag != 0;
-	auto value = uprop.get<uint32_t>(PR_EMS_AB_ROOM_CAPACITY);
+	value = uprop.get<uint32_t>(PR_EMS_AB_ROOM_CAPACITY);
 	pol.capacity = value != nullptr ? *value : 0;
-	value = uprop.get<uint32_t>(PR_DISPLAY_TYPE_EX);
-	pol.dtyp = value == nullptr ? 0 : *value & DTE_MASK_LOCAL;
 	flag = uprop.get<uint8_t>(PR_SCHDINFO_AUTO_ACCEPT_APPTS);
 	if (flag != nullptr)
 		pol.accept_appts = !!*flag;
@@ -985,7 +984,7 @@ static ec_error_t mr_insert_to_cal(rxparam &par, const PROPID_ARRAY &propids,
 	if (msg == nullptr)
 		return ecServerOOM;
 	auto &prop = msg->proplist;
-	static constexpr uint32_t rmprops[] = {
+	static constexpr proptag_t rmprops[] = {
 		PidTagMid, PidTagChangeNumber, PR_CHANGE_KEY,
 		PR_PREDECESSOR_CHANGE_LIST,
 		PR_RECEIVED_BY_ENTRYID, PR_RECEIVED_BY_NAME,
@@ -998,10 +997,11 @@ static ec_error_t mr_insert_to_cal(rxparam &par, const PROPID_ARRAY &propids,
 	};
 	for (auto t : rmprops)
 		prop.erase(t);
-	static constexpr uint32_t v_busy = olBusy;
+	static constexpr uint32_t v_busy = olBusy, stateflags = asfMeeting | asfReceived;
 	ec_error_t err;
 	if ((err = prop.set(PROP_TAG(PT_LONG, propids[l_response_status]), &accept_type)) != ecSuccess ||
 	    (err = prop.set(PROP_TAG(PT_LONG, propids[l_busy_status]), &v_busy)) != ecSuccess ||
+	    (err = prop.set(PROP_TAG(PT_LONG, propids[l_appt_state_flags]), &stateflags)) != ecSuccess ||
 	    (err = prop.set(PR_MESSAGE_CLASS, "IPM.Appointment")) != ecSuccess)
 		return err;
 	uint64_t cal_mid = 0, cal_cn = 0;
@@ -1031,7 +1031,7 @@ static ec_error_t mr_send_response(rxparam &par, bool recurring_flg,
 	if (rsp_ctnt == nullptr)
 		return ecMAPIOOM;
 	auto &rsp_prop = rsp_ctnt->proplist;
-	static const uint32_t copytags_1[] = {
+	const proptag_t copytags_1[] = {
 		/* OXOCAL §3.1.4.8.4 */
 		PROP_TAG(PT_UNICODE, propids[l_location]),
 		PROP_TAG(PT_UNICODE, propids[l_where]),
@@ -1047,7 +1047,7 @@ static ec_error_t mr_send_response(rxparam &par, bool recurring_flg,
 		PR_SUBJECT_PREFIX, PR_NORMALIZED_SUBJECT,
 		PR_CONVERSATION_INDEX_TRACKING,
 	};
-	static const uint32_t copytags_2[] = {
+	const proptag_t copytags_2[] = {
 		/* OXOCAL §3.1.4.8.4 */
 		PROP_TAG(PT_BINARY, propids[l_tzstruct]),
 		PROP_TAG(PT_BINARY, propids[l_apptrecur]),
@@ -1056,19 +1056,19 @@ static ec_error_t mr_send_response(rxparam &par, bool recurring_flg,
 		PROP_TAG(PT_LONG, propids[l_tz]),
 		PROP_TAG(PT_UNICODE, propids[l_tzdesc]),
 	};
-	for (const auto propid : copytags_1) {
-		auto v = rq_prop.getval(propid);
+	for (const auto tag : copytags_1) {
+		auto v = rq_prop.getval(tag);
 		if (v != nullptr) {
-			auto err = rsp_prop.set(propid, v);
+			auto err = rsp_prop.set(tag, v);
 			if (err != ecSuccess)
 				return err;
 		}
 	}
 	if (recurring_flg)
-		for (const auto propid : copytags_2) {
-			auto v = rq_prop.getval(propid);
+		for (const auto tag : copytags_2) {
+			auto v = rq_prop.getval(tag);
 			if (v != nullptr) {
-				auto err = rsp_prop.set(propid, v);
+				auto err = rsp_prop.set(tag, v);
 				if (err != ecSuccess)
 					return err;
 			}
@@ -1144,10 +1144,17 @@ static ec_error_t mr_send_response(rxparam &par, bool recurring_flg,
 			return err;
 	}
 
+	auto log_id = "f" + std::to_string(rop_util_get_gc_value(par.cur.fid)) +
+	              ":m" + std::to_string(rop_util_get_gc_value(par.cur.mid));
 	MAIL imail;
 	rp_storedir = par.cur.dirc();
-	if (!oxcmail_export(rsp_ctnt.get(), "-", false, oxcmail_body::plain_and_html,
-	    &imail, cu_alloc, cu_get_propids, cu_get_propname)) {
+
+	oxcmail_converter cvt;
+	cvt.log_id = log_id.c_str();
+	cvt.alloc = cu_alloc;
+	cvt.get_propids = cu_get_propids;
+	cvt.get_propname = cu_get_propname;
+	if (!cvt.mapi_to_inet(*rsp_ctnt, imail)) {
 		mlog(LV_ERR, "mr_send_response: oxcmail_export failed for an unspecified reason.\n");
 		return ecError;
 	}
@@ -1175,22 +1182,28 @@ static ec_error_t mr_do_request(rxparam &par, const PROPID_ARRAY &propids,
 
 	/* Lookup conflict state */
 	bool res_in_use = false, response_allowed = false;
-	auto start_nt = rq_prop.get<uint64_t>(PR_START_DATE);
-	auto end_nt   = rq_prop.get<uint64_t>(PR_END_DATE);
+	auto start_nt = rq_prop.get<uint64_t>(PROP_TAG(PT_SYSTIME, propids[l_start_whole]));
+	auto end_nt   = rq_prop.get<uint64_t>(PROP_TAG(PT_SYSTIME, propids[l_end_whole]));
+	if (start_nt == nullptr || end_nt == nullptr) {
+		start_nt = rq_prop.get<uint64_t>(PR_START_DATE);
+		end_nt   = rq_prop.get<uint64_t>(PR_END_DATE);
+	}
 	if (start_nt != nullptr && end_nt != nullptr) {
 		std::vector<freebusy_event> fbdata;
 		auto start_ts = rop_util_nttime_to_unix(*start_nt);
 		auto end_ts   = rop_util_nttime_to_unix(*end_nt);
 		/* XXX: May need PR_SENDER rather than Envelope-From */
 		response_allowed = freebusy_perms(par.ev_from, par.cur.dirc()) != 0;
-		if (!get_freebusy(nullptr, par.cur.dirc(), start_ts, end_ts, fbdata))
-			mlog(LV_ERR, "W-PREC: cannot retrieve freebusy %s", par.cur.dirc());
+		auto err = get_freebusy(nullptr, par.cur.dirc(), start_ts, end_ts, fbdata);
+		if (err != ecSuccess)
+			mlog(LV_ERR, "W-PREC: cannot retrieve freebusy %s: %s",
+				par.cur.dirc(), mapi_strerror(err));
 
 		for (const freebusy_event &event : fbdata)
 			if ((event.start_time >= start_ts && event.start_time <= end_ts) ||
 			    (event.end_time   >= start_ts && event.end_time <= end_ts) ||
 			    (event.start_time < start_ts  && event.end_time > end_ts))
-				if (event.busy_status == olBusy) {
+				if (event.busy_status != olFree) {
 					res_in_use = true;
 					break;
 				}
@@ -1207,7 +1220,7 @@ static ec_error_t mr_do_request(rxparam &par, const PROPID_ARRAY &propids,
 	}
 
 	/* Enter meeting into calendar */
-	auto tent = policy.accept_appts ? respAccepted : respTentative;
+	auto tent = policy.accept_appts ? respAccepted : respNotResponded;
 	auto cal_fid = rop_util_make_eid_ex(1, PRIVATE_FID_CALENDAR);
 	auto err = mr_insert_to_cal(par, propids, cal_fid, tent);
 	if (err != ecSuccess)
@@ -1314,7 +1327,7 @@ ec_error_t rxparam::run()
 			break;
 	}
 	if (del) {
-		const EID_ARRAY ids = {1, reinterpret_cast<uint64_t *>(&cur.mid)};
+		const EID_ARRAY ids = {1, &cur.mid};
 		BOOL partial;
 		if (!exmdb_client->delete_messages(cur.dirc(), CP_ACP, nullptr,
 		    cur.fid, &ids, true/*hard*/, &partial))
@@ -1339,10 +1352,9 @@ ec_error_t rxparam::run()
 void rxparam::notify()
 {
 	static constexpr proptag_t tagdata[] = {PR_MESSAGE_FLAGS, PR_MESSAGE_CLASS};
-	static constexpr PROPTAG_ARRAY tags = {std::size(tagdata), deconst(tagdata)};
 	TPROPVAL_ARRAY vals{};
 	if (!exmdb_client->get_message_properties(cur.dirc(), nullptr, CP_UTF8,
-	    cur.mid, &tags, &vals)) {
+	    cur.mid, tagdata, &vals)) {
 		mlog(LV_ERR, "ruleproc: getprops FL/C failed");
 		return;
 	}
@@ -1365,7 +1377,7 @@ static ec_error_t exmdb_local_rules_execute(const char *dir, const char *ev_from
 	p.do_autoproc = flags & DELIVERY_DO_MRAUTOPROC;
 	return std::move(p).run();
 } catch (const std::bad_alloc &) {
-	mlog(LV_ERR, "E-1121: ENOMEM");
+	mlog(LV_ERR, "%s: ENOMEM", __func__);
 	return ecServerOOM;
 }
 
