@@ -1,4 +1,6 @@
 // SPDX-License-Identifier: GPL-2.0-only WITH linking exception
+// SPDX-FileCopyrightText: 2021–2026 grommunio GmbH
+// This file is part of Gromox.
 /*
  *	this file includes some utility functions that will be used by many 
  *	programs
@@ -20,6 +22,7 @@
 #include <unistd.h>
 #include <json/reader.h>
 #include <libHX/ctype_helper.h>
+#include <libHX/scope.hpp>
 #include <libHX/string.h>
 #include <gromox/defs.h>
 #include <gromox/fileio.h>
@@ -184,45 +187,6 @@ void utf8_filter(char *string)
 	}
 }
 
-void wchar_to_utf8(uint32_t wchar, char *zstring)
-{
-	auto string = reinterpret_cast<unsigned char *>(zstring);
-	if (wchar < 0x7f) {
-		string[0] = wchar;
-		string[1] = '\0';
-	} else if (wchar < 0x7ff) {
-		string[0] = 192 + (wchar/64);
-		string[1] = 128 + (wchar%64);
-		string[2] = '\0';
-	} else if (wchar < 0xffff) {
-		string[0] = 224 + wchar/(64*64);
-		string[1] = 128 + (wchar/64)%64;
-		string[2] = 128 + wchar%64;
-		string[3] = '\0';
-	} else if (wchar < 0x1FFFFF) {
-		string[0] = 240 + wchar/(64*64*64);
-		string[1] = 128 + (wchar/(64*64))%64;
-		string[2] = 128 + (wchar/64)%64;
-		string[3] = 128 + wchar % 64;
-		string[4] = '\0';
-	} else if (wchar < 0x3FFFFFF) {
-		string[0] = 248 + wchar/(64*64*64*64);
-		string[1] = 128 + (wchar/(64*64*64))%64;
-		string[2] = 128 + (wchar/(64*64))%64;
-		string[3] = 128 + (wchar/64)%64;
-		string[4] = 128 + wchar % 64;
-		string[5] = '\0';
-	} else if (wchar < 0x7FFFFFFF) {
-		string[0] = 252 + wchar/(64*64*64*64*64);
-		string[1] = 128 + (wchar/(64*64*64*64))%64;
-		string[2] = 128 + (wchar/(64*64*64))%64;
-		string[3] = 128 + (wchar/(64*64))%64;
-		string[4] = 128 + (wchar/64)%64;
-		string[5] = 128 + wchar % 64;
-		string[6] = '\0';
-	}
-}
-
 static bool have_jpms()
 {
 	auto cd = iconv_open("UTF-8", "iso-2022-jp-ms");
@@ -270,10 +234,6 @@ const char* replace_iconv_charset(const char *charset)
 BOOL string_mb_to_utf8(const char *charset, const char *in_string,
     char *out_string, size_t out_len)
 {
-	iconv_t conv_id;
-	char *pin, *pout;
-	char tmp_charset[64];
-	
 	if (0 == strcasecmp(charset, "UTF-8") ||
 		0 == strcasecmp(charset, "ASCII") ||
 		0 == strcasecmp(charset, "US-ASCII")) {
@@ -291,21 +251,38 @@ BOOL string_mb_to_utf8(const char *charset, const char *in_string,
 	if (out_len > 0)
 		/* Leave room for \0 */
 		--out_len;
-	snprintf(tmp_charset, std::size(tmp_charset), "%s//IGNORE", replace_iconv_charset(charset));
-	conv_id = iconv_open("UTF-8", tmp_charset);
+	auto cs = replace_iconv_charset(charset);
+	auto conv_id = iconv_open("UTF-8", cs);
 	if (conv_id == iconv_t(-1)) {
 		/* EINVAL could happen as a result of EMFILE... */
-		mlog(LV_ERR, "E-2108: iconv_open %s: %s",
-		        tmp_charset, strerror(errno));
+		mlog(LV_ERR, "E-2108: iconv_open %s: %s", cs, strerror(errno));
 		return FALSE;
 	}
-	pin = (char*)in_string;
-	pout = out_string;
+	auto pin  = deconst(in_string);
+	auto pout = out_string;
 	auto in_len = length;
-	if (iconv(conv_id, &pin, &in_len, &pout, &out_len) == static_cast<size_t>(-1)) {
+	while (in_len > 0) {
+		auto ret = iconv(conv_id, &pin, &in_len, &pout, &out_len);
+		if (ret != static_cast<size_t>(-1))
+			continue;
+		if (errno == E2BIG)
+			break;
+		if (errno == EILSEQ || errno == EINVAL) {
+			if (in_len > 0) {
+				++pin;
+				--in_len;
+			}
+			continue;
+		}
 		iconv_close(conv_id);
 		return FALSE;
 	}
+	auto ret = iconv(conv_id, nullptr, nullptr, &pout, &out_len);
+	if (ret == static_cast<size_t>(-1) && errno != E2BIG &&
+	    errno != EILSEQ && errno != EINVAL) {
+		iconv_close(conv_id);
+		return false;
+	}
 	iconv_close(conv_id);
 	if (orig_outlen > 0)
 		*pout = '\0';
@@ -338,13 +315,31 @@ BOOL string_utf8_to_mb(const char *charset, const char *in_string,
 		mlog(LV_ERR, "E-2109: iconv_open %s: %s", cs, strerror(errno));
 		return FALSE;
 	}
-	auto pin = const_cast<char *>(in_string);
-	auto pout = out_string;
+	auto pin    = deconst(in_string);
+	auto pout   = out_string;
 	auto in_len = length;
-	if (iconv(conv_id, &pin, &in_len, &pout, &out_len) == static_cast<size_t>(-1)) {
+	while (in_len > 0) {
+		auto ret = iconv(conv_id, &pin, &in_len, &pout, &out_len);
+		if (ret != static_cast<size_t>(-1))
+			continue;
+		if (errno == E2BIG)
+			break;
+		if (errno == EILSEQ || errno == EINVAL) {
+			if (in_len > 0) {
+				++pin;
+				--in_len;
+			}
+			continue;
+		}
 		iconv_close(conv_id);
 		return FALSE;
 	}
+	auto ret = iconv(conv_id, nullptr, nullptr, &pout, &out_len);
+	if (ret == static_cast<size_t>(-1) && errno != E2BIG &&
+	    errno != EILSEQ && errno != EINVAL) {
+		iconv_close(conv_id);
+		return false;
+	}
 	iconv_close(conv_id);
 	if (orig_outlen > 0)
 		*pout = '\0';
@@ -353,50 +348,83 @@ BOOL string_utf8_to_mb(const char *charset, const char *in_string,
 
 ssize_t utf8_to_utf16le(const char *src, void *dst, size_t len)
 {
-	size_t in_len;
-	size_t out_len;
-	iconv_t conv_id;
-
 	len = std::min(len, static_cast<size_t>(SSIZE_MAX));
-	conv_id = iconv_open("UTF-16LE", "UTF-8");
+	auto conv_id = iconv_open("UTF-16LE", "UTF-8");
 	if (conv_id == (iconv_t)-1) {
 		mlog(LV_ERR, "E-2110: iconv_open: %s", strerror(errno));
 		return -1;
 	}
 	auto pin  = deconst(src);
 	auto pout = static_cast<char *>(dst);
-	in_len = strlen(src) + 1;
+	auto in_len = strlen(src) + 1;
 	memset(dst, 0, len);
-	out_len = len;
-	if (iconv(conv_id, &pin, &in_len, &pout, &len) == static_cast<size_t>(-1)) {
+	auto out_len = len;
+	while (in_len > 0) {
+		auto ret = iconv(conv_id, &pin, &in_len, &pout, &out_len);
+		if (ret != static_cast<size_t>(-1))
+			continue;
+		if (errno == E2BIG)
+			break;
+		if (errno == EILSEQ || errno == EINVAL) {
+			if (in_len > 0) {
+				++pin;
+				--in_len;
+			}
+			continue;
+		}
 		iconv_close(conv_id);
 		return -1;
-	} else {
-		iconv_close(conv_id);
-		return out_len - len;
 	}
+	auto ret = iconv(conv_id, nullptr, nullptr, &pout, &out_len);
+	if (ret == static_cast<size_t>(-1) && errno != E2BIG &&
+	    errno != EILSEQ && errno != EINVAL) {
+		iconv_close(conv_id);
+		return -1;
+	}
+	iconv_close(conv_id);
+	return len - out_len;
 }
 
 BOOL utf16le_to_utf8(const void *src, size_t src_len, char *dst, size_t len)
 {
-	char *pin, *pout;
-	iconv_t conv_id;
-	
-	conv_id = iconv_open("UTF-8", "UTF-16LE");
+	auto conv_id = iconv_open("UTF-8", "UTF-16LE");
 	if (conv_id == (iconv_t)-1) {
 		mlog(LV_ERR, "E-2111: iconv_open: %s", strerror(errno));
 		return false;
 	}
-	pin = (char*)src;
-	pout = dst;
+	auto pin  = static_cast<char *>(deconst(src));
+	auto pout = dst;
+	auto out_len = len;
 	memset(dst, 0, len);
-	if (iconv(conv_id, &pin, &src_len, &pout, &len) == static_cast<size_t>(-1)) {
+	static const char unul[2]{};
+	while (src_len > 0) {
+		auto ret = iconv(conv_id, &pin, &src_len, &pout, &out_len);
+		if (ret != static_cast<size_t>(-1))
+			continue;
+		if (errno == E2BIG)
+			break;
+		if (errno == EILSEQ || errno == EINVAL) {
+			if (src_len >= 2) {
+				pin += 2;
+				src_len -= 2;
+				continue;
+			} else if (pin != unul) {
+				pin = deconst(unul);
+				src_len = 2;
+				continue;
+			}
+		}
 		iconv_close(conv_id);
 		return FALSE;
-	} else {
-		iconv_close(conv_id);
-		return TRUE;
 	}
+	auto ret = iconv(conv_id, nullptr, nullptr, &pout, &out_len);
+	if (ret == static_cast<size_t>(-1) && errno != E2BIG &&
+	    errno != EILSEQ && errno != EINVAL) {
+		iconv_close(conv_id);
+		return false;
+	}
+	iconv_close(conv_id);
+	return TRUE;
 }
 
 /*
