@@ -440,7 +440,7 @@ static pack_result zrpc_pull(PULL_CTX &x, zcresp_notifdequeue &d)
 	return pack_result::ok;
 }
 
-static pack_result zrpc_push(PUSH_CTX &x, const zcreq_queryrows &d)
+static pack_result zrpc_push(PUSH_CTX &x, const zcreq_queryrows_v &d)
 {
 	TRY(x.p_guid(d.hsession));
 	TRY(x.p_uint32(d.htable));
@@ -452,14 +452,13 @@ static pack_result zrpc_push(PUSH_CTX &x, const zcreq_queryrows &d)
 		TRY(x.p_uint8(1));
 		TRY(x.p_restriction(*d.prestriction));
 	}
-	if (d.pproptags == nullptr) {
+	if (!d.pproptags.has_value()) {
 		TRY(x.p_uint8(0));
-	return pack_result::ok;
 	} else {
 		TRY(x.p_uint8(1));
 		TRY(x.p_proptag_a(*d.pproptags));
-	return pack_result::ok;
 	}
+	return pack_result::ok;
 }
 
 static pack_result zrpc_pull(PULL_CTX &x, zcresp_queryrows &d)
@@ -468,11 +467,11 @@ static pack_result zrpc_pull(PULL_CTX &x, zcresp_queryrows &d)
 	return pack_result::ok;
 }
 
-static pack_result zrpc_push(PUSH_CTX &x, const zcreq_setcolumns &d)
+static pack_result zrpc_push(PUSH_CTX &x, const zcreq_setcolumns_v &d)
 {
 	TRY(x.p_guid(d.hsession));
 	TRY(x.p_uint32(d.htable));
-	TRY(x.p_proptag_a(*d.pproptags));
+	TRY(x.p_proptag_a(d.pproptags));
 	TRY(x.p_uint32(d.flags));
 	return pack_result::ok;
 }
@@ -646,18 +645,17 @@ static pack_result zrpc_push(PUSH_CTX &x, const zcreq_setpropvals &d)
 	return pack_result::ok;
 }
 
-static pack_result zrpc_push(PUSH_CTX &x, const zcreq_getpropvals &d)
+static pack_result zrpc_push(PUSH_CTX &x, const zcreq_getpropvals_v &d)
 {	
 	TRY(x.p_guid(d.hsession));
 	TRY(x.p_uint32(d.hobject));
-	if (d.pproptags == nullptr) {
+	if (!d.pproptags.has_value()) {
 		TRY(x.p_uint8(0));
-	return pack_result::ok;
 	} else {
 		TRY(x.p_uint8(1));
 		TRY(x.p_proptag_a(*d.pproptags));
-		return pack_result::ok;
 	}
+	return pack_result::ok;
 }
 
 static pack_result zrpc_pull(PULL_CTX &x, zcresp_getpropvals &d)
@@ -666,11 +664,11 @@ static pack_result zrpc_pull(PULL_CTX &x, zcresp_getpropvals &d)
 	return pack_result::ok;
 }
 
-static pack_result zrpc_push(PUSH_CTX &x, const zcreq_deletepropvals &d)
+static pack_result zrpc_push(PUSH_CTX &x, const zcreq_deletepropvals_v &d)
 {
 	TRY(x.p_guid(d.hsession));
 	TRY(x.p_uint32(d.hobject));
-	TRY(x.p_proptag_a(*d.pproptags));
+	TRY(x.p_proptag_a(d.pproptags));
 	return pack_result::ok;
 }
 
@@ -724,11 +722,11 @@ static pack_result zrpc_pull(PULL_CTX &x, zcresp_getpropnames &d)
 	return pack_result::ok;
 }
 
-static pack_result zrpc_push(PUSH_CTX &x, const zcreq_copyto &d)
+static pack_result zrpc_push(PUSH_CTX &x, const zcreq_copyto_v &d)
 {
 	TRY(x.p_guid(d.hsession));
 	TRY(x.p_uint32(d.hsrcobject));
-	TRY(x.p_proptag_a(*d.pexclude_proptags));
+	TRY(x.p_proptag_a(d.pexclude_proptags));
 	TRY(x.p_uint32(d.hdstobject));
 	TRY(x.p_uint32(d.flags));
 	return pack_result::ok;
@@ -1135,7 +1133,7 @@ pack_result rpc_ext_push_request(const zcreq *prequest, BINARY *pbin_out)
 	TRY(push_ctx.advance(sizeof(uint32_t)));
 	TRY(push_ctx.p_uint8(static_cast<uint8_t>(prequest->call_id)));
 	switch (prequest->call_id) {
-#define E(t) case zcore_callid::t: b_result = zrpc_push(push_ctx, *static_cast<const zcreq_ ## t *>(prequest)); break;
+#define E(t) case zcore_callid::t: b_result = zrpc_push(push_ctx, *static_cast<const zcreq_ ## t ::view_t *>(prequest)); break;
 	E(logon)
 	E(checksession)
 	E(uinfo)
