@@ -3,9 +3,9 @@
 #include <gromox/zcore_client.hpp>
 #include <gromox/zcore_rpc.hpp>
 #include "php.h"
-ec_error_t zclient_logon(const char *username, const char *password, const char *rhost, uint32_t flags, GUID *hsession)
+ec_error_t zclient_logon(const char *username, /*maybenull*/ const char *password, const char *rhost, uint32_t flags, GUID *hsession)
 {
-	zcreq_logon q{};
+	zcreq_logon::view_t q{};
 	zcresp_logon r{};
 
 	q.call_id = zcore_callid::logon;
@@ -23,7 +23,7 @@ ec_error_t zclient_logon(const char *username, const char *password, const char
 
 ec_error_t zclient_uinfo(const char *username, BINARY *entryid, std::string *pdisplay_name, std::string *px500dn, uint32_t *privilege_bits)
 {
-	zcreq_uinfo q{};
+	zcreq_uinfo::view_t q{};
 	zcresp_uinfo r{};
 
 	q.call_id = zcore_callid::uinfo;
@@ -41,7 +41,7 @@ ec_error_t zclient_uinfo(const char *username, BINARY *entryid, std::string *pdi
 
 ec_error_t zclient_unloadobject(GUID hsession, uint32_t hobject)
 {
-	zcreq_unloadobject q{};
+	zcreq_unloadobject::view_t q{};
 	zcresp_unloadobject r{};
 
 	q.call_id = zcore_callid::unloadobject;
@@ -54,7 +54,7 @@ ec_error_t zclient_unloadobject(GUID hsession, uint32_t hobject)
 
 ec_error_t zclient_openentry(GUID hsession, BINARY entryid, uint32_t flags, zs_objtype *mapi_type, uint32_t *hobject)
 {
-	zcreq_openentry q{};
+	zcreq_openentry::view_t q{};
 	zcresp_openentry r{};
 
 	q.call_id = zcore_callid::openentry;
@@ -72,7 +72,7 @@ ec_error_t zclient_openentry(GUID hsession, BINARY entryid, uint32_t flags, zs_o
 
 ec_error_t zclient_openstoreentry(GUID hsession, uint32_t hobject, BINARY entryid, uint32_t flags, zs_objtype *mapi_type, uint32_t *hxobject)
 {
-	zcreq_openstoreentry q{};
+	zcreq_openstoreentry::view_t q{};
 	zcresp_openstoreentry r{};
 
 	q.call_id = zcore_callid::openstoreentry;
@@ -91,7 +91,7 @@ ec_error_t zclient_openstoreentry(GUID hsession, uint32_t hobject, BINARY entryi
 
 ec_error_t zclient_openabentry(GUID hsession, BINARY entryid, zs_objtype *mapi_type, uint32_t *hobject)
 {
-	zcreq_openabentry q{};
+	zcreq_openabentry::view_t q{};
 	zcresp_openabentry r{};
 
 	q.call_id = zcore_callid::openabentry;
@@ -108,7 +108,7 @@ ec_error_t zclient_openabentry(GUID hsession, BINARY entryid, zs_objtype *mapi_t
 
 ec_error_t zclient_resolvename(GUID hsession, const TARRAY_SET *pcond_set, TARRAY_SET *result_set)
 {
-	zcreq_resolvename q{};
+	zcreq_resolvename::view_t q{};
 	zcresp_resolvename r{};
 
 	q.call_id = zcore_callid::resolvename;
@@ -124,7 +124,7 @@ ec_error_t zclient_resolvename(GUID hsession, const TARRAY_SET *pcond_set, TARRA
 
 ec_error_t zclient_getpermissions(GUID hsession, uint32_t hobject, PERMISSION_SET *perm_set)
 {
-	zcreq_getpermissions q{};
+	zcreq_getpermissions::view_t q{};
 	zcresp_getpermissions r{};
 
 	q.call_id = zcore_callid::getpermissions;
@@ -140,7 +140,7 @@ ec_error_t zclient_getpermissions(GUID hsession, uint32_t hobject, PERMISSION_SE
 
 ec_error_t zclient_modifypermissions(GUID hsession, uint32_t hfolder, const PERMISSION_SET *pset)
 {
-	zcreq_modifypermissions q{};
+	zcreq_modifypermissions::view_t q{};
 	zcresp_modifypermissions r{};
 
 	q.call_id = zcore_callid::modifypermissions;
@@ -154,7 +154,7 @@ ec_error_t zclient_modifypermissions(GUID hsession, uint32_t hfolder, const PERM
 
 ec_error_t zclient_modifyrules(GUID hsession, uint32_t hfolder, uint32_t flags, const RULE_LIST *plist)
 {
-	zcreq_modifyrules q{};
+	zcreq_modifyrules::view_t q{};
 	zcresp_modifyrules r{};
 
 	q.call_id = zcore_callid::modifyrules;
@@ -169,7 +169,7 @@ ec_error_t zclient_modifyrules(GUID hsession, uint32_t hfolder, uint32_t flags,
 
 ec_error_t zclient_getabgal(GUID hsession, BINARY *entryid)
 {
-	zcreq_getabgal q{};
+	zcreq_getabgal::view_t q{};
 	zcresp_getabgal r{};
 
 	q.call_id = zcore_callid::getabgal;
@@ -184,7 +184,7 @@ ec_error_t zclient_getabgal(GUID hsession, BINARY *entryid)
 
 ec_error_t zclient_loadstoretable(GUID hsession, uint32_t *hobject)
 {
-	zcreq_loadstoretable q{};
+	zcreq_loadstoretable::view_t q{};
 	zcresp_loadstoretable r{};
 
 	q.call_id = zcore_callid::loadstoretable;
@@ -199,7 +199,7 @@ ec_error_t zclient_loadstoretable(GUID hsession, uint32_t *hobject)
 
 ec_error_t zclient_openstore(GUID hsession, BINARY entryid, uint32_t *hobject)
 {
-	zcreq_openstore q{};
+	zcreq_openstore::view_t q{};
 	zcresp_openstore r{};
 
 	q.call_id = zcore_callid::openstore;
@@ -213,9 +213,9 @@ ec_error_t zclient_openstore(GUID hsession, BINARY entryid, uint32_t *hobject)
 	return r.result;
 }
 
-ec_error_t zclient_openprofilesec(GUID hsession, const FLATUID *puid, uint32_t *hobject)
+ec_error_t zclient_openprofilesec(GUID hsession, /*maybenull*/ const FLATUID *puid, uint32_t *hobject)
 {
-	zcreq_openprofilesec q{};
+	zcreq_openprofilesec::view_t q{};
 	zcresp_openprofilesec r{};
 
 	q.call_id = zcore_callid::openprofilesec;
@@ -231,7 +231,7 @@ ec_error_t zclient_openprofilesec(GUID hsession, const FLATUID *puid, uint32_t *
 
 ec_error_t zclient_loadhierarchytable(GUID hsession, uint32_t hfolder, uint32_t flags, uint32_t *hobject)
 {
-	zcreq_loadhierarchytable q{};
+	zcreq_loadhierarchytable::view_t q{};
 	zcresp_loadhierarchytable r{};
 
 	q.call_id = zcore_callid::loadhierarchytable;
@@ -248,7 +248,7 @@ ec_error_t zclient_loadhierarchytable(GUID hsession, uint32_t hfolder, uint32_t
 
 ec_error_t zclient_loadcontenttable(GUID hsession, uint32_t hfolder, uint32_t flags, uint32_t *hobject)
 {
-	zcreq_loadcontenttable q{};
+	zcreq_loadcontenttable::view_t q{};
 	zcresp_loadcontenttable r{};
 
 	q.call_id = zcore_callid::loadcontenttable;
@@ -265,7 +265,7 @@ ec_error_t zclient_loadcontenttable(GUID hsession, uint32_t hfolder, uint32_t fl
 
 ec_error_t zclient_loadrecipienttable(GUID hsession, uint32_t hmessage, uint32_t *hobject)
 {
-	zcreq_loadrecipienttable q{};
+	zcreq_loadrecipienttable::view_t q{};
 	zcresp_loadrecipienttable r{};
 
 	q.call_id = zcore_callid::loadrecipienttable;
@@ -281,7 +281,7 @@ ec_error_t zclient_loadrecipienttable(GUID hsession, uint32_t hmessage, uint32_t
 
 ec_error_t zclient_loadruletable(GUID hsession, uint32_t hfolder, uint32_t *hobject)
 {
-	zcreq_loadruletable q{};
+	zcreq_loadruletable::view_t q{};
 	zcresp_loadruletable r{};
 
 	q.call_id = zcore_callid::loadruletable;
@@ -297,7 +297,7 @@ ec_error_t zclient_loadruletable(GUID hsession, uint32_t hfolder, uint32_t *hobj
 
 ec_error_t zclient_createmessage(GUID hsession, uint32_t hfolder, uint32_t flags, uint32_t *hobject)
 {
-	zcreq_createmessage q{};
+	zcreq_createmessage::view_t q{};
 	zcresp_createmessage r{};
 
 	q.call_id = zcore_callid::createmessage;
@@ -314,7 +314,7 @@ ec_error_t zclient_createmessage(GUID hsession, uint32_t hfolder, uint32_t flags
 
 ec_error_t zclient_deletemessages(GUID hsession, uint32_t hfolder, const BINARY_ARRAY *pentryids, uint32_t flags)
 {
-	zcreq_deletemessages q{};
+	zcreq_deletemessages::view_t q{};
 	zcresp_deletemessages r{};
 
 	q.call_id = zcore_callid::deletemessages;
@@ -329,7 +329,7 @@ ec_error_t zclient_deletemessages(GUID hsession, uint32_t hfolder, const BINARY_
 
 ec_error_t zclient_copymessages(GUID hsession, uint32_t hsrcfolder, uint32_t hdstfolder, const BINARY_ARRAY *pentryids, uint32_t flags)
 {
-	zcreq_copymessages q{};
+	zcreq_copymessages::view_t q{};
 	zcresp_copymessages r{};
 
 	q.call_id = zcore_callid::copymessages;
@@ -345,7 +345,7 @@ ec_error_t zclient_copymessages(GUID hsession, uint32_t hsrcfolder, uint32_t hds
 
 ec_error_t zclient_setreadflags(GUID hsession, uint32_t hfolder, const BINARY_ARRAY *pentryids, uint32_t flags)
 {
-	zcreq_setreadflags q{};
+	zcreq_setreadflags::view_t q{};
 	zcresp_setreadflags r{};
 
 	q.call_id = zcore_callid::setreadflags;
@@ -360,7 +360,7 @@ ec_error_t zclient_setreadflags(GUID hsession, uint32_t hfolder, const BINARY_AR
 
 ec_error_t zclient_createfolder(GUID hsession, uint32_t hparent_folder, uint32_t folder_type, const char *folder_name, const char *folder_comment, uint32_t flags, uint32_t *hobject)
 {
-	zcreq_createfolder q{};
+	zcreq_createfolder::view_t q{};
 	zcresp_createfolder r{};
 
 	q.call_id = zcore_callid::createfolder;
@@ -380,7 +380,7 @@ ec_error_t zclient_createfolder(GUID hsession, uint32_t hparent_folder, uint32_t
 
 ec_error_t zclient_deletefolder(GUID hsession, uint32_t hparent_folder, BINARY entryid, uint32_t flags)
 {
-	zcreq_deletefolder q{};
+	zcreq_deletefolder::view_t q{};
 	zcresp_deletefolder r{};
 
 	q.call_id = zcore_callid::deletefolder;
@@ -395,7 +395,7 @@ ec_error_t zclient_deletefolder(GUID hsession, uint32_t hparent_folder, BINARY e
 
 ec_error_t zclient_emptyfolder(GUID hsession, uint32_t hfolder, uint32_t flags)
 {
-	zcreq_emptyfolder q{};
+	zcreq_emptyfolder::view_t q{};
 	zcresp_emptyfolder r{};
 
 	q.call_id = zcore_callid::emptyfolder;
@@ -407,9 +407,9 @@ ec_error_t zclient_emptyfolder(GUID hsession, uint32_t hfolder, uint32_t flags)
 	return r.result;
 }
 
-ec_error_t zclient_copyfolder(GUID hsession, uint32_t hsrc_folder, BINARY entryid, uint32_t hdst_folder, const char *new_name, uint32_t flags)
+ec_error_t zclient_copyfolder(GUID hsession, uint32_t hsrc_folder, BINARY entryid, uint32_t hdst_folder, /*maybenull*/ const char *new_name, uint32_t flags)
 {
-	zcreq_copyfolder q{};
+	zcreq_copyfolder::view_t q{};
 	zcresp_copyfolder r{};
 
 	q.call_id = zcore_callid::copyfolder;
@@ -426,7 +426,7 @@ ec_error_t zclient_copyfolder(GUID hsession, uint32_t hsrc_folder, BINARY entryi
 
 ec_error_t zclient_getstoreentryid(const char *mailbox_dn, BINARY *entryid)
 {
-	zcreq_getstoreentryid q{};
+	zcreq_getstoreentryid::view_t q{};
 	zcresp_getstoreentryid r{};
 
 	q.call_id = zcore_callid::getstoreentryid;
@@ -439,9 +439,9 @@ ec_error_t zclient_getstoreentryid(const char *mailbox_dn, BINARY *entryid)
 	return r.result;
 }
 
-ec_error_t zclient_entryidfromsourcekey(GUID hsession, uint32_t hstore, BINARY folder_key, const BINARY *pmessage_key, BINARY *entryid)
+ec_error_t zclient_entryidfromsourcekey(GUID hsession, uint32_t hstore, BINARY folder_key, /*maybenull*/ const BINARY *pmessage_key, BINARY *entryid)
 {
-	zcreq_entryidfromsourcekey q{};
+	zcreq_entryidfromsourcekey::view_t q{};
 	zcresp_entryidfromsourcekey r{};
 
 	q.call_id = zcore_callid::entryidfromsourcekey;
@@ -457,9 +457,9 @@ ec_error_t zclient_entryidfromsourcekey(GUID hsession, uint32_t hstore, BINARY f
 	return r.result;
 }
 
-ec_error_t zclient_storeadvise(GUID hsession, uint32_t hstore, const BINARY *pentryid, uint32_t event_mask, uint32_t *sub_id)
+ec_error_t zclient_storeadvise(GUID hsession, uint32_t hstore, /*maybenull*/ const BINARY *pentryid, uint32_t event_mask, uint32_t *sub_id)
 {
-	zcreq_storeadvise q{};
+	zcreq_storeadvise::view_t q{};
 	zcresp_storeadvise r{};
 
 	q.call_id = zcore_callid::storeadvise;
@@ -477,7 +477,7 @@ ec_error_t zclient_storeadvise(GUID hsession, uint32_t hstore, const BINARY *pen
 
 ec_error_t zclient_unadvise(GUID hsession, uint32_t hstore, uint32_t sub_id)
 {
-	zcreq_unadvise q{};
+	zcreq_unadvise::view_t q{};
 	zcresp_unadvise r{};
 
 	q.call_id = zcore_callid::unadvise;
@@ -491,7 +491,7 @@ ec_error_t zclient_unadvise(GUID hsession, uint32_t hstore, uint32_t sub_id)
 
 ec_error_t zclient_notifdequeue(const NOTIF_SINK *psink, uint32_t timeval, std::vector<ZNOTIFICATION> *notifications)
 {
-	zcreq_notifdequeue q{};
+	zcreq_notifdequeue::view_t q{};
 	zcresp_notifdequeue r{};
 
 	q.call_id = zcore_callid::notifdequeue;
@@ -505,9 +505,9 @@ ec_error_t zclient_notifdequeue(const NOTIF_SINK *psink, uint32_t timeval, std::
 	return r.result;
 }
 
-ec_error_t zclient_queryrows(GUID hsession, uint32_t htable, uint32_t start, uint32_t count, const RESTRICTION *prestriction, const PROPTAG_ARRAY *pproptags, TARRAY_SET *rowset)
+ec_error_t zclient_queryrows(GUID hsession, uint32_t htable, uint32_t start, uint32_t count, /*maybenull*/ const RESTRICTION *prestriction, const std::optional<std::vector<gromox::proptag_t>> &pproptags, TARRAY_SET *rowset)
 {
-	zcreq_queryrows q{};
+	zcreq_queryrows::view_t q{};
 	zcresp_queryrows r{};
 
 	q.call_id = zcore_callid::queryrows;
@@ -516,7 +516,7 @@ ec_error_t zclient_queryrows(GUID hsession, uint32_t htable, uint32_t start, uin
 	q.start = start;
 	q.count = count;
 	q.prestriction = deconst(prestriction);
-	q.pproptags = deconst(pproptags);
+	q.pproptags = pproptags;
 	if (!zclient_do_rpc(&q, &r))
 		return ecRpcFailed;
 	if (r.result != ecSuccess)
@@ -525,15 +525,15 @@ ec_error_t zclient_queryrows(GUID hsession, uint32_t htable, uint32_t start, uin
 	return r.result;
 }
 
-ec_error_t zclient_setcolumns(GUID hsession, uint32_t htable, const PROPTAG_ARRAY *pproptags, uint32_t flags)
+ec_error_t zclient_setcolumns(GUID hsession, uint32_t htable, proptag_cspan pproptags, uint32_t flags)
 {
-	zcreq_setcolumns q{};
+	zcreq_setcolumns::view_t q{};
 	zcresp_setcolumns r{};
 
 	q.call_id = zcore_callid::setcolumns;
 	q.hsession = hsession;
 	q.htable = htable;
-	q.pproptags = deconst(pproptags);
+	q.pproptags = pproptags;
 	q.flags = flags;
 	if (!zclient_do_rpc(&q, &r))
 		return ecRpcFailed;
@@ -542,7 +542,7 @@ ec_error_t zclient_setcolumns(GUID hsession, uint32_t htable, const PROPTAG_ARRA
 
 ec_error_t zclient_seekrow(GUID hsession, uint32_t htable, uint32_t bookmark, int32_t seek_rows, int32_t *sought_rows)
 {
-	zcreq_seekrow q{};
+	zcreq_seekrow::view_t q{};
 	zcresp_seekrow r{};
 
 	q.call_id = zcore_callid::seekrow;
@@ -560,7 +560,7 @@ ec_error_t zclient_seekrow(GUID hsession, uint32_t htable, uint32_t bookmark, in
 
 ec_error_t zclient_sorttable(GUID hsession, uint32_t htable, const SORTORDER_SET *psortset)
 {
-	zcreq_sorttable q{};
+	zcreq_sorttable::view_t q{};
 	zcresp_sorttable r{};
 
 	q.call_id = zcore_callid::sorttable;
@@ -574,7 +574,7 @@ ec_error_t zclient_sorttable(GUID hsession, uint32_t htable, const SORTORDER_SET
 
 ec_error_t zclient_getrowcount(GUID hsession, uint32_t htable, uint32_t *count)
 {
-	zcreq_getrowcount q{};
+	zcreq_getrowcount::view_t q{};
 	zcresp_getrowcount r{};
 
 	q.call_id = zcore_callid::getrowcount;
@@ -590,7 +590,7 @@ ec_error_t zclient_getrowcount(GUID hsession, uint32_t htable, uint32_t *count)
 
 ec_error_t zclient_restricttable(GUID hsession, uint32_t htable, const RESTRICTION *prestriction, uint32_t flags)
 {
-	zcreq_restricttable q{};
+	zcreq_restricttable::view_t q{};
 	zcresp_restricttable r{};
 
 	q.call_id = zcore_callid::restricttable;
@@ -605,7 +605,7 @@ ec_error_t zclient_restricttable(GUID hsession, uint32_t htable, const RESTRICTI
 
 ec_error_t zclient_findrow(GUID hsession, uint32_t htable, uint32_t bookmark, const RESTRICTION *prestriction, uint32_t flags, uint32_t *row_idx)
 {
-	zcreq_findrow q{};
+	zcreq_findrow::view_t q{};
 	zcresp_findrow r{};
 
 	q.call_id = zcore_callid::findrow;
@@ -624,7 +624,7 @@ ec_error_t zclient_findrow(GUID hsession, uint32_t htable, uint32_t bookmark, co
 
 ec_error_t zclient_createbookmark(GUID hsession, uint32_t htable, uint32_t *bookmark)
 {
-	zcreq_createbookmark q{};
+	zcreq_createbookmark::view_t q{};
 	zcresp_createbookmark r{};
 
 	q.call_id = zcore_callid::createbookmark;
@@ -640,7 +640,7 @@ ec_error_t zclient_createbookmark(GUID hsession, uint32_t htable, uint32_t *book
 
 ec_error_t zclient_freebookmark(GUID hsession, uint32_t htable, uint32_t bookmark)
 {
-	zcreq_freebookmark q{};
+	zcreq_freebookmark::view_t q{};
 	zcresp_freebookmark r{};
 
 	q.call_id = zcore_callid::freebookmark;
@@ -652,9 +652,9 @@ ec_error_t zclient_freebookmark(GUID hsession, uint32_t htable, uint32_t bookmar
 	return r.result;
 }
 
-ec_error_t zclient_getreceivefolder(GUID hsession, uint32_t hstore, const char *pstrclass, BINARY *entryid)
+ec_error_t zclient_getreceivefolder(GUID hsession, uint32_t hstore, /*maybenull*/ const char *pstrclass, BINARY *entryid)
 {
-	zcreq_getreceivefolder q{};
+	zcreq_getreceivefolder::view_t q{};
 	zcresp_getreceivefolder r{};
 
 	q.call_id = zcore_callid::getreceivefolder;
@@ -671,7 +671,7 @@ ec_error_t zclient_getreceivefolder(GUID hsession, uint32_t hstore, const char *
 
 ec_error_t zclient_modifyrecipients(GUID hsession, uint32_t hmessage, uint32_t flags, const TARRAY_SET *prcpt_list)
 {
-	zcreq_modifyrecipients q{};
+	zcreq_modifyrecipients::view_t q{};
 	zcresp_modifyrecipients r{};
 
 	q.call_id = zcore_callid::modifyrecipients;
@@ -686,7 +686,7 @@ ec_error_t zclient_modifyrecipients(GUID hsession, uint32_t hmessage, uint32_t f
 
 ec_error_t zclient_submitmessage(GUID hsession, uint32_t hmessage)
 {
-	zcreq_submitmessage q{};
+	zcreq_submitmessage::view_t q{};
 	zcresp_submitmessage r{};
 
 	q.call_id = zcore_callid::submitmessage;
@@ -699,7 +699,7 @@ ec_error_t zclient_submitmessage(GUID hsession, uint32_t hmessage)
 
 ec_error_t zclient_loadattachmenttable(GUID hsession, uint32_t hmessage, uint32_t *hobject)
 {
-	zcreq_loadattachmenttable q{};
+	zcreq_loadattachmenttable::view_t q{};
 	zcresp_loadattachmenttable r{};
 
 	q.call_id = zcore_callid::loadattachmenttable;
@@ -715,7 +715,7 @@ ec_error_t zclient_loadattachmenttable(GUID hsession, uint32_t hmessage, uint32_
 
 ec_error_t zclient_openattachment(GUID hsession, uint32_t hmessage, uint32_t attach_id, uint32_t *hobject)
 {
-	zcreq_openattachment q{};
+	zcreq_openattachment::view_t q{};
 	zcresp_openattachment r{};
 
 	q.call_id = zcore_callid::openattachment;
@@ -732,7 +732,7 @@ ec_error_t zclient_openattachment(GUID hsession, uint32_t hmessage, uint32_t att
 
 ec_error_t zclient_createattachment(GUID hsession, uint32_t hmessage, uint32_t *hobject)
 {
-	zcreq_createattachment q{};
+	zcreq_createattachment::view_t q{};
 	zcresp_createattachment r{};
 
 	q.call_id = zcore_callid::createattachment;
@@ -748,7 +748,7 @@ ec_error_t zclient_createattachment(GUID hsession, uint32_t hmessage, uint32_t *
 
 ec_error_t zclient_deleteattachment(GUID hsession, uint32_t hmessage, uint32_t attach_id)
 {
-	zcreq_deleteattachment q{};
+	zcreq_deleteattachment::view_t q{};
 	zcresp_deleteattachment r{};
 
 	q.call_id = zcore_callid::deleteattachment;
@@ -762,7 +762,7 @@ ec_error_t zclient_deleteattachment(GUID hsession, uint32_t hmessage, uint32_t a
 
 ec_error_t zclient_setpropvals(GUID hsession, uint32_t hobject, const TPROPVAL_ARRAY *ppropvals)
 {
-	zcreq_setpropvals q{};
+	zcreq_setpropvals::view_t q{};
 	zcresp_setpropvals r{};
 
 	q.call_id = zcore_callid::setpropvals;
@@ -774,15 +774,15 @@ ec_error_t zclient_setpropvals(GUID hsession, uint32_t hobject, const TPROPVAL_A
 	return r.result;
 }
 
-ec_error_t zclient_getpropvals(GUID hsession, uint32_t hobject, const PROPTAG_ARRAY *pproptags, TPROPVAL_ARRAY *propvals)
+ec_error_t zclient_getpropvals(GUID hsession, uint32_t hobject, const std::optional<proptag_cspan> &pproptags, TPROPVAL_ARRAY *propvals)
 {
-	zcreq_getpropvals q{};
+	zcreq_getpropvals::view_t q{};
 	zcresp_getpropvals r{};
 
 	q.call_id = zcore_callid::getpropvals;
 	q.hsession = hsession;
 	q.hobject = hobject;
-	q.pproptags = deconst(pproptags);
+	q.pproptags = pproptags;
 	if (!zclient_do_rpc(&q, &r))
 		return ecRpcFailed;
 	if (r.result != ecSuccess)
@@ -791,15 +791,15 @@ ec_error_t zclient_getpropvals(GUID hsession, uint32_t hobject, const PROPTAG_AR
 	return r.result;
 }
 
-ec_error_t zclient_deletepropvals(GUID hsession, uint32_t hobject, const PROPTAG_ARRAY *pproptags)
+ec_error_t zclient_deletepropvals(GUID hsession, uint32_t hobject, proptag_cspan pproptags)
 {
-	zcreq_deletepropvals q{};
+	zcreq_deletepropvals::view_t q{};
 	zcresp_deletepropvals r{};
 
 	q.call_id = zcore_callid::deletepropvals;
 	q.hsession = hsession;
 	q.hobject = hobject;
-	q.pproptags = deconst(pproptags);
+	q.pproptags = pproptags;
 	if (!zclient_do_rpc(&q, &r))
 		return ecRpcFailed;
 	return r.result;
@@ -807,7 +807,7 @@ ec_error_t zclient_deletepropvals(GUID hsession, uint32_t hobject, const PROPTAG
 
 ec_error_t zclient_setmessagereadflag(GUID hsession, uint32_t hmessage, uint32_t flags)
 {
-	zcreq_setmessagereadflag q{};
+	zcreq_setmessagereadflag::view_t q{};
 	zcresp_setmessagereadflag r{};
 
 	q.call_id = zcore_callid::setmessagereadflag;
@@ -821,7 +821,7 @@ ec_error_t zclient_setmessagereadflag(GUID hsession, uint32_t hmessage, uint32_t
 
 ec_error_t zclient_openembedded(GUID hsession, uint32_t hattachment, uint32_t flags, uint32_t *hobject)
 {
-	zcreq_openembedded q{};
+	zcreq_openembedded::view_t q{};
 	zcresp_openembedded r{};
 
 	q.call_id = zcore_callid::openembedded;
@@ -838,7 +838,7 @@ ec_error_t zclient_openembedded(GUID hsession, uint32_t hattachment, uint32_t fl
 
 ec_error_t zclient_getnamedpropids(GUID hsession, uint32_t hstore, const PROPNAME_ARRAY *ppropnames, PROPID_ARRAY *propids)
 {
-	zcreq_getnamedpropids q{};
+	zcreq_getnamedpropids::view_t q{};
 	zcresp_getnamedpropids r{};
 
 	q.call_id = zcore_callid::getnamedpropids;
@@ -855,7 +855,7 @@ ec_error_t zclient_getnamedpropids(GUID hsession, uint32_t hstore, const PROPNAM
 
 ec_error_t zclient_getpropnames(GUID hsession, uint32_t hstore, const PROPID_ARRAY &ppropids, PROPNAME_ARRAY *propnames)
 {
-	zcreq_getpropnames q{};
+	zcreq_getpropnames::view_t q{};
 	zcresp_getpropnames r{};
 
 	q.call_id = zcore_callid::getpropnames;
@@ -870,15 +870,15 @@ ec_error_t zclient_getpropnames(GUID hsession, uint32_t hstore, const PROPID_ARR
 	return r.result;
 }
 
-ec_error_t zclient_copyto(GUID hsession, uint32_t hsrcobject, const PROPTAG_ARRAY *pexclude_proptags, uint32_t hdstobject, uint32_t flags)
+ec_error_t zclient_copyto(GUID hsession, uint32_t hsrcobject, proptag_cspan pexclude_proptags, uint32_t hdstobject, uint32_t flags)
 {
-	zcreq_copyto q{};
+	zcreq_copyto::view_t q{};
 	zcresp_copyto r{};
 
 	q.call_id = zcore_callid::copyto;
 	q.hsession = hsession;
 	q.hsrcobject = hsrcobject;
-	q.pexclude_proptags = deconst(pexclude_proptags);
+	q.pexclude_proptags = pexclude_proptags;
 	q.hdstobject = hdstobject;
 	q.flags = flags;
 	if (!zclient_do_rpc(&q, &r))
@@ -888,7 +888,7 @@ ec_error_t zclient_copyto(GUID hsession, uint32_t hsrcobject, const PROPTAG_ARRA
 
 ec_error_t zclient_savechanges(GUID hsession, uint32_t hobject)
 {
-	zcreq_savechanges q{};
+	zcreq_savechanges::view_t q{};
 	zcresp_savechanges r{};
 
 	q.call_id = zcore_callid::savechanges;
@@ -901,7 +901,7 @@ ec_error_t zclient_savechanges(GUID hsession, uint32_t hobject)
 
 ec_error_t zclient_hierarchysync(GUID hsession, uint32_t hfolder, uint32_t *hobject)
 {
-	zcreq_hierarchysync q{};
+	zcreq_hierarchysync::view_t q{};
 	zcresp_hierarchysync r{};
 
 	q.call_id = zcore_callid::hierarchysync;
@@ -917,7 +917,7 @@ ec_error_t zclient_hierarchysync(GUID hsession, uint32_t hfolder, uint32_t *hobj
 
 ec_error_t zclient_contentsync(GUID hsession, uint32_t hfolder, uint32_t *hobject)
 {
-	zcreq_contentsync q{};
+	zcreq_contentsync::view_t q{};
 	zcresp_contentsync r{};
 
 	q.call_id = zcore_callid::contentsync;
@@ -931,9 +931,9 @@ ec_error_t zclient_contentsync(GUID hsession, uint32_t hfolder, uint32_t *hobjec
 	return r.result;
 }
 
-ec_error_t zclient_configsync(GUID hsession, uint32_t hctx, uint32_t flags, const BINARY *pstate, const RESTRICTION *prestriction, uint8_t *b_changed, uint32_t *count)
+ec_error_t zclient_configsync(GUID hsession, uint32_t hctx, uint32_t flags, const BINARY *pstate, /*maybenull*/ const RESTRICTION *prestriction, uint8_t *b_changed, uint32_t *count)
 {
-	zcreq_configsync q{};
+	zcreq_configsync::view_t q{};
 	zcresp_configsync r{};
 
 	q.call_id = zcore_callid::configsync;
@@ -953,7 +953,7 @@ ec_error_t zclient_configsync(GUID hsession, uint32_t hctx, uint32_t flags, cons
 
 ec_error_t zclient_statesync(GUID hsession, uint32_t hctx, BINARY *state)
 {
-	zcreq_statesync q{};
+	zcreq_statesync::view_t q{};
 	zcresp_statesync r{};
 
 	q.call_id = zcore_callid::statesync;
@@ -969,7 +969,7 @@ ec_error_t zclient_statesync(GUID hsession, uint32_t hctx, BINARY *state)
 
 ec_error_t zclient_syncmessagechange(GUID hsession, uint32_t hctx, uint8_t *b_new, TPROPVAL_ARRAY *proplist)
 {
-	zcreq_syncmessagechange q{};
+	zcreq_syncmessagechange::view_t q{};
 	zcresp_syncmessagechange r{};
 
 	q.call_id = zcore_callid::syncmessagechange;
@@ -986,7 +986,7 @@ ec_error_t zclient_syncmessagechange(GUID hsession, uint32_t hctx, uint8_t *b_ne
 
 ec_error_t zclient_syncfolderchange(GUID hsession, uint32_t hctx, TPROPVAL_ARRAY *proplist)
 {
-	zcreq_syncfolderchange q{};
+	zcreq_syncfolderchange::view_t q{};
 	zcresp_syncfolderchange r{};
 
 	q.call_id = zcore_callid::syncfolderchange;
@@ -1002,7 +1002,7 @@ ec_error_t zclient_syncfolderchange(GUID hsession, uint32_t hctx, TPROPVAL_ARRAY
 
 ec_error_t zclient_syncreadstatechanges(GUID hsession, uint32_t hctx, STATE_ARRAY *states)
 {
-	zcreq_syncreadstatechanges q{};
+	zcreq_syncreadstatechanges::view_t q{};
 	zcresp_syncreadstatechanges r{};
 
 	q.call_id = zcore_callid::syncreadstatechanges;
@@ -1018,7 +1018,7 @@ ec_error_t zclient_syncreadstatechanges(GUID hsession, uint32_t hctx, STATE_ARRA
 
 ec_error_t zclient_syncdeletions(GUID hsession, uint32_t hctx, uint32_t flags, BINARY_ARRAY *bins)
 {
-	zcreq_syncdeletions q{};
+	zcreq_syncdeletions::view_t q{};
 	zcresp_syncdeletions r{};
 
 	q.call_id = zcore_callid::syncdeletions;
@@ -1035,7 +1035,7 @@ ec_error_t zclient_syncdeletions(GUID hsession, uint32_t hctx, uint32_t flags, B
 
 ec_error_t zclient_hierarchyimport(GUID hsession, uint32_t hfolder, uint32_t *hobject)
 {
-	zcreq_hierarchyimport q{};
+	zcreq_hierarchyimport::view_t q{};
 	zcresp_hierarchyimport r{};
 
 	q.call_id = zcore_callid::hierarchyimport;
@@ -1051,7 +1051,7 @@ ec_error_t zclient_hierarchyimport(GUID hsession, uint32_t hfolder, uint32_t *ho
 
 ec_error_t zclient_contentimport(GUID hsession, uint32_t hfolder, uint32_t *hobject)
 {
-	zcreq_contentimport q{};
+	zcreq_contentimport::view_t q{};
 	zcresp_contentimport r{};
 
 	q.call_id = zcore_callid::contentimport;
@@ -1067,7 +1067,7 @@ ec_error_t zclient_contentimport(GUID hsession, uint32_t hfolder, uint32_t *hobj
 
 ec_error_t zclient_configimport(GUID hsession, uint32_t hctx, uint8_t sync_type, const BINARY *pstate)
 {
-	zcreq_configimport q{};
+	zcreq_configimport::view_t q{};
 	zcresp_configimport r{};
 
 	q.call_id = zcore_callid::configimport;
@@ -1082,7 +1082,7 @@ ec_error_t zclient_configimport(GUID hsession, uint32_t hctx, uint8_t sync_type,
 
 ec_error_t zclient_stateimport(GUID hsession, uint32_t hctx, BINARY *state)
 {
-	zcreq_stateimport q{};
+	zcreq_stateimport::view_t q{};
 	zcresp_stateimport r{};
 
 	q.call_id = zcore_callid::stateimport;
@@ -1098,7 +1098,7 @@ ec_error_t zclient_stateimport(GUID hsession, uint32_t hctx, BINARY *state)
 
 ec_error_t zclient_importmessage(GUID hsession, uint32_t hctx, uint32_t flags, const TPROPVAL_ARRAY *pproplist, uint32_t *hobject)
 {
-	zcreq_importmessage q{};
+	zcreq_importmessage::view_t q{};
 	zcresp_importmessage r{};
 
 	q.call_id = zcore_callid::importmessage;
@@ -1116,7 +1116,7 @@ ec_error_t zclient_importmessage(GUID hsession, uint32_t hctx, uint32_t flags, c
 
 ec_error_t zclient_importfolder(GUID hsession, uint32_t hctx, const TPROPVAL_ARRAY *pproplist)
 {
-	zcreq_importfolder q{};
+	zcreq_importfolder::view_t q{};
 	zcresp_importfolder r{};
 
 	q.call_id = zcore_callid::importfolder;
@@ -1130,7 +1130,7 @@ ec_error_t zclient_importfolder(GUID hsession, uint32_t hctx, const TPROPVAL_ARR
 
 ec_error_t zclient_importdeletion(GUID hsession, uint32_t hctx, uint32_t flags, const BINARY_ARRAY *pbins)
 {
-	zcreq_importdeletion q{};
+	zcreq_importdeletion::view_t q{};
 	zcresp_importdeletion r{};
 
 	q.call_id = zcore_callid::importdeletion;
@@ -1145,7 +1145,7 @@ ec_error_t zclient_importdeletion(GUID hsession, uint32_t hctx, uint32_t flags,
 
 ec_error_t zclient_importreadstates(GUID hsession, uint32_t hctx, const STATE_ARRAY *pstates)
 {
-	zcreq_importreadstates q{};
+	zcreq_importreadstates::view_t q{};
 	zcresp_importreadstates r{};
 
 	q.call_id = zcore_callid::importreadstates;
@@ -1157,9 +1157,9 @@ ec_error_t zclient_importreadstates(GUID hsession, uint32_t hctx, const STATE_AR
 	return r.result;
 }
 
-ec_error_t zclient_getsearchcriteria(GUID hsession, uint32_t hfolder, BINARY_ARRAY *folder_array, RESTRICTION **prestriction, uint32_t *search_stat)
+ec_error_t zclient_getsearchcriteria(GUID hsession, uint32_t hfolder, BINARY_ARRAY *folder_array, /*mayoutnull*/ RESTRICTION **prestriction, uint32_t *search_stat)
 {
-	zcreq_getsearchcriteria q{};
+	zcreq_getsearchcriteria::view_t q{};
 	zcresp_getsearchcriteria r{};
 
 	q.call_id = zcore_callid::getsearchcriteria;
@@ -1175,9 +1175,9 @@ ec_error_t zclient_getsearchcriteria(GUID hsession, uint32_t hfolder, BINARY_ARR
 	return r.result;
 }
 
-ec_error_t zclient_setsearchcriteria(GUID hsession, uint32_t hfolder, uint32_t flags, const BINARY_ARRAY *pfolder_array, const RESTRICTION *prestriction)
+ec_error_t zclient_setsearchcriteria(GUID hsession, uint32_t hfolder, uint32_t flags, const BINARY_ARRAY *pfolder_array, /*maybenull*/ const RESTRICTION *prestriction)
 {
-	zcreq_setsearchcriteria q{};
+	zcreq_setsearchcriteria::view_t q{};
 	zcresp_setsearchcriteria r{};
 
 	q.call_id = zcore_callid::setsearchcriteria;
@@ -1193,7 +1193,7 @@ ec_error_t zclient_setsearchcriteria(GUID hsession, uint32_t hfolder, uint32_t f
 
 ec_error_t zclient_messagetorfc822(GUID hsession, uint32_t hmessage, BINARY *eml_bin)
 {
-	zcreq_messagetorfc822 q{};
+	zcreq_messagetorfc822::view_t q{};
 	zcresp_messagetorfc822 r{};
 
 	q.call_id = zcore_callid::messagetorfc822;
@@ -1209,7 +1209,7 @@ ec_error_t zclient_messagetorfc822(GUID hsession, uint32_t hmessage, BINARY *eml
 
 ec_error_t zclient_rfc822tomessage(GUID hsession, uint32_t hmessage, uint32_t mxf_flags, const BINARY *peml_bin)
 {
-	zcreq_rfc822tomessage q{};
+	zcreq_rfc822tomessage::view_t q{};
 	zcresp_rfc822tomessage r{};
 
 	q.call_id = zcore_callid::rfc822tomessage;
@@ -1224,7 +1224,7 @@ ec_error_t zclient_rfc822tomessage(GUID hsession, uint32_t hmessage, uint32_t mx
 
 ec_error_t zclient_messagetoical(GUID hsession, uint32_t hmessage, BINARY *ical_bin)
 {
-	zcreq_messagetoical q{};
+	zcreq_messagetoical::view_t q{};
 	zcresp_messagetoical r{};
 
 	q.call_id = zcore_callid::messagetoical;
@@ -1240,7 +1240,7 @@ ec_error_t zclient_messagetoical(GUID hsession, uint32_t hmessage, BINARY *ical_
 
 ec_error_t zclient_icaltomessage(GUID hsession, uint32_t hmessage, const BINARY *pical_bin)
 {
-	zcreq_icaltomessage q{};
+	zcreq_icaltomessage::view_t q{};
 	zcresp_icaltomessage r{};
 
 	q.call_id = zcore_callid::icaltomessage;
@@ -1254,7 +1254,7 @@ ec_error_t zclient_icaltomessage(GUID hsession, uint32_t hmessage, const BINARY
 
 ec_error_t zclient_messagetovcf(GUID hsession, uint32_t hmessage, BINARY *vcf_bin)
 {
-	zcreq_messagetovcf q{};
+	zcreq_messagetovcf::view_t q{};
 	zcresp_messagetovcf r{};
 
 	q.call_id = zcore_callid::messagetovcf;
@@ -1270,7 +1270,7 @@ ec_error_t zclient_messagetovcf(GUID hsession, uint32_t hmessage, BINARY *vcf_bi
 
 ec_error_t zclient_vcftomessage(GUID hsession, uint32_t hmessage, const BINARY *pvcf_bin)
 {
-	zcreq_vcftomessage q{};
+	zcreq_vcftomessage::view_t q{};
 	zcresp_vcftomessage r{};
 
 	q.call_id = zcore_callid::vcftomessage;
@@ -1284,7 +1284,7 @@ ec_error_t zclient_vcftomessage(GUID hsession, uint32_t hmessage, const BINARY *
 
 ec_error_t zclient_setpasswd(const char *username, const char *passwd, const char *new_passwd)
 {
-	zcreq_setpasswd q{};
+	zcreq_setpasswd::view_t q{};
 	zcresp_setpasswd r{};
 
 	q.call_id = zcore_callid::setpasswd;
@@ -1298,7 +1298,7 @@ ec_error_t zclient_setpasswd(const char *username, const char *passwd, const cha
 
 ec_error_t zclient_linkmessage(GUID hsession, BINARY search_entryid, BINARY message_entryid)
 {
-	zcreq_linkmessage q{};
+	zcreq_linkmessage::view_t q{};
 	zcresp_linkmessage r{};
 
 	q.call_id = zcore_callid::linkmessage;
@@ -1312,7 +1312,7 @@ ec_error_t zclient_linkmessage(GUID hsession, BINARY search_entryid, BINARY mess
 
 ec_error_t zclient_checksession(GUID hsession)
 {
-	zcreq_checksession q{};
+	zcreq_checksession::view_t q{};
 	zcresp_checksession r{};
 
 	q.call_id = zcore_callid::checksession;
@@ -1324,7 +1324,7 @@ ec_error_t zclient_checksession(GUID hsession)
 
 ec_error_t zclient_imtomessage2(GUID session, uint32_t folder, uint32_t data_type, const char *im_data, LONG_ARRAY *msg_handles)
 {
-	zcreq_imtomessage2 q{};
+	zcreq_imtomessage2::view_t q{};
 	zcresp_imtomessage2 r{};
 
 	q.call_id = zcore_callid::imtomessage2;
@@ -1342,7 +1342,7 @@ ec_error_t zclient_imtomessage2(GUID session, uint32_t folder, uint32_t data_typ
 
 ec_error_t zclient_essdn_to_username(const char *essdn, char **username)
 {
-	zcreq_essdn_to_username q{};
+	zcreq_essdn_to_username::view_t q{};
 	zcresp_essdn_to_username r{};
 
 	q.call_id = zcore_callid::essdn_to_username;
@@ -1357,7 +1357,7 @@ ec_error_t zclient_essdn_to_username(const char *essdn, char **username)
 
 ec_error_t zclient_logon_token(const char *token, const char *rhost, GUID *hsession)
 {
-	zcreq_logon_token q{};
+	zcreq_logon_token::view_t q{};
 	zcresp_logon_token r{};
 
 	q.call_id = zcore_callid::logon_token;
@@ -1373,7 +1373,7 @@ ec_error_t zclient_logon_token(const char *token, const char *rhost, GUID *hsess
 
 ec_error_t zclient_getuserfreebusy(GUID hsession, BINARY entryid, time_t starttime, time_t endtime, std::vector<freebusy_event> *fb_events)
 {
-	zcreq_getuserfreebusy q{};
+	zcreq_getuserfreebusy::view_t q{};
 	zcresp_getuserfreebusy r{};
 
 	q.call_id = zcore_callid::getuserfreebusy;
@@ -1391,7 +1391,7 @@ ec_error_t zclient_getuserfreebusy(GUID hsession, BINARY entryid, time_t startti
 
 ec_error_t zclient_getuserfreebusyical(GUID hsession, BINARY entryid, time_t starttime, time_t endtime, BINARY *ical_bin)
 {
-	zcreq_getuserfreebusyical q{};
+	zcreq_getuserfreebusyical::view_t q{};
 	zcresp_getuserfreebusyical r{};
 
 	q.call_id = zcore_callid::getuserfreebusyical;
@@ -1409,7 +1409,7 @@ ec_error_t zclient_getuserfreebusyical(GUID hsession, BINARY entryid, time_t sta
 
 ec_error_t zclient_logon_np(const char *username, const char *password, const char *rhost, uint32_t flags, GUID *hsession)
 {
-	zcreq_logon_np q{};
+	zcreq_logon_np::view_t q{};
 	zcresp_logon_np r{};
 
 	q.call_id = zcore_callid::logon_np;
