@@ -9,8 +9,10 @@
 #include <limits>
 #include <libHX/endian.h>
 #include <libHX/string.h>
+#include <gromox/cookie_parser.hpp>
 #include <gromox/element_data.hpp>
 #include <gromox/ext_buffer.hpp>
+#include <gromox/fileio.h>
 #include <gromox/ical.hpp>
 #include <gromox/idset.hpp>
 #include <gromox/mail_func.hpp>
@@ -24,6 +26,29 @@
 #define assert(x) do { if (!(x)) { printf("%s failed\n", #x); return EXIT_FAILURE; } } while (false)
 using namespace gromox;
 
+static int t_cookie_jar()
+{
+	cookie_jar jar;
+	jar.add("a=1+; b=\"1+\"; d=1\"2\"3; e=%40+%4@%41%4; f= space ; g+ =5; g+ =6; Secure;");
+	assert(jar.size() == 7);
+	auto val = jar["a"];
+	assert(val != nullptr && strcmp(val, "1+") == 0);
+	val = jar["b"];
+	assert(val != nullptr && strcmp(val, "1+") == 0);
+	val = jar["d"];
+	assert(val != nullptr && (strcmp(val, "1") == 0 || strcmp(val, "123") == 0 || strcmp(val, "1\"2\"3") == 0));
+	val = jar["e"];
+	assert(val != nullptr && strcmp(val, "\x40+\x41") == 0);
+	val = jar["f"];
+	assert(val != nullptr && strstr(val, "space") != nullptr);
+	val = jar["g+"];
+	if (val == nullptr)
+		val = jar["g+ "];
+	assert(val != nullptr && strcmp(val, "6") == 0);
+	assert(jar["Secure"] != nullptr);
+	return EXIT_SUCCESS;
+}
+
 static int t_utf7()
 {
 	auto a = "Gelöschte Elemente", b = "Gel&APY-schte Elemente";
@@ -79,6 +104,45 @@ static int t_convert()
 	utf8_filter(largeout);
 	if (strcmp(largeout, "??? foo \xef\xbb\xbf") != 0)
 		return EXIT_FAILURE;
+
+	static constexpr char s1[] = "\x41\xe2\x98\x83\xef\xb8\x8f\x42";
+	auto sout = iconvtext(s1, "utf-8", "iso-8859-1");
+	assert(strcmp(sout.c_str(), "AB") == 0);
+	sout = iconvtext(s1, "utf-8", "iso-8859-1", ICONVTEXT_TRANSLIT);
+	assert(strcmp(sout.c_str(), "A?B") == 0);
+
+	std::string s2(4095, 'A');
+	s2 += "\xe4";
+	sout = iconvtext(s2, "iso-8859-1", "utf-8");
+	assert(sout.size() == 4097 && static_cast<unsigned char>(sout[4095]) == 0xC3 &&
+	       static_cast<unsigned char>(sout[4096]) == 0xA4U);
+
+	static constexpr char s3[] = "\x1B\x24\x42";
+	sout = iconvtext(s3, "iso-2022-jp", "utf-8");
+	assert(sout.size() == 0);
+
+	assert(string_utf8_to_mb("windows-1252", "A┌B", largeout, std::size(largeout)));
+	assert(strcmp(largeout, "AB") == 0);
+	assert(string_utf8_to_mb("windows-1252", "A\xed\xa0\x80""B", largeout, std::size(largeout)));
+	assert(strcmp(largeout, "AB") == 0);
+	assert(string_utf8_to_mb("windows-1252", "A\xff""B", largeout, std::size(largeout)));
+	assert(strcmp(largeout, "AB") == 0);
+	assert(string_mb_to_utf8("windows-1252", "A\x81""B", largeout, std::size(largeout)));
+	assert(strcmp(largeout, "AB") == 0);
+
+	assert(utf8_to_utf16le("A\xed\xa0\x80""B", largeout, std::size(largeout)) == 6);
+	assert(memcmp(largeout, "A\x00""B\x00\x00\x00", 6) == 0);
+	assert(utf16le_to_utf8("A\x00\x00\xd8""B\x00\x00\x00", 8, largeout, std::size(largeout)));
+	assert(strcmp(largeout, "AB") == 0);
+	assert(utf16le_to_utf8("A\x00""B", 3, largeout, std::size(largeout)));
+	assert(strcmp(largeout, "A") == 0);
+
+	sout = iconvtext("E", "windows-1258", "utf-8");
+	assert(sout.size() == 1);
+	assert(string_mb_to_utf8("windows-1258", "E", largeout, std::size(largeout)));
+	assert(strcmp(largeout, "E") == 0);
+	assert(string_utf8_to_mb("windows-1258", "\xc2", largeout, std::size(largeout)));
+	assert(*largeout == '\0');
 	return EXIT_SUCCESS;
 }
 
@@ -527,6 +591,8 @@ static int t_time()
 
 static int runner()
 {
+	if (t_cookie_jar() != 0)
+		return EXIT_FAILURE;
 	if (t_utf7() != 0)
 		return EXIT_FAILURE;
 	char buf[2];
