@@ -1,5 +1,6 @@
 #pragma once
 #include <cstdio>
+#include "genimport.hpp"
 
 /*
  * Override HX_getopt_help_cb because it calls exit(0), which is really bad for
@@ -22,6 +23,7 @@ namespace getfreebusy { extern int main(int, char **); }
 namespace purgesoftdel { extern int main(int, char **); }
 namespace set_locale { extern int main(int, char **); }
 namespace sync_midb { extern int main(int, char **); }
+namespace zaddrxlat { extern int main(int, char **); }
 
 namespace global {
 
@@ -33,6 +35,9 @@ extern unsigned int g_continuous_mode, g_verbose_mode, g_command_num;
 
 }
 
+extern ec_error_t select_contents_from_folder(eid_t, unsigned int flags, const RESTRICTION *, std::vector<eid_t> &);
+extern ec_error_t select_hierarchy(eid_t, unsigned int flags, std::vector<eid_t> &);
+
 template<typename... Args> int mbop_fprintf(FILE *f, Args &&...args)
 {
 	if (global::g_verbose_mode)
