@@ -1,5 +1,5 @@
 // SPDX-License-Identifier: GPL-2.0-only WITH linking exception
-// SPDX-FileCopyrightText: 2021–2025 grommunio GmbH
+// SPDX-FileCopyrightText: 2021–2026 grommunio GmbH
 // This file is part of Gromox.
 #include <algorithm>
 #include <atomic>
@@ -205,6 +205,21 @@ static TIMER *put_timer(TIMER &&ptimer)
 	return &g_exec_list.back();
 }
 
+static void do_tasks(time_t &last_cltime)
+{
+	std::unique_lock li_hold(g_list_lock);
+	auto cur_time = time(nullptr);
+	for (auto ptimer = g_exec_list.begin(); ptimer != g_exec_list.end(); ) {
+		if (ptimer->exec_time > cur_time)
+			break;
+		std::list<TIMER> stash;
+		stash.splice(stash.end(), g_exec_list, ptimer++);
+		execute_timer(&stash.front());
+	}
+	if (cur_time - last_cltime > 7 * 86400)
+		save_timers(last_cltime, cur_time);
+}
+
 int main(int argc, char **argv)
 {
 	pthread_t thr_accept_id{};
@@ -279,7 +294,6 @@ int main(int argc, char **argv)
 		}
 	}
 
-	auto cur_time = time(nullptr);
 	for (size_t i = 0; i < item_num; ++i) {
 		if (pitem[i].tid > g_last_tid)
 			g_last_tid = pitem[i].tid;
@@ -360,21 +374,8 @@ int main(int argc, char **argv)
 	printf("[system]: TIMER is now running\n");
 
 	while (!g_notify_stop) {
-		std::unique_lock li_hold(g_list_lock);
-		cur_time = time(nullptr);
-		for (auto ptimer = g_exec_list.begin(); ptimer != g_exec_list.end(); ) {
-			if (ptimer->exec_time > cur_time)
-				break;
-			std::list<TIMER> stash;
-			stash.splice(stash.end(), g_exec_list, ptimer++);
-			execute_timer(&stash.front());
-		}
-
-		if (cur_time - last_cltime > 7 * 86400)
-			save_timers(last_cltime, cur_time);
-		li_hold.unlock();
+		do_tasks(last_cltime);
 		sleep(1);
-
 	}
 	return EXIT_SUCCESS;
 }
@@ -426,7 +427,6 @@ static void *tmr_acceptwork(void *param)
 
 static void execute_timer(TIMER *ptimer)
 {
-	int len;
 	int status;
 	pid_t pid;
 	char result[1024];
@@ -441,18 +441,18 @@ static void execute_timer(TIMER *ptimer)
 			_exit(-1);
 		} else if (pid > 0) {
 			if (waitpid(pid, &status, 0) > 0) {
-				strcpy(result, WIFEXITED(status) && !WEXITSTATUS(status) ? "DONE" : "EXEC-FAILURE");
+				gx_strlcpy(result, WIFEXITED(status) && !WEXITSTATUS(status) ? "DONE" : "EXEC-FAILURE", std::size(result));
 			} else {
-				strcpy(result, "FAIL-TO-WAIT");
+				gx_strlcpy(result, "FAIL-TO-WAIT", std::size(result));
 			}
 		} else {
-			strcpy(result, "FAIL-TO-FORK");
+			gx_strlcpy(result, "FAIL-TO-FORK", std::size(result));
 		}
 	} else {
-		strcpy(result, "FORMAT-ERROR");
+		gx_strlcpy(result, "FORMAT-ERROR", std::size(result));
 	}
 
-	len = sprintf(temp_buff, "%d\t0\t%s\n", ptimer->t_id, result);
+	auto len = gx_snprintf(temp_buff, std::size(temp_buff), "%d\t0\t%s\n", ptimer->t_id, result);
 	if (HXio_fullwrite(g_list_fd, temp_buff, len) < 0)
 		fprintf(stderr, "write to timerlist: %s\n", strerror(errno));
 }
@@ -461,7 +461,6 @@ enum { X_STOP, X_LOOP };
 
 static int tmr_thrwork_1()
 {
-	int temp_len;
 	char *pspace, temp_line[1024];
 	
 	std::unique_lock co_hold(g_connection_lock);
@@ -492,7 +491,7 @@ static int tmr_thrwork_1()
 			for (auto pos = g_exec_list.begin(); pos != g_exec_list.end(); ++pos) {
 				auto ptimer = &*pos;
 				if (t_id == ptimer->t_id) {
-					temp_len = sprintf(temp_line, "%d\t0\tCANCEL\n",
+					auto temp_len = gx_snprintf(temp_line, std::size(temp_line), "%d\t0\tCANCEL\n",
 								ptimer->t_id);
 					g_exec_list.erase(pos);
 					removed_timer = true;
@@ -530,7 +529,7 @@ static int tmr_thrwork_1()
 			std::unique_lock li_hold(g_list_lock);
 			auto ptimer = put_timer(std::move(tmr));
 
-			temp_len = sprintf(temp_line, "%d\t%lld\t", ptimer->t_id,
+			auto temp_len = gx_snprintf(temp_line, std::size(temp_line), "%d\t%lld\t", ptimer->t_id,
 			           static_cast<long long>(ptimer->exec_time));
 			encode_line(ptimer->command.c_str(), temp_line + temp_len);
 			temp_len = strlen(temp_line);
@@ -538,7 +537,7 @@ static int tmr_thrwork_1()
 			if (HXio_fullwrite(g_list_fd, temp_line, temp_len) < 0)
 				fprintf(stderr, "write to timerlist: %s\n", strerror(errno));
 			li_hold.unlock();
-			temp_len = sprintf(temp_line, "TRUE %d\r\n", ptimer->t_id);
+			temp_len = gx_snprintf(temp_line, std::size(temp_line), "TRUE %d\r\n", ptimer->t_id);
 			pconnection->sk_write(temp_line, temp_len);
 		} else if (0 == strcasecmp(pconnection->line, "QUIT")) {
 			pconnection->sk_write("BYE\r\n");
